/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include "resource.h"

#include <commctrl.h>
#include <commdlg.h> 

#include "oEvent.h"
#include "xmlparser.h"
#include "gdioutput.h"
#include "csvparser.h"
#include "SportIdent.h"
#include "oListInfo.h"
#include "TabList.h"
#include "TabRunner.h"
#include "TabTeam.h"
#include "meos_util.h"
#include <cassert>

TabList::TabList(oEvent *poe):TabBase(poe)
{
  assert(!FixedTabs::listTab);
  FixedTabs::listTab=this;
  currentListType=EStdNone;
}

TabList::~TabList(void)
{
  FixedTabs::classTab=0;
}

int ListsCB(gdioutput *gdi, int type, void *data);

int ListsEventCB(gdioutput *gdi, int type, void *data)
{
	if(type!=GUI_EVENT)
		return -1;

  FixedTabs::listTab->rebuildList(*gdi);

	return 0;
}

void TabList::rebuildList(gdioutput &gdi)
{
 if (!SelectedList.empty()) {
    ButtonInfo bi;
    bi.id=SelectedList;
    ListsCB(&gdi, GUI_BUTTON, &bi);
  }
}

int openRunnerTeamCB(gdioutput *gdi, int type, void *data)
{
	if (type==GUI_LINK && data) {
    oBase *b = static_cast<oBase*>((static_cast<TextInfo*>(data))->getExtra());

    oTeam *t = dynamic_cast<oTeam*>(b);
    if (t!=0 && gdi->canClear())
      FixedTabs::teamTab->loadPage(*gdi, t->getId());
    oRunner *r = dynamic_cast<oRunner*>(b);
    if (r!=0 && gdi->canClear())
      FixedTabs::runnerTab->loadPage(*gdi, r->getId());
 	}
	return 0;
}

int NoStartRunnerCB(gdioutput *gdi, int type, void *data)
{
	if(type==GUI_LINK)
	{
		TextInfo *ti=(TextInfo *)data;
		int id=atoi(ti->id.c_str());

		//LoadRunnerPage();
		//gdi->MakeEvent("LoadRunner", id);
		pRunner p=FixedTabs::listTab->getEvent()->getRunner(id, 0);
		if(p){
			p->setStatus(StatusDNS);
			p->syncronize();
			ti->CallBack=0;
			ti->Highlight=false;
			ti->Active=false;
			ti->color=RGB(255,0,0);
			gdi->setText(ti->id, "Ej start", true);
		}
		
	}
	return 0;
}

int ListsCB(gdioutput *gdi, int type, void *data)
{
  if (!FixedTabs::listTab) 
		throw std::exception("tabList undefined.");

  return FixedTabs::listTab->listCB(*gdi, type, data);
}

void TabList::generateList(gdioutput &gdi)
{
  gdi.clearPage(false);
  oe->generateList(gdi, true, currentList);
  gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", ListsCB, "", "", true);
  gdi.addButton(gdi.getWidth()+20, 18+gdi.getButtonHeight(), "Print", "Skriv ut...", ListsCB, "", "Skriv ut listan.", true);
  gdi.addButton(gdi.getWidth()+20, 21+2*gdi.getButtonHeight(), "HTML", "Webb...", ListsCB, "", "Spara fr webben.", true);
  gdi.registerEvent("DataUpdate", ListsEventCB);
  gdi.setData("DataSync", 1);
  gdi.registerEvent("GeneralList", ListsCB);
  gdi.setOnClearCb(ListsCB);
  SelectedList="GeneralList";
}

int TabList::listCB(gdioutput &gdi, int type, void *data)
{
	int index;

	if (type==GUI_BUTTON || type==GUI_EVENT) {
		BaseInfo bi=*(BaseInfo *)data;

		if (bi.id=="Cancel")	{
			SelectedList="";
      currentListType=EStdNone;
			loadPage(gdi);
		}
		else if (bi.id=="Print")	{
			gdi.print(oe);			
		}
    else if (bi.id=="HTML")	{
      string file=gdi.browseForSave("Webbdokument*.html;*.htm", "html", index);

      if(!file.empty()) {
        gdi.writeTableHTML(file, oe->getName());
        gdi.openDoc(file.c_str());
      }
		}
    else if (bi.id=="SelectAll") {
      set<int> lst;
      lst.insert(-1);
      gdi.setSelection("ListSelection", lst);
    }
    else if (bi.id=="SelectNone") {
      set<int> lst;
      gdi.setSelection("ListSelection", lst);
    }
    else if (bi.id=="Generate") {
      ListBoxInfo lbi;
      if (gdi.getSelectedItem("ListType", &lbi)) {
        currentListType=EStdListType(lbi.data);        
      }
      else if(gdi.getSelectedItem("ResultType", &lbi)) {
        switch(lbi.data) {
          case 1:
            currentListType=EStdResultList;
            break;
          case 2:
            currentListType=EStdPatrolResultList;
            break;
          case 3:
            currentListType=EStdTeamResultListAll;
            break;
          case 4:
            currentListType=EStdTeamResultList;
            break;
          case 5:
            currentListType=EStdTeamResultListLeg;
            break;
          default: 
            return 0;
        }

      }
      else return 0;

      oListParam par;
      gdi.getSelectedItem("ResultSpecial", &lbi);
      par.useControlIdResult = lbi.data;

      gdi.getSelectedItem("LegNumber", &lbi);
      par.legNumber = lbi.data == 1000 ? -1 : lbi.data;

      gdi.getSelection("ListSelection", par.selection);
      par.filterMaxPer = gdi.getTextNo("ClassLimit");
      par.pageBreak = gdi.isChecked("PageBreak");
      par.listCode = (EStdListType)currentListType;
      par.showInterTimes = gdi.isChecked("ShowInterTimes");
      
      oe->generateListInfo(par, gdi.getLineHeight(), currentList);

      generateList(gdi);
      gdi.refresh();
    }
    else if (bi.id=="GeneralList") {
      if (SelectedList=="GeneralList") {
        int oX=gdi.GetOffsetX();
        int oY=gdi.GetOffsetY();
        generateList(gdi);
        gdi.setOffset(oX, oY);
      }
      else
        loadGeneralList(gdi);
    }
    else if (bi.id=="ResultIndividual") {
      oListParam par;
      par.listCode = EStdResultList;
      par.showInterTimes = true;
      par.legNumber = -1;
      oe->generateListInfo(par, gdi.getLineHeight(), currentList);
      generateList(gdi);
      gdi.refresh();
    }
    else if (bi.id=="StartIndividual") {
      oListParam par;
      par.listCode = EStdStartList;
      par.legNumber = -1;
      oe->generateListInfo(par,  gdi.getLineHeight(), currentList);
      currentList.setCallback(openRunnerTeamCB);
      generateList(gdi);
      gdi.refresh();
    }
		else if (bi.id=="PreReport") {
			SelectedList=bi.id;
			gdi.clearPage(false);
			oe->generatePreReport(gdi);

			gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", ListsCB, "", "", true);
			gdi.addButton(gdi.getWidth()+20, 45, "Print", "Skriv ut...", ListsCB, "", "Skriv ut listan.", true);
      gdi.addButton(gdi.getWidth()+20, 145, "HTML", "Webb...",  ListsCB, "", "Spara fr webben.", true);
      gdi.refresh();
		}
		else if (bi.id=="InForestList") {
			SelectedList=bi.id;
			gdi.clearPage(false);

			gdi.registerEvent("DataUpdate", ListsEventCB);
			gdi.setData("DataSync", 1);
			gdi.registerEvent(bi.id, ListsCB);

			oe->generateInForestList(gdi, 0, NoStartRunnerCB);
			gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", ListsCB, "", "", true);
			gdi.addButton(gdi.getWidth()+20, 45, "Print", "Skriv ut...", ListsCB, "", "Skriv ut listan.", true);
      gdi.addButton(gdi.getWidth()+20, 145, "HTML", "Webb...", ListsCB, "", "Spara fr webben.", true);
      gdi.refresh();
		}
		else if(bi.id=="TeamStartList")	{
      oListParam par;
      par.listCode = EStdTeamStartList;
      oe->generateListInfo(par,  gdi.getLineHeight(), currentList);
      currentList.setCallback(openRunnerTeamCB);
      generateList(gdi);
      gdi.refresh();
		}
		else if(bi.id=="TeamResults")	{
      oListParam par;
      par.listCode = EStdTeamResultListAll;
      oe->generateListInfo(par,  gdi.getLineHeight(), currentList);
      generateList(gdi);
      gdi.refresh();
		}
		else if(bi.id=="MinuteStartList"){
			SelectedList=bi.id;
			gdi.clearPage(false);

			gdi.registerEvent("DataUpdate", ListsEventCB);
			gdi.setData("DataSync", 1);
			gdi.registerEvent(bi.id, ListsCB);
	
      oe->generateMinuteStartlist(gdi);
			gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", ListsCB, "", "", true);
			gdi.addButton(gdi.getWidth()+20, 45, "MinuteStartList_HTML", "Webb...", ListsCB, "","", true);			
			gdi.addButton(gdi.getWidth()+20, 75, "Print", "Skriv ut...", ListsCB, "", "Skriv ut listan.", true);			
      gdi.addButton(gdi.getWidth()+20, 145, "HTML", "Webb...", ListsCB, "", "Spara fr webben.", true);
      gdi.refresh();
		}
		else if(bi.id=="ResultList") {
      settingsResultList(gdi);
		}
		else if(bi.id=="ResultListFT") {
			SelectedList=bi.id;
			gdi.clearPage(false);

			gdi.registerEvent("DataUpdate", ListsEventCB);
			gdi.setData("DataSync", 1);
			gdi.registerEvent(bi.id, ListsCB);

			oe->generateResultlistFinishTime(gdi, false,  0);

			gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", ListsCB, "", "", true);
			gdi.addButton(gdi.getWidth()+20, 45, "ResultListFT_HTML", "Webb...", ListsCB, "","", true);
			gdi.addButton(gdi.getWidth()+20, 75, "Print", "Skriv ut...", ListsCB, "", "Skriv ut listan.", true);
      gdi.addButton(gdi.getWidth()+20, 145, "HTML", "Webb...", ListsCB, "", "Spara fr webben.", true);
      gdi.refresh();
		}
		else if (bi.id=="ResultListCFT")	{
			SelectedList=bi.id;
			gdi.clearPage(false);

			gdi.registerEvent("DataUpdate", ListsEventCB);
			gdi.setData("DataSync", 1);
			gdi.registerEvent(bi.id, ListsCB);

			oe->generateResultlistFinishTime(gdi, true,  0);
			gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", ListsCB, "", "", true);
			gdi.addButton(gdi.getWidth()+20, 45, "ResultListCFT_HTML", "Webb...", ListsCB, "","", true);			
			gdi.addButton(gdi.getWidth()+20, 75, "ResultListFT_ONATT", "Dataexport...", ListsCB, 
				"","Exportformat fr OK Linns onsdagsnatt", true);			
			gdi.addButton(gdi.getWidth()+105, 45, "Print", "Skriv ut...", ListsCB, "", "Skriv ut listan.", true);
      gdi.refresh();
		}
		else if (bi.id=="ResultListFT_ONATT") {
			string file=gdi.browseForSave("Webbdokument*.txt;*.csv", "txt", index);
      throw std::exception("Not implemented");
		}
		else if (bi.id=="ResultWinSplits") {
			string file=gdi.browseForSave("IOF Strcktider XML*.xml", "xml", index);

			if(!file.empty())
				oe->exportIOFSplits(file.c_str());
		}
  }
  else if (type==GUI_LISTBOX) {
		ListBoxInfo lbi=*(ListBoxInfo *)data;

		if(lbi.id=="ListType"){
      EStdListType type = EStdListType(lbi.data);
      oListInfo li;
      oe->getListType(type, li);
			
      if (li.supportClasses) {
        gdi.enableInput("ListSelection");
        oe->fillClasses(gdi, "ListSelection", oEvent::extraNone, oEvent::filterNone);
        set<int> lst;
        lst.insert(-1);
        gdi.setSelection("ListSelection", lst);
      }
      else {
        gdi.clearList("ListSelection");
        gdi.disableInput("ListSelection");
      }

      if (li.supportLegs) {
        gdi.enableInput("LegNumber");
        oe->fillLegNumbers(gdi, "LegNumber");
      }
      else {
        gdi.disableInput("LegNumber");
        gdi.clearList("LegNumber");
      }
		}
    else if(lbi.id=="ResultType"){

      if(lbi.data==1) {
        gdi.enableInput("ResultSpecial");
        oe->fillControls(gdi, "ResultSpecial", 1);
        gdi.addItem("ResultSpecial", lang.tl("Ml"), 0);
        gdi.selectItemByData("ResultSpecial", 0);
      }
      else {
        gdi.clearList("ResultsSpecial");
        gdi.disableInput("ResultSpecial");
      }

      gdi.enableInput("LegNumber");
      oe->fillLegNumbers(gdi, "LegNumber");

      oe->fillClasses(gdi, "ListSelection", oEvent::extraNone, oEvent::filterNone);
      set<int> lst;
      lst.insert(-1);
      gdi.setSelection("ListSelection", lst);
      gdi.enableInput("Generate");
		}
	}
  else if (type==GUI_CLEAR) {
    offsetY=gdi.GetOffsetY();
    offsetX=gdi.GetOffsetX();
    return true;
  }

	return 0;
}

void TabList::loadGeneralList(gdioutput &gdi)
{
  gdi.fillDown();
  gdi.clearPage(false);
  gdi.addString("", boldLarge, "Skapa generell lista");

  gdi.addSelection("ListType", 200, 300, ListsCB, "Lista");
  oe->fillListTypes(gdi, "ListType", 0);
  
  gdi.pushY();  
  gdi.fillRight();
  gdi.addListBox("ListSelection", 200, 300, 0, "Urval", "", true);
  gdi.popY();
  gdi.fillDown();
  gdi.addCheckbox("PageBreak", "Sidbrytning mellan klasser", 0, 0);
  gdi.addCheckbox("ShowInterResults", "Visa mellantider", 0, 0);
  gdi.addInput("ClassLimit", "", 5, 0, "Begrnsa per klass");  
  gdi.dropLine();

  gdi.addSelection("LegNumber", 140, 300, ListsCB, "Strcka:");
  gdi.disableInput("LegNumber");

  gdi.dropLine();

  gdi.addButton("SelectAll", "Vlj allt", ListsCB);
  gdi.addButton("SelectNone", "Vlj inget", ListsCB);

  gdi.dropLine();
  gdi.fillRight();
  gdi.addButton("Generate", "Generera", ListsCB);
  gdi.addButton("Cancel", "Avbryt", ListsCB);

  gdi.refresh();
}

void TabList::settingsResultList(gdioutput &gdi)
{
  gdi.fillDown();
  gdi.clearPage(false);
  gdi.addString("", boldLarge, MakeDash("Resultatlista - instllningar"));

  //gdi.addSelection("ListType", 200, 300, ListsCB, "Lista");
  //oe->fillListTypes(gdi, "ListType", 0);
  
  gdi.pushY();  
  //gdi.fillDown();
  gdi.fillRight();
  gdi.addListBox("ListSelection", 200, 300, 0, "Urval", "", true);
  
  gdi.popY();
//  gdi.setCX(250);
  gdi.fillDown();

  gdi.addString("", 1, "Typ av lista");
  gdi.addListBox("ResultType", 150, 100, ListsCB);
  gdi.addItem("ResultType", lang.tl("Individuell"), 1);
  gdi.addItem("ResultType", lang.tl("Patrull"), 2);
  gdi.addItem("ResultType", lang.tl("Stafett - total"), 3);
  gdi.addItem("ResultType", lang.tl("Stafett - sammanstllning"), 4);
  gdi.addItem("ResultType", lang.tl("Stafett - strcka"), 5);

  gdi.addCheckbox("PageBreak", "Sidbrytning mellan klasser", 0, 0);
  gdi.addCheckbox("ShowInterResults", "Lista med mellantider", 0, 0, 
            "Mellantider visas fr namngivna kontroller.");
  
  gdi.fillRight();
  gdi.pushX();
  gdi.addString("", 0, "Topplista, N bsta:");
  gdi.dropLine(-0.2);
  gdi.addInput("ClassLimit", "", 5, 0);  
  gdi.popX();
  gdi.dropLine(2);
  gdi.addSelection("ResultSpecial", 140, 300, ListsCB, "Listval:");
  gdi.disableInput("ResultSpecial");
  gdi.popX();
  gdi.dropLine(3);
  gdi.addSelection("LegNumber", 140, 300, ListsCB, "Strcka:");
  gdi.disableInput("LegNumber");
  gdi.popX();

  gdi.dropLine(3);
  gdi.addButton("SelectAll", "Vlj allt", ListsCB);
  gdi.addButton("SelectNone", "Vlj inget", ListsCB);
 
  gdi.popX();
  gdi.dropLine(3);
  gdi.fillRight();
  gdi.addButton("Generate", "Skapa listan", ListsCB);
  gdi.disableInput("Generate");
  gdi.addButton("Cancel", "Avbryt", ListsCB);

  gdi.refresh();
}


bool TabList::loadPage(gdioutput &gdi)
{
  oe->checkDB();
	gdi.selectTab(tabId);

	gdi.clearPage(false);

  if (SelectedList!="") {
    ButtonInfo bi;
    bi.id=SelectedList;
    ListsCB(&gdi, GUI_BUTTON, &bi);
    //gdi.SendCtrlMessage(SelectedList);
    gdi.setOffset(offsetX, offsetY);
    return 0;
  }

  gdi.addString("", boldLarge, "Listor och sammanstllningar");

  gdi.addString("", 10, "help:30750");

  gdi.dropLine(1);
  gdi.addString("", 1, "Startlistor");
	gdi.fillRight();
	gdi.pushX();
	//gdi.addButton("StartList", "Startlista", ListsCB);
  gdi.addButton("StartIndividual", "Individuell", ListsCB);
  gdi.addButton("TeamStartList", "Stafett", ListsCB);
	gdi.addButton("MinuteStartList", "Minutstartlista", ListsCB);

  gdi.dropLine(3);
  gdi.fillDown();
  gdi.popX();
  gdi.addString("", 1, "Resultatlistor");
  gdi.fillRight();
  gdi.addButton("ResultIndividual", "Individuell", ListsCB);
  gdi.addButton("TeamResults", "Stafett", ListsCB);
	
	//gdi.addButton("ResultListFT", "Mlgng, gemensam", ListsCB, "Resultatlista sorterad efter mltid, alla klasser gemensamt.");
	//gdi.addButton("ResultListCFT", "Mlgng, klassvis", ListsCB, "Resultatlista sorterad efter mltid (frst i ml), klassvis.");

	gdi.addButton("ResultList", "Avancerat...", ListsCB);

	gdi.fillDown();	
  gdi.popX();
  gdi.dropLine(3);
	//gdi.addButton("ResultWinSplits", "Resultat i WinSplits", ListsCB);

	
  gdi.addString("", 1, "Rapporter");

	gdi.fillRight();
	gdi.pushX();

	gdi.addButton("InForestList", "Kvar-i-skogen", ListsCB, "Lista ver lpare i skogen och ej startande lpare.");
	gdi.addButton("PreReport", "Kr kontroll infr tvlingen...", ListsCB);

	gdi.popX();
	gdi.fillDown();
	gdi.dropLine(4);

  gdi.addButton("GeneralList", "Alla listor...", ListsCB);

	//gdi.registerEvent("DataUpdate", ListsEventCB);	
	gdi.refresh();

  gdi.setOnClearCb(ListsCB);
    
  offsetY=0;
  offsetX=0;
	gdi.refresh();
  return true;
}
