/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include "resource.h"

#include <commctrl.h>
#include <commdlg.h> 

#include "oEvent.h"
#include "xmlparser.h"
#include "gdioutput.h"
#include "csvparser.h"
#include "SportIdent.h"

#include "TabRunner.h"
#include "Table.h"
#include <cassert>

TabRunner::TabRunner(oEvent *poe):TabBase(poe)
{
  lastRace=0;
  tableMode=false;
  assert(!FixedTabs::runnerTab);
  FixedTabs::runnerTab=this;
  runnerId=0;
}

TabRunner::~TabRunner(void)
{
  FixedTabs::runnerTab=0;
}

void disablePunchCourse(gdioutput &gdi);

bool TabRunner::loadPage(gdioutput &gdi, int runnerId)
{
  oe->checkDB();
  assert(this);
  loadPage(gdi);
  pRunner r=oe->getRunner(runnerId, 0);
  if(r){
	  selectRunner(gdi, r);
	  return true;
  }
  return false;
}

void TabRunner::selectRunner(gdioutput &gdi, pRunner r)
{
  if(!r) {
    runnerId=0;
	  gdi.setText("Name", "");
    gdi.selectItemByData("RCourse", 0);

    //Don't clear club and class

    gdi.setText("CardNo", "");
		gdi.setText("Start", "-");
		gdi.setText("Finish", "-");

		gdi.setText("Time", "-");
		gdi.selectItemByData("Status", 0);

		gdi.clearList("Punches");
    gdi.clearList("Course");
	
		gdi.selectItemByData("Runners", -1);
		gdi.setInputFocus("Name", true);
    gdi.clearList("MultiR");
    gdi.disableInput("MultiR");
    gdi.enableEditControls(false);
 		gdi.disableInput("Remove");
		gdi.disableInput("Save");
    disablePunchCourse(gdi);
    return;
  } 

  gdi.enableEditControls(true);
 	disablePunchCourse(gdi);

  pRunner parent=r->getMultiRunner(0);

	r->syncronize();
	r->apply(false);

  if(parent!=r) {
	  parent->syncronize();
	  parent->apply(false);
  }

  gdi.selectItemByData("Runners", parent->getId());

  runnerId=r->getId();

	gdi.setText("Name", r->getName());
	gdi.setText("Club", r->getClub());

	oe->fillClasses(gdi, "RClass", oEvent::extraNone, oEvent::filterNone);
	gdi.addItem("RClass", lang.tl("Ny klass"), 0);
	gdi.selectItemByData("RClass", r->getClassId());

  int numMulti=parent->getNumMulti();
  if(numMulti==0) {
    gdi.clearList("MultiR");
    gdi.disableInput("MultiR");
    lastRace=0;
  }
  else {
    char bf[32];
    gdi.clearList("MultiR");
    gdi.enableInput("MultiR");

    for (int k=0;k<numMulti+1;k++) {
      sprintf_s(bf, lang.tl("Lopp %d").c_str(), k+1);
      gdi.addItem("MultiR", bf, k);
    }
    gdi.selectItemByData("MultiR", r->getRaceNo());
  }

	oe->fillCourses(gdi, "RCourse", true);
	gdi.addItem("RCourse", lang.tl("[Klassens bana]"), 0);
	gdi.selectItemByData("RCourse", r->getCourseId());

	gdi.setText("CardNo", parent->getCardNo());
  gdi.check("RentCard", parent->getDI().getInt("CardFee")>0);
  gdi.setText("Fee", parent->getDI().getInt("Fee"));	

  if(parent!=r) {
    gdi.disableInput("CardNo");
    gdi.disableInput("RentCard");
    gdi.disableInput("Fee");
  }
  else {
    gdi.enableInput("CardNo");
    gdi.enableInput("RentCard");
    gdi.enableInput("Fee");
  }
  
	gdi.enableInput("Remove");
	gdi.enableInput("Save");

  gdi.setText("Start", r->getStartTimeS());
	gdi.setText("Finish", r->getFinishTimeS());

	gdi.setText("Time", r->getRunningTimeS());

	//gdi.setText("Status", r->GetStatusS());
	gdi.selectItemByData("Status", r->getStatus());

	pCard pc=r->getCard();

	pCourse pcourse=r->getCourse();

	if(pc)	{
		gdi.setTabStops("Punches", 70);
		pc->fillPunches(gdi, "Punches", pcourse);
	}
	else gdi.clearList("Punches");

	if(pcourse)	{
    pcourse->syncronize();
		gdi.setTabStops("Course", 50);			
		pcourse->fillCourse(gdi, "Course");
	}
	else
		gdi.clearList("Course");

}

int RunnerCB(gdioutput *gdi, int type, void *data)
{
	if (!FixedTabs::runnerTab) 
		throw std::exception("tabRunner undefined.");

  return FixedTabs::runnerTab->runnerCB(*gdi, type, data);
}

int PunchesCB(gdioutput *gdi, int type, void *data)
{
 	if (!FixedTabs::runnerTab) 
		throw std::exception("tabRunner undefined.");

  return FixedTabs::runnerTab->punchesCB(*gdi, type, data);
}

pRunner TabRunner::save(gdioutput &gdi, int runnerId)
{
  bool create=(runnerId==0);

  if (create)
    return 0;

	string name=gdi.getText("Name");

	if(name.empty()) 
    throw std::exception("Alla deltagare mste ha ett namn.");
	
	pRunner r;
	if(runnerId==0) 
		r=oe->addRunner("", 0,0, true);
	else 
    r=oe->getRunner(runnerId, 0);
	
	if (r) {
    runnerId=r->getId();

	  r->setName(name);				
	  r->setCardNo(gdi.getTextNo("CardNo"), true);
    if(gdi.isChecked("RentCard"))
      r->getDI().setInt("CardFee", oe->GetEventDI().getInt("CardFee"));
    else
      r->getDI().setInt("CardFee", 0);
    
    if (oe->useEconomy())
      r->getDI().setInt("Fee", gdi.getTextNo("Fee"));
    
	  r->setStartTimeS(gdi.getText("Start"));
	  r->setFinishTimeS(gdi.getText("Finish"));

	  ListBoxInfo lbi;
	  gdi.getSelectedItem("Club", &lbi);
  	
    if (!lbi.text.empty()) {
		  pClub pc=oe->getClub(lbi.text);
      if(!pc)
        pc=oe->addClub(lbi.text);
		  pc->syncronize();										
	  }

	  r->setClub(lbi.text);
	  oe->fillClubs(gdi, "Club");
	  gdi.setText("Club", lbi.text);

	  gdi.getSelectedItem("Status", &lbi);

	  r->setStatus((RunnerStatus)lbi.data);

	  gdi.getSelectedItem("RClass", &lbi);

	  if (lbi.data<=0) {
      if (gdi.ask("Vill du skapa en ny klass?")) {
		    pClass pc=oe->addClass(oe->getAutoClassName());
		    r->setClassId(pc->getId());
      }
	  }
	  else r->setClassId(lbi.data);

	  gdi.getSelectedItem("RCourse", &lbi);
	  r->setCourseId(lbi.data);

	  vector<int> mp;
	  r->evaluateCard(mp, 0, true);

	  r->syncronize();
  }
  else
    runnerId=0;

  oe->fillRunners(gdi, "Runners");

	if (create) 
    selectRunner(gdi, 0);
  else 
    selectRunner(gdi, r);

  return r;
}

int TabRunner::runnerCB(gdioutput &gdi, int type, void *data)
{
	if(type==GUI_BUTTON){
		ButtonInfo bi=*(ButtonInfo *)data;

    if (bi.id=="Search") {
      ListBoxInfo lbi;
      gdi.getSelectedItem("Runners", &lbi);
      oe->fillRunners(gdi, "Runners", !formMode, !formMode);
      pRunner r=oe->findRunner(gdi.getText("SearchText"), lbi.data);

      if(r) {
        if (formMode)
          selectRunner(gdi, r);
        gdi.selectItemByData("Runners", r->getId());
      }
      else
        gdi.alert("Lparen hittades inte");
    }
    else if(bi.id=="Pair") {
      ListBoxInfo lbi;
      pRunner r=0;
      if (gdi.getSelectedItem("Runners", &lbi) &&
                      (r=oe->getRunner(lbi.data, 0))!=0) {
        int newCardId=static_cast<pCard>(bi.getExtra())->getId();
        int oldCardId=r->setCard(newCardId);
        gdi.restore("CardTable");
        Table &t=gdi.getTable();
        t.reloadRow(newCardId);
        if(oldCardId)
          t.reloadRow(oldCardId);
        gdi.enableTables();
        gdi.refresh();
      }      
    }
    else if(bi.id=="Unpair") {
      ListBoxInfo lbi;
      pCard c=static_cast<pCard>(bi.getExtra());

      if(c->getOwner())
        c->getOwner()->setCard(0);

      gdi.restore("CardTable");
      Table &t=gdi.getTable();
      t.reloadRow(c->getId());
      gdi.enableTables();
      gdi.refresh();
    }
    else if (bi.id=="SwitchMode") {
      if (!tableMode && runnerId>0)
        save(gdi, runnerId);

      tableMode=!tableMode;
      loadPage(gdi);
    }
    else if (bi.id=="FormMode") {
      tableMode=false;
      gdi.enableTables();
      loadPage(gdi);
    }
    else if (bi.id=="InForest") {
      if (!tableMode && runnerId>0)
        save(gdi, runnerId);

      showInForestList(gdi);
    }
    else if (bi.id=="CancelInForest") {
      clearInForestData();
      loadPage(gdi);
    }
    else if (bi.id=="SetDNS") {
      for (size_t k=0; k<unknown.size(); k++)
        if (unknown[k]->getStatus()==StatusUnknown) 
          unknown[k]->setStatus(StatusDNS);

      //Reevaluate and syncronize all
      oe->reEvaluateAll(true);
      clearInForestData();
      showInForestList(gdi);
    }
    else if (bi.id=="SetUnknown") {
      for (size_t k=0; k<known_dns.size(); k++)
        if (known_dns[k]->getStatus()==StatusDNS) 
          known_dns[k]->setStatus(StatusUnknown);

      //Reevaluate and syncronize all
      oe->reEvaluateAll(true);
      clearInForestData();
      showInForestList(gdi);
    }
    else if (bi.id=="Cancel") {
      gdi.restore("CardTable");
      gdi.enableTables();
      gdi.refresh();
    }
    else if (bi.id=="SplitPrint") {
			if(!runnerId)
				return 0;
			pRunner r=oe->getRunner(runnerId, 0);
			if(!r) return 0;

      gdioutput gdiprint(splitPrinter);
      r->printSplits(gdiprint);
      gdiprint.print(oe, 0, false, true);
      gdiprint.getPrinterSettings(splitPrinter);
    }
    else if (bi.id=="Cards") {
      if (!tableMode && runnerId>0)
        save(gdi, runnerId);

      gdi.clearPage(false);
      gdi.dropLine(0.5);
      gdi.addString("", boldLarge, "Hantera lparbrickor");
      gdi.addString("", 10, "help:14343");
      gdi.addButton(2, 2, "FormMode", "terg...", RunnerCB);
      cardModeStartY=gdi.getCY();      
      Table *t=oe->getCardsTB();
      gdi.addTable(t,gdi.getCX(),gdi.getCY()+15);
      gdi.registerEvent("CellAction", RunnerCB);
      gdi.refresh();
      formMode = false;
    }
		else if(bi.id=="Save") {
		  save(gdi, runnerId);
     	return true;
		}
		else if(bi.id=="Add") {
      if (runnerId>0)
        save(gdi, runnerId);

      ListBoxInfo lbi;
      gdi.getSelectedItem("RClass", &lbi);

      pRunner r = oe->addRunner(oe->getAutoRunnerName(), 0,0, false);
      if (lbi.data>0)
        r->setClassId(lbi.data);
      else  
        r->setClassId(oe->getFirstClassId());

      oe->fillRunners(gdi, "Runners");
      selectRunner(gdi, r);
      gdi.setInputFocus("Name", true);
		}
		else if(bi.id=="Remove") {
			if(!runnerId)
				return 0;
		
			if(gdi.ask("Vill du verkligen ta bort lparen?")) {
				if(oe->isRunnerUsed(runnerId))
					gdi.alert("Lparen ingr i ett lag och kan inte tas bort.");
				else {
					oe->removeRunner(runnerId);
					oe->fillRunners(gdi, "Runners");
          selectRunner(gdi, 0);
				}
			}
		}
		else if(bi.id=="RemoveC")	{
			ListBoxInfo lbi;
			gdi.getSelectedItem("Punches", &lbi);

			DWORD rid=runnerId;
			if(!rid)
				return 0;

			pRunner r=oe->getRunner(rid, 0);

			if(!r) return 0;

			pCard card=r->getCard();

			if(!card) return 0;

			card->deletePunch(lbi.data);

			//Update runner
			vector<int> mp;
			r->evaluateCard(mp);			

      card->fillPunches(gdi, "Punches", r->getCourse());

      gdi.setText("Time", r->getRunningTimeS());
			gdi.selectItemByData("Status", r->getStatus());
		}
		else if(bi.id=="Check")	{
		}
	}
	else if (type==GUI_LISTBOX) {
		ListBoxInfo bi=*(ListBoxInfo *)data;

		if (bi.id=="Runners") {
      if (gdi.isInputChanged(""))
        save(gdi, runnerId);

			pRunner r=oe->getRunner(bi.data, 0);

      if (r==0)
        throw std::exception("Internal error runner index");

      if(lastRace<=r->getNumMulti())
			  r=r->getMultiRunner(lastRace);

      selectRunner(gdi, r);
		}
		else if (bi.id=="RClass") {
			gdi.selectItemByData("RCourse", 0);

			if (bi.data==0) {
				gdi.clearList("Course");				
			}
			else {
				pClass Class=oe->getClass(bi.data);

				if(Class && Class->getCourse())
					Class->getCourse()->fillCourse(gdi, "Course");
        else
          gdi.clearList("Course");
			}
		}
		else if (bi.id=="RCourse") {
			if (bi.data==0) {
				gdi.clearList("Course");
        pRunner r=oe->getRunner(runnerId, 0);
        if(r) {  //Fix for multi classes, course depends on runner.
          ListBoxInfo lbi;
				  gdi.getSelectedItem("RClass", &lbi);
          if (lbi.data>0) {
            r->setClassId(lbi.data);
            r->syncronize(true);
            if(r->getCourse())
              r->getCourse()->fillCourse(gdi, "Course");
          }
        }
        else {
				  ListBoxInfo lbi;
				  gdi.getSelectedItem("RClass", &lbi);
				  pClass Class=oe->getClass(lbi.data);
				  if(Class && Class->getCourse())
					  Class->getCourse()->fillCourse(gdi, "Course");       
        }
			}
			else {
				pCourse course=oe->getCourse(bi.data);

				if(course)
					course->fillCourse(gdi, "Course");
			}
		}
    else if (bi.id=="MultiR") {
      pRunner r=oe->getRunner(runnerId, 0);
      lastRace=bi.data;
      if(r) 
        selectRunner(gdi, r->getMultiRunner(bi.data));
    }
	}
	else if (type==GUI_EVENT) {
		EventInfo ei=*(EventInfo *)data;

		if(ei.id=="LoadRunner") {
			pRunner r=oe->getRunner(ei.data, 0);
			if (r) {
        selectRunner(gdi, r);
				gdi.selectItemByData("Runners", r->getId());
			}
		}
    else if(ei.id=="CellAction") {
      oBase *b=static_cast<oBase *>(ei.getExtra());
      cellAction(gdi, ei.data, b);
    }
	}
  else if(type==GUI_CLEAR) {
    if (runnerId>0)
      save(gdi, runnerId);

    return true;
  }

	return 0;
}

int SportIdentCB(gdioutput *gdi, int type, void *data);

void TabRunner::clearInForestData() 
{
  unknown_dns.clear();
  known_dns.clear();
  known.clear();
  unknown.clear();
}

void TabRunner::showInForestList(gdioutput &gdi)
{
  gdi.clearPage(true);

  gdi.addString("", boldLarge, "Hantera kvar-i-skogen");
  gdi.dropLine();
  gdi.setRestorePoint("Help");
  gdi.addString("", 10, "help:425188");
  gdi.dropLine();
  gdi.pushX();
  gdi.fillRight();
  gdi.addButton("Import", "Importera stmplingar (SportIdent)...", SportIdentCB);
  gdi.fillDown();
  gdi.addButton("CancelInForest", "Avbryt", RunnerCB);
  gdi.popX();
  gdi.addButton("SetDNS", "Stt oknda lpare utan registrering till <Ej Start>", RunnerCB);
  gdi.addButton("SetUnknown", "terstll lpare <Ej Start> med registrering till <Status Oknd>", RunnerCB);
  gdi.dropLine();
  
  clearInForestData();
  oe->analyseDNS(unknown_dns, known_dns, known, unknown);

  if (!unknown.empty()) {
    gdi.dropLine();
    gdi.dropLine(0.5);
    gdi.addString("", 1, "Lpare, Status Oknd, som saknar registrering");
    listRunners(gdi, unknown);
  }
  else {
    gdi.disableInput("SetDNS");
  }

  if (!known.empty()) {
    gdi.dropLine();
    gdi.addString("", 1, "Lpare, Status Oknd, med registrering (kvar-i-skogen)");
    gdi.dropLine(0.5);
    listRunners(gdi, known);
  }

  if (!known_dns.empty()) {
    gdi.dropLine();
    gdi.addString("", 1, "Lpare, Ej Start, med registrering (kvar-i-skogen!?)");
    gdi.dropLine(0.5);
    listRunners(gdi, known_dns);
  }
  else
    gdi.disableInput("SetUnknown");
}

void TabRunner::listRunners(gdioutput &gdi, const vector<pRunner> &r) const
{
  char bf[64];
  int yp = gdi.getCY();
  int xp = gdi.getCX();

  for (size_t k=0; k<r.size(); k++) {
    sprintf_s(bf, "%d.", k+1);
    gdi.addStringUT(yp, xp, 0, bf);
    gdi.addStringUT(yp, xp+40, 0, r[k]->getNameAndRace(), 190);
    gdi.addStringUT(yp, xp+200, 0, r[k]->getClass(), 140);
    gdi.addStringUT(yp, xp+350, 0, r[k]->getClub(), 190);
    sprintf_s(bf, "(%d)", r[k]->getCardNo());
    if (r[k]->getCardNo()>0)
      gdi.addStringUT(yp, xp+550, 0, bf, 240);
    yp += gdi.getLineHeight();
  }
}

void TabRunner::cellAction(gdioutput &gdi, DWORD id, oBase *obj)
{
  if (id==TID_RUNNER || id==TID_CARD) {
    gdi.disableTables();
    pCard c=dynamic_cast<pCard>(obj);
    if (c) {
      gdi.setRestorePoint("CardTable");
      char bf[256];
      int orgx=gdi.getCX();
      gdi.dropLine(1);
      gdi.setCY(cardModeStartY);
      gdi.scrollTo(orgx, cardModeStartY);
      sprintf_s(bf, lang.tl("Para ihop bricka %s med deltagare").c_str(), c->getCardNo().c_str());
      gdi.addString("", 1, bf);
      sprintf_s(bf, lang.tl("Nuvarande innehavare: %s").c_str(), c->getOwner() ? c->getOwner()->getName().c_str() : "-");
      gdi.addString("", 0, bf);
      
      gdi.dropLine(1);
      gdi.pushX();
      gdi.fillRight();
      gdi.addListBox("Card", 150, 300, 0, "Vald bricka:");
      c->fillPunches(gdi, "Card", 0);
      gdi.disableInput("Card");

      gdi.pushX();
      gdi.fillRight();
      gdi.popX();

      gdi.fillDown();
      gdi.addListBox("Runners", 350, 300, 0, "Deltagare:");
      gdi.setTabStops("Runners", 200, 300);
      oe->fillRunners(gdi, "Runners", true, true);
      if(c->getOwner())
        gdi.selectItemByData("Runners", c->getOwner()->getId());

      gdi.popX();
      gdi.fillRight();
      gdi.addInput("SearchText", "", 15);
      gdi.addButton("Search", "Sk deltagare", RunnerCB, "", "Sk p namn, bricka eller startnummer.");
     
      gdi.popX();
      gdi.dropLine(3);
      gdi.addButton("Pair", "Para ihop", RunnerCB).setExtra(c);
      gdi.addButton("Unpair", "Stt som oparad", RunnerCB).setExtra(c);
      gdi.addButton("Cancel", "Avbryt", RunnerCB);
      gdi.fillDown();
      gdi.popX();
      gdi.refresh();
    }
  }
}


void disablePunchCourseAdd(gdioutput &gdi)
{
  gdi.disableInput("AddC");
	gdi.disableInput("AddAllC");
  gdi.selectItemByData("Course", -1);
}

void disablePunchCourseChange(gdioutput &gdi)
{
	gdi.disableInput("SaveC");
	gdi.disableInput("RemoveC");
	gdi.disableInput("PTime");
	gdi.setText("PTime", "");  
  gdi.selectItemByData("Punches", -1);

}

void disablePunchCourse(gdioutput &gdi)
{
  disablePunchCourseAdd(gdi);
  disablePunchCourseChange(gdi);
}

void UpdateStatus(gdioutput &gdi, pRunner r)
{
	if(!r) return;

	gdi.setText("Start", r->getStartTimeS());
	gdi.setText("Finish", r->getFinishTimeS());
	gdi.setText("Time", r->getRunningTimeS());
	gdi.selectItemByData("Status", r->getStatus());
}

int TabRunner::punchesCB(gdioutput &gdi, int type, void *data)
{
	DWORD rid=runnerId;
	if(!rid)
		return 0;

	pRunner r=oe->getRunner(rid, 0);

	if(!r){
		gdi.alert("Deltagaren mste sparas innan stmplingar kan hanteras.");
		return 0;
	}

	
	if(type==GUI_LISTBOX){
		ListBoxInfo bi=*(ListBoxInfo *)data;

		if (bi.id=="Punches") {
			if (bi.data>=0 && bi.data<1000) {
				pCard card=r->getCard();

				if(!card) return 0;
				
				string ptime=card->getPunchTime(bi.data);

				if (ptime!="") {
					gdi.enableInput("SaveC");
					gdi.setText("PTime", ptime);
				}
				gdi.enableInput("RemoveC");
				gdi.enableInput("PTime");
			}
			else {
				gdi.disableInput("SaveC");
				gdi.disableInput("RemoveC");
				gdi.setText("PTime", "");
			}
      disablePunchCourseAdd(gdi);
		}
		else if (bi.id=="Course") {
			if (bi.data>=0) {
				pCourse pc=r->getCourse();
				
				if(!pc) return 0;

				gdi.enableInput("AddC");
				gdi.enableInput("AddAllC");
			}
			else{
				gdi.disableInput("AddC");
				gdi.disableInput("AddAllC");
			}
      disablePunchCourseChange(gdi);
		}
	}
	else if(type==GUI_BUTTON){
		ButtonInfo bi=*(ButtonInfo *)data;
		pCard card=r->getCard();

		if(!card){
			if(!gdi.ask("Ingen bricka har lsts in fr denna lpare. Vill du lgga till stmplingar manuellt?"))
				return 0;
	
			card=oe->allocateCard(r);

			card->setCardNo(r->getCardNo());
			vector<int> mp;
			r->addPunches(card, mp);
		}
	
		if(bi.id=="AddC"){
			vector<int> mp;
			r->evaluateCard(mp);
			vector<int>::iterator it=mp.begin();
			
			pCourse pc=r->getCourse();

			if(!pc) return 0;

			ListBoxInfo lbi;

			if(!gdi.getSelectedItem("Course", &lbi))
				return 0;
			
		  oControl *oc=pc->getControl(lbi.data);

			if(!oc) return 0;

			while(it!=mp.end()){
				if(oc->hasNumber(*it)){
					vector<int> nmp;
					r->evaluateCard(nmp, *it); //Add this punch					
				}
				++it;
			}
			//syncronize SQL
			card->syncronize();

			card->fillPunches(gdi, "Punches", pc);
			UpdateStatus(gdi, r);
		}
		else if(bi.id=="AddAllC"){
			vector<int> mp;
			r->evaluateCard(mp);
			vector<int>::iterator it=mp.begin();
	

			while(it!=mp.end()){
				vector<int> nmp;
        r->evaluateCard(nmp, *it); //Add this punch					
				++it;
        if(nmp.empty())
          break;
			}

			//syncronize SQL
			card->syncronize();

			card->fillPunches(gdi, "Punches", r->getCourse());
			UpdateStatus(gdi, r);
		}
		else if(bi.id=="SaveC"){						
			//int time=oe->GetRelTime();

			ListBoxInfo lbi;

			if(!gdi.getSelectedItem("Punches", &lbi))
				return 0;
			
			pCard pc=r->getCard();
			
			if(!pc) return 0;
			
			pc->setPunchTime(lbi.data, gdi.getText("PTime"));

			vector<int> mp;
			r->evaluateCard(mp);

			//syncronize SQL
			card->syncronize();
			r->syncronize();

      card->fillPunches(gdi, "Punches", r->getCourse());
			UpdateStatus(gdi, r);		
		}
	}
	return 0;
}

bool TabRunner::loadPage(gdioutput &gdi)
{
  oe->reEvaluateAll(true);

  clearInForestData();
	gdi.selectTab(tabId);
	gdi.clearPage(false);
	int basex=gdi.getCX();
  const int button_w=gdi.scaleLength(100);
  string switchMode;

  switchMode=tableMode ? "Formulrlge" : "Tabellge";

  gdi.addButton(2, 2, button_w, "SwitchMode", switchMode, RunnerCB);
  if(tableMode) {
    Table *tbl=oe->getRunnersTB();
    gdi.addTable(tbl, basex, 30);
    formMode=false;
    return true;
  }

  formMode=true;
  gdi.pushX();
  gdi.addString("", boldLarge, "Deltagare");
  gdi.fillRight();
  gdi.addInput("SearchText", "", 15).isEdit(false).ignore(true);
  gdi.dropLine(-0.2);
  gdi.addButton("Search", "Sk", RunnerCB, "", "Sk p namn, bricka eller startnummer.");
  gdi.dropLine(2);
  gdi.popX();

	gdi.fillDown();
	gdi.addListBox("Runners", 200, 350, RunnerCB).isEdit(false).ignore(true);
	oe->fillRunners(gdi, "Runners");

	gdi.newColumn();
	gdi.fillDown();

  gdi.dropLine(0.5);
	gdi.addInput("Name", "", 22, 0, "Namn:");
	gdi.addCombo("Club", 220, 300, 0, "Klubb:");
	oe->fillClubs(gdi, "Club");
  gdi.pushX();

  gdi.fillRight();
	gdi.addSelection("RClass", 150, 300, RunnerCB, "Klass:");
	
	oe->fillClasses(gdi, "RClass", oEvent::extraNone, oEvent::filterNone);	
	gdi.addItem("RClass", lang.tl("Ny klass"), 0);
	
  gdi.fillDown();
  
  if (oe->useEconomy())
    gdi.addInput("Fee", "", 6, 0, "Avgift:");
  else
    gdi.dropLine(3);

  gdi.dropLine(0.4);

  if (oe->hasMultiRunner()) {
    gdi.fillRight();
    gdi.popX();
    gdi.addString("", 0, "Vlj lopp:");
    gdi.fillDown();
    gdi.dropLine(-0.2);
    gdi.addSelection("MultiR", 160, 100, RunnerCB);
  }

  gdi.popX();

	gdi.addSelection("RCourse", 220, 300, RunnerCB, "Bana:");
	oe->fillCourses(gdi, "RCourse", true);
	gdi.addItem("RCourse", lang.tl("[Klassens bana]"), 0);

  gdi.pushX();
  gdi.fillRight();
  gdi.addInput("CardNo", "", 8, 0, "Bricka:");
  gdi.dropLine(1);
  gdi.addCheckbox("RentCard", "Hyrd", 0, false);
	
  gdi.dropLine(2);
  gdi.popX();
	
	gdi.addInput("Start", "", 6, 0, "Starttid:");
	
	gdi.fillDown();
	gdi.addInput("Finish", "", 6, 0, "Mltid:");
	
	gdi.popX();
	
	gdi.pushX();
	gdi.fillRight();

  gdi.addInput("Time", "", 6, 0, "Lptid:").isEdit(false).ignore(true);
	gdi.disableInput("Time");

	gdi.fillDown();
	gdi.addSelection("Status", 90, 160, 0, "Status:");
  oe->fillStatus(gdi, "Status");
	gdi.popX();
	gdi.selectItemByData("Status", 0);

  gdi.dropLine(2);
  gdi.fillRight();
  gdi.addButton("Save", "Spara", RunnerCB);	
  gdi.disableInput("Save");
	gdi.addButton("Remove", "Radera", RunnerCB);
	gdi.disableInput("Remove");
	gdi.addButton("Add", "Ny deltagare", RunnerCB);
  gdi.fillDown();
  gdi.newColumn();
	gdi.addListBox("Punches", 140, 300, PunchesCB, "Stmplingar:").ignore(true);
	gdi.addButton("RemoveC", "Ta bort stmpling >>", RunnerCB);

	gdi.pushX();
	gdi.fillRight();
	gdi.addInput("PTime", "", 5, 0, "Tid:");
	gdi.fillDown();
	gdi.dropLine();
	gdi.addButton("SaveC", "Spara", PunchesCB);

	gdi.popX();
  gdi.addButton("SplitPrint", "Skriv ut strcktider", RunnerCB).isEdit(true);
	gdi.pushY();

	gdi.newColumn();
  gdi.fillDown();
	gdi.addListBox("Course", 140, 300, PunchesCB, "Banmall:").ignore(true);
	gdi.addButton("AddC", "<< Lgg till stmpling", PunchesCB);
	gdi.addButton("AddAllC", "<< Lgg till alla", PunchesCB);
	
	disablePunchCourse(gdi);
	gdi.popX();
	gdi.popY();

	gdi.fillDown();
	gdi.setCX(basex+100);
	gdi.dropLine();
	gdi.addString("", 10, "help:41072");		

	gdi.registerEvent("LoadRunner", RunnerCB);
	
  gdi.setOnClearCb(RunnerCB);

  gdi.addButton(2+button_w, 2, button_w, "InForest", "Kvar-i-skogen", RunnerCB);
  gdi.addButton(2+2*button_w, 2 ,button_w, "Cards", "Hantera brickor", RunnerCB);

  selectRunner(gdi, oe->getRunner(runnerId, 0));
  gdi.refresh();
  return true;
}
