/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"
#include <vector>

#include "meos_util.h"

string getLocalTime()
{
	SYSTEMTIME st;
	GetLocalTime(&st);
	return convertSystemTime(st);
}

string getLocalTimeFileName()
{
	SYSTEMTIME st;
	GetLocalTime(&st);

  char bf[32];
	sprintf_s(bf, "%d%02d%02d_%02d%02d%02d", st.wYear, st.wMonth, st.wDay,
		st.wHour, st.wMinute, st.wSecond);

	return bf;
}


string getLocalTimeOnly()
{
	SYSTEMTIME st;
	GetLocalTime(&st);
	return convertSystemTimeOnly(st);
}

string convertSystemTime(const SYSTEMTIME &st)
{
	char bf[32];
	sprintf_s(bf, "%d-%02d-%02d %02d:%02d:%02d", st.wYear, st.wMonth, st.wDay,
		st.wHour, st.wMinute, st.wSecond);

	return bf;
}


string convertSystemTimeOnly(const SYSTEMTIME &st)
{
	char bf[32];
	sprintf_s(bf, "%02d:%02d:%02d", st.wHour, st.wMinute, st.wSecond);

	return bf;
}


string convertSystemDate(const SYSTEMTIME &st)
{
	char bf[32];
	sprintf_s(bf, "%d-%02d-%02d", st.wYear, st.wMonth, st.wYear);

	return bf;
}

//Absolute time string to absolute time int
int convertAbsoluteTimeHMS(const string &m) 
{
	if(m.length()==0)
		return -1;

	int hour=atoi(m.c_str());
	if(hour<0 || hour>23)
		return -1;

	int minute=0;
	int second=0;
	int kp=m.find_first_of(':');
	
  if (kp!=string::npos) {
		string mtext=m.substr(kp+1);
		minute=atoi(mtext.c_str());

		if(minute<0 || minute>60)
			minute=0;

		kp=mtext.find_last_of(':');

    if (kp!=string::npos) {
			second=atoi(mtext.substr(kp+1).c_str());
			if(second<0 || second>60)
				second=0;
		}
	}
	int t=hour*3600+minute*60+second;
	if(t<0)	return 0;

	return t;
}

// Parse +-MM:SS or +-HH:MM:SS
int convertAbsoluteTimeMS(const string &m)
{
	if(m.length()==0)
		return NOTIME;

	int minute=0;
	int second=0;

  int signpos=m.find_first_of('-');
  string mtext;
  
  int sign=1;

  if(signpos!=string::npos) {
    sign=-1;
    mtext=m.substr(signpos+1);		
  }
  else
    mtext=m;
  
	minute=atoi(mtext.c_str());

	if(minute<0 || minute>60*24)
		minute=0;

	int kp=mtext.find_first_of(':');

  if (kp!=string::npos) {
    mtext = mtext.substr(kp+1);
		second = atoi(mtext.c_str());
		if(second<0 || second>60)
			second=0;
	}

	int t=minute*60+second;

	kp=mtext.find_first_of(':');
  if (kp!=string::npos) {
    //Allow also for format +-HH:MM:SS
    mtext = mtext.substr(kp+1);
		second=atoi(mtext.c_str());
		if(second<0 || second>60)
			second=0;
    else
      t = t*60 + second;
	}


	return sign*t;

/*

	int t=convertAbsoluteTimeHMS(m);

	if (t<=0)
		return t;

	if (t%60==0)
		return t/60;
	else 
		return t; //This is really abuse!*/
}

//Generate +-MM:SS or +-HH:MM:SS
string getTimeMS(int m)
{
	char bf[32];
  if (abs(m)<3600)
	  sprintf_s(bf, "%02d:%02d", m/60, abs(m)%60);
  else
    sprintf_s(bf, "%02d:%02d:%02d", m/3600, (abs(m)/60)%60, abs(m)%60);
	return bf;
}

string formatTime(int rt)
{
  if(rt>0) {
		char bf[16];
		if(rt>=3600)
			sprintf_s(bf, 16, "%d:%02d:%02d", rt/3600,(rt/60)%60, rt%60);
		else
			sprintf_s(bf, 16, "%d:%02d", (rt/60), rt%60);

		return bf;
	}
	return "-";
}

string formatTimeHMS(int rt)
{
  if(rt>0) {
		char bf[16];
	  sprintf_s(bf, 16, "%02d:%02d:%02d", rt/3600,(rt/60)%60, rt%60);

		return bf;
	}
	return "-";
}

void split(const string &line, const string &separators, vector<string> &split_vector)
{
  split_vector.clear();

  if(line.empty())
    return;

  size_t startpos=0;
  size_t nextp=line.find(separators.c_str(), startpos, separators.size());
  split_vector.push_back(line.substr(startpos, nextp-startpos));

  while(nextp!=line.npos) {
    startpos=nextp+1;
    nextp=line.find(separators.c_str(), startpos, separators.size());
    split_vector.push_back(line.substr(startpos, nextp-startpos));
  }
}

string MakeDash(const string &t)
{
  return MakeDash(t.c_str());
}

string MakeDash(const char *t)
{
  string out(t);
  for (size_t i=0;i<out.length(); i++) {
		if(t[i]=='-')
			out[i]=BYTE(0x96);
	}
	return out;
}

string FormatRank(int rank)
{
	char r[16];
	sprintf_s(r, "(%04d)", rank);
	return r;
}

string itos(int i)
{
	char bf[32];
	_itoa_s(i, bf, 10);
	return bf;
}

bool filterMatchString(const string &c, const char *filt_lc)
{
  char key[2048];
  strcpy_s(key, c.c_str());
  CharLowerBuff(key, c.length());

  return strstr(key, filt_lc)!=0;
}

int countWords(const char *p) {
  int nwords=0;
  const unsigned char *ep=LPBYTE(p);
  while (*ep) {
    if (!isspace(*ep)) {
      nwords++;
      while ( *ep && !isspace(*ep) )
        ep++;
    }
    while (*ep && isspace(*ep))
        ep++;
  }
  return nwords;
}

string trim(const string &s)
{
	//return s;
	const char *ptr=s.c_str();
	int len=s.length();

	int i=0;
	while(i<len && isspace(BYTE(ptr[i]))) i++;

	int k=len-1;

	while(k>=0 && isspace(BYTE(ptr[k]))) k--;

	if(k>=0 && i<len)
		return s.substr(i, k+1);
	else return "";
}

bool fileExist(const char *file)
{
  return GetFileAttributes(file) != INVALID_FILE_ATTRIBUTES;
}


bool stringMatch(const string &a, const string &b) {
  string aa = trim(a);
  string bb = trim(b);

  return CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, aa.c_str(), aa.length(), bb.c_str(), bb.length())==2;
}

const string &encodeXML(const string &in)
{
  static string out;
  const char *bf = in.c_str();
  int len = in.length();
  bool needEncode = false;
  for (int k=0;k<len ;k++)
    needEncode |=  (bf[k]=='&') | (bf[k]=='>') | (bf[k]=='<') | (bf[k]=='"');

  if (!needEncode)
    return in;
  out.clear();
  for (int k=0;k<len ;k++) {
    if (bf[k]=='&')
      out+="&amp;";
    else if (bf[k]=='<')
      out+="&lt;";
    else if (bf[k]=='>')
      out+="&gt;";
    else if (bf[k]=='"')
      out+="&quot;";
    else 
      out+=bf[k];
  }
  return out;
}

const string &decodeXML(const string &in)
{
  const char *bf = in.c_str();
  int len = in.length();
  bool needDecode = false;
  for (int k=0;k<len ;k++)
    needDecode |=  (bf[k]=='&');

  if (!needDecode)
    return in;
 
  static string out;
  out.clear();
  for (int k=0;k<len ;k++) {
    if (bf[k]=='&') {
      if ( memcmp(&bf[k], "&amp;", 5)==0 )
        out+="&", k+=4;
      else if  ( memcmp(&bf[k], "&lt;", 4)==0 )
        out+="<", k+=3;
      else if  ( memcmp(&bf[k], "&gt;", 4)==0 )
        out+=">", k+=3;
      else if  ( memcmp(&bf[k], "&quot;", 4)==0 )
        out+="\"", k+=5;
      else 
        out+=bf[k];
    }
    else 
      out+=bf[k];
  }

  return out;
}

const char *decodeXML(const char *in)
{
  const char *bf = in;
  bool needDecode = false;
  for (int k=0; bf[k] ;k++)
    needDecode |=  (bf[k]=='&');

  if (!needDecode)
    return in;
  
  static string out;
  out.clear();
  for (int k=0;bf[k] ;k++) {
    if (bf[k]=='&') {
      if ( memcmp(&bf[k], "&amp;", 5)==0 )
        out+="&", k+=4;
      else if  ( memcmp(&bf[k], "&lt;", 4)==0 )
        out+="<", k+=3;
      else if  ( memcmp(&bf[k], "&gt;", 4)==0 )
        out+=">", k+=3;
      else if  ( memcmp(&bf[k], "&quot;", 4)==0 )
        out+="\"", k+=5;
      else 
        out+=bf[k];
    }
    else 
      out+=bf[k];
  }

  return out.c_str();
}
