/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden

************************************************************************/

// oCard.cpp: implementation of the oCard class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "oCard.h"
#include "oEvent.h"
#include "gdioutput.h"
#include "table.h"
#include "Localizer.h"

#include "SportIdent.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

oCard::oCard(oEvent *poe): oBase(poe)
{
  Id=oe->getFreeCardId();
	CardNo=0;
	ReadId=0;
  tOwner=0;
}

oCard::oCard(oEvent *poe, int id): oBase(poe)
{
  Id=id;
	CardNo=0;
	ReadId=0;
  tOwner=0;
  oe->qFreeCardId = max(id, oe->qFreeCardId);
}


oCard::~oCard()
{
	
}

bool oCard::Write(xmlparser &xml)
{
	if(Removed) return true;

	xml.StartTag("Card");
	xml.Write("CardNo", CardNo);
	xml.Write("Punches", getPunchString());
	xml.Write("ReadId", ReadId);
	xml.Write("Id", Id);
	xml.Write("Updated", Modified.GetStamp());
	xml.EndTag();

	return true;
}

void oCard::Set(const xmlobject *xo)
{
	const xmlList *xl=xo->GetObjects();

	if(xl){
		xmlList::const_iterator it;

		for(it=xl->begin(); it != xl->end(); ++it){
			if(it->Is("CardNo")){
				CardNo=atoi(it->Get().c_str());			
			}
			else if(it->Is("Punches")){
				importPunches(it->Get());
			}
			else if(it->Is("ReadId")){
				ReadId=atoi(it->Get().c_str());			
			}
			else if(it->Is("Id")){
				Id=atoi(it->Get().c_str());			
			}
			else if(it->Is("Updated")){
				Modified.SetStamp(it->Get());
			}
		}
	}
}

void oCard::setCardNo(int c)
{
	if(CardNo!=c)
		updateChanged();

	CardNo=c;
}

string oCard::getCardNo() const
{
  char bf[32];
  sprintf_s(bf, "%d", CardNo);
  return bf;
}

void oCard::addPunch(int type, int time, int matchControlId)
{
	oPunch p(oe);
	p.Time = time;
	p.Type = type;
  p.tMatchControlId = matchControlId;
  p.isUsed =  matchControlId!=0;

  if (Punches.empty())
    Punches.push_back(p);
  else {
    oPunch oldBack = Punches.back();
    if (oldBack.isFinish()) { //Make sure finish is last.
      Punches.pop_back();
      Punches.push_back(p);
      Punches.push_back(oldBack);
    }
    else
      Punches.push_back(p);
  }
	updateChanged();
}

string oCard::getPunchString()
{
	oPunchList::iterator it;

	string punches;

	for(it=Punches.begin(); it != Punches.end(); ++it){
		punches+=it->codeString();
	}

	return punches;
}

void oCard::importPunches(string s)
{
	int startpos=0;
	int endpos;

	endpos=s.find_first_of(';', startpos);
	Punches.clear();

	while(endpos!=string::npos) {
		oPunch p(oe);
		p.decodeString(s.substr(startpos, endpos));
		Punches.push_back(p);
		startpos=endpos+1;
		endpos=s.find_first_of(';', startpos);
	}
	return;
}

bool oCard::fillPunches(gdioutput &gdi, string name, pCourse crs)
{
	oPunchList::iterator it;	
	syncronize(true);

	gdi.clearList(name);
	
	bool HasStart=false;
	bool HasFinish=false;
	bool Extra=false;
	int k=0;

  pControl ctrl=0;
 
  int matchPunch=0;
  int punchRemain=1;
  if(crs)
    ctrl=crs->getControl(matchPunch);

  if(ctrl)
    punchRemain=ctrl->getNumMulti();

	for (it=Punches.begin(); it != Punches.end(); ++it){			
		if(!HasStart && !it->isStart()){
			if(it->isUsed){
				gdi.addItem(name, lang.tl("Start")+"\t-", -1);
				HasStart=true;
			}
		}
		
		if(it->isStart())  
			HasStart=true;
    else if(it->isUsed && !it->isFinish() &&  !it->isCheck()) {
      while(ctrl && it->tMatchControlId!=ctrl->getId()) {
        while(0<punchRemain--)
          gdi.addItem(name, "-\t-", -1);

        ctrl=crs ? crs->getControl(++matchPunch):0;
        punchRemain=ctrl ? ctrl->getNumMulti() : 1;
      }
    }
    
    if(it->isUsed || it->isFinish() || it->isStart()) {
			gdi.addItem(name, it->getString(), k);
      if(!(it->isFinish() || it->isStart())) {
        punchRemain--;

        if(punchRemain<=0) {
          ctrl=crs ? crs->getControl(++matchPunch):0;
          punchRemain = ctrl ? ctrl->getNumMulti() : 1;
        }
      }
    }
		else Extra=true;
		
    k++;

		if(it->isFinish())
			HasFinish=true;
	}

	if(!HasStart)
		gdi.addItem(name, lang.tl("Start")+"\t-", -1);
	
	if(!HasFinish)
		gdi.addItem(name, lang.tl("Ml")+"\t-", 1024);
	

	if(Extra)
	{
		//Show punches that are not used.
		k=0;
		gdi.addItem(name, "", 1023);
		gdi.addItem(name, lang.tl("Extra stmplingar"), 1023);
		for (it=Punches.begin(); it != Punches.end(); ++it)
		{			
			if(!it->isUsed && !it->isFinish() && !it->isStart())
				gdi.addItem(name, it->getString(), 1023);

			//k++;

		}
	}
	return true;
}

void oCard::insertPunchAfter(int pos, int type, int time)
{
	if(pos==1023)
		return;

	oPunchList::iterator it;	

	oPunch punch(oe);
	punch.Time=time;
	punch.Type=type;

	int k=-1;
	for (it=Punches.begin(); it != Punches.end(); ++it)	{	
		if(k==pos) {
			updateChanged();
			Punches.insert(it, punch);
			return;
		}
		k++;		
	}

	updateChanged();
	//Insert last
	Punches.push_back(punch);
}

void oCard::deletePunch(int pos)
{
	int k=0;
	oPunchList::iterator it;
	
	for (it=Punches.begin(); it != Punches.end(); ++it)	{	
		if(k==pos) {
			Punches.erase(it);
			updateChanged();
			return;
		}
		k++;		
	}
}

string oCard::getInfo() const
{
  char bf[128];
  sprintf_s(bf, lang.tl("Lparbricka %d").c_str(), CardNo);
  return bf;
}

oPunch *oCard::getPunch(int pos)
{
	int k=0;
	oPunchList::iterator it;

	for (it=Punches.begin(); it != Punches.end(); ++it) {	
		if(k==pos)	return &*it;		
		k++;		
	}
	return 0;
}


oPunch *oCard::getPunchByType(int Type) const
{
	oPunchList::const_iterator it;

	for (it=Punches.begin(); it != Punches.end(); ++it)
		if(it->Type==Type) 
			return pPunch(&*it);		

	return 0;
}

oPunch *oCard::getPunchById(int Id) const
{
	oPunchList::const_iterator it;

	for (it=Punches.begin(); it != Punches.end(); ++it)
    if(it->tMatchControlId==Id) 
			return pPunch(&*it);		

	return 0;
}

void oCard::setReadId(const SICard &card)
{
	updateChanged();
	ReadId=card.nPunch*100000+card.FinishPunch.Time;
}

bool oCard::isCardRead(const SICard &card) const
{
	if(ReadId==(card.nPunch*100000+card.FinishPunch.Time))
		return true;
	else return false;
}


bool oCard::setPunchTime(int pos, string time)
{
	oPunch *op=getPunch(pos);
	if(!op) return false;

	DWORD ot=op->Time;
	op->setTime(time);
	
	if(ot!=op->Time)
		updateChanged();
	
	return true;
}

string oCard::getPunchTime(int pos)
{
	oPunch *op=getPunch(pos);
	if(!op) return "";

	return op->getTime();
}

pCard oEvent::getCard(int Id) const
{
	oCardList::const_iterator it;
		
	for (it=Cards.begin(); it != Cards.end(); ++it){
		if(it->Id==Id)	
      return const_cast<pCard>(&*it);
	}	
	return 0;
}

pCard oEvent::addCard(const oCard &oc)
{
  if (oc.Id<=0)
    return 0;

  Cards.push_back(oc);

  return &Cards.back();
}


pCard oEvent::getCardByNumber(int cno) const
{
	oCardList::const_iterator it;
		
	for (it=Cards.begin(); it != Cards.end(); ++it){
		if(it->CardNo==cno)	
      return const_cast<pCard>(&*it);
	}	
	return 0;
}

bool oEvent::isCardRead(const SICard &card) const
{
  oCardList::const_iterator it;

	for(it=Cards.begin(); it!=Cards.end(); ++it) {
		if(it->CardNo==card.CardNumber && it->isCardRead(card))
			return true;
	}

	return false;
}

void oCard::getCellData(int id, string &contents) const
{
  oPunch *p;
  string time;
  char npunch[16];

  switch(id) {
    case TID_CARD:
      contents=getCardNo();
      break;

    case TID_START:
      p=getPunchByType(oPunch::PunchStart);
      contents = p ? p->getTime() : "-";
      break;

    case TID_FINISH:
      p=getPunchByType(oPunch::PunchFinish);
      contents = p ? p->getTime() : "-";
      break;

    case TID_COURSE:
      sprintf_s(npunch, "%d", getNumPunches());
      contents = npunch;
      break;

    case TID_RUNNER:
      if(!tOwner) 
        contents="Oparad";
      else 
        contents=tOwner->getNameAndRace();
      break;
  }
}

Table *oEvent::getCardsTB() //Table mode
{	
 	oCardList::iterator it;	

  Table *table=new Table(this, 20, "Brickor", "cards");

	table->addColumn("Bricka", 120, true);
	table->addColumn("Deltagare", 200, false);

  table->addColumn("Starttid", 70, false);
  table->addColumn("Mltid", 70, false);
  table->addColumn("Stmplingar", 70, false);
  
  table->update();
/*
  for (it=Cards.begin(); it!=Cards.end(); ++it) {
    if(!it->isRemoved()) {
      table->addRow(it->getId());
					
      string runner("Oparad");
      if(it->tOwner) 
        runner=it->tOwner->getNameAndRace();

      table->set(0, *it, TID_CARD, it->getCardNo(), true, cellAction);

      table->set(1, *it, TID_RUNNER, runner, true, cellAction);

      oPunch *p=it->getPunchByType(oPunch::PunchStart);
      string time="-";
      if (p) time=p->getTime();
      table->set(2, *it, TID_START, time, false, cellEdit);	

      p=it->getPunchByType(oPunch::PunchFinish);
      time="-";
      if (p) time=p->getTime();
      table->set(3, *it, TID_FINISH, time, false, cellEdit);	

      char npunch[16];
      sprintf_s(npunch, "%d", it->getNumPunches());
      table->set(4, *it, TID_COURSE, npunch, false, cellEdit);
    }
  }
*/
	return table;
}

void oEvent::generateCardTableData(Table *table)
{	
 	oCardList::iterator it;	
	syncronizeList(oLCardId);
  syncronizeList(oLRunnerId);

  for (it=Cards.begin(); it!=Cards.end(); ++it) {
    if(!it->isRemoved()) {
      table->addRow(it->getId());
					
      string runner("Oparad");
      if(it->tOwner) 
        runner=it->tOwner->getNameAndRace();

      table->set(0, *it, TID_CARD, it->getCardNo(), true, cellAction);

      table->set(1, *it, TID_RUNNER, runner, true, cellAction);

      oPunch *p=it->getPunchByType(oPunch::PunchStart);
      string time="-";
      if (p) time=p->getTime();
      table->set(2, *it, TID_START, time, false, cellEdit);	

      p=it->getPunchByType(oPunch::PunchFinish);
      time="-";
      if (p) time=p->getTime();
      table->set(3, *it, TID_FINISH, time, false, cellEdit);	

      char npunch[16];
      sprintf_s(npunch, "%d", it->getNumPunches());
      table->set(4, *it, TID_COURSE, npunch, false, cellEdit);
    }
  }
}

