/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

// oControl.cpp: implementation of the oControl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "oControl.h"
#include "oEvent.h"
#include "gdioutput.h"
#include "meos_util.h"
#include <cassert>
#include "Localizer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

oControl::oControl(oEvent *poe): oBase(poe)
{
	getDI().initData();
	nNumbers=0;
	Status=StatusOK;
}

oControl::oControl(oEvent *poe, int id): oBase(poe)
{
  Id = id;
	getDI().initData();
	nNumbers=0;
	Status=StatusOK;
}


oControl::~oControl()
{
}

bool oControl::write(xmlparser &xml)
{
	if(Removed) return true;

	xml.StartTag("Control");

	xml.Write("Id", Id);
	xml.Write("Updated", Modified.GetStamp());
	xml.Write("Name", Name);
	xml.Write("Numbers", codeNumbers());
	xml.Write("Status", Status);
	
	getDI().write(xml);
	xml.EndTag();

	return true;
}

void oControl::set(int pId, int pNumber, string pName)
{
	Id=pId;
	Numbers[0]=pNumber;
	nNumbers=1;
	Name=pName;
	
	updateChanged();
}


void oControl::setStatus(ControlStatus st){
	if(st!=Status){
		Status=st;
		updateChanged();
	}
}

void oControl::setName(string name)
{
  if(name!=getName()){
		Name=name;
		updateChanged();
	}
}


void oControl::set(const xmlobject *xo)
{
	const xmlList *xl=xo->GetObjects();
	nNumbers=0;
	Numbers[0]=0;
	if(xl){
		xmlList::const_iterator it;

		for(it=xl->begin(); it != xl->end(); ++it){
			if(it->Is("Id")){
				Id=atoi(it->Get().c_str());			
			}
			else if(it->Is("Number")){
				Numbers[0]=atoi(it->Get().c_str());			
				nNumbers=1;
			}
			else if(it->Is("Numbers")){
				decodeNumbers(it->Get());
			}
			else if(it->Is("Status")){
				Status=(ControlStatus)atoi(it->Get().c_str());			
			}
			else if(it->Is("Name")){
				Name=it->Get();
			}
			else if(it->Is("Updated")){
				Modified.SetStamp(it->Get());
			}
      else if(it->Is("oData")){
        getDI().set(&*it);
			}
		}
	}

}

string oControl::getString()
{
	char bf[32];
	if(Status==StatusOK)
		return codeNumbers('|');
  else if(Status==StatusMultiple)
    return codeNumbers('+');
	else
		sprintf_s(bf, 32, "~%s", codeNumbers().c_str());
	return bf;
}

string oControl::getLongString()
{
	char bf[32];
	if(Status==StatusOK){
		if(nNumbers==1)
			return codeNumbers('|');
		else
			return string("VALFRI(")+codeNumbers(',')+")";
		//sprintf(bf, "%d", Number);
	}
	else
		sprintf_s(bf, 32, "TRASIG(%s)", codeNumbers(',').c_str());
	return bf;
}

bool oControl::hasNumber(int i) 
{
	for(int n=0;n<nNumbers;n++)
    if(Numbers[n]==i) {
      // Mark this number as checked
      checkedNumbers[n]=true;
      return true;
    }
	if(nNumbers>0)
		return false;
	else return true;
}

bool oControl::hasNumberUnchecked(int i) 
{
	for(int n=0;n<nNumbers;n++)
    if(Numbers[n]==i && checkedNumbers[n]==0) {
      // Mark this number as checked
      checkedNumbers[n]=true;
      return true;
    }
	if(nNumbers>0)
		return false;
	else return true;
}


int oControl::getNumMulti() 
{
  if(Status==StatusMultiple)
    return nNumbers;
  else
    return 1;
}


string oControl::codeNumbers(char sep)
{
	string n;
	char bf[16];

	for(int i=0;i<nNumbers;i++){
		_itoa_s(Numbers[i], bf, 16, 10);
		n+=bf;
		if(i+1<nNumbers)
			n+=sep;
	}
	return n;
}

bool oControl::decodeNumbers(string s)
{
	const char *str=s.c_str();

	nNumbers=0;

	while(*str){
		int cid=atoi(str);

		while(*str && (*str!=';' && *str!=',')) str++;
		if(*str==';'  || *str==',') str++;		

		if(cid>0 && cid<1024 && nNumbers<32)
			Numbers[nNumbers++]=cid;
	}

	if(Numbers==0){
		Numbers[0]=0;
		nNumbers=1;
		return false;
	}
	else return true;
}

bool oControl::setNumbers(const string &numbers)
{	
	int nn=nNumbers;
	int bf[32];

	if(unsigned(nNumbers)<32)
		memcpy(bf, Numbers, sizeof(int)*nNumbers);

	bool success=decodeNumbers(numbers);

	if(nNumbers!=nn || memcmp(bf, Numbers, sizeof(int)*nNumbers)!=0)
		updateChanged();

	return success;	
}

string oControl::getName() const
{
	if(!Name.empty())
		return Name;
	else {
		char bf[16];
		sprintf_s(bf, "[%d]", Id);
		return bf;
	}
}

oDataInterface oControl::getDI(void)
{
	return oe->oControlData->getInterface(oData, sizeof(oData), this);
}

oDataConstInterface oControl::getDCI(void) const
{
	return oe->oControlData->getConstInterface(oData, sizeof(oData), this);
}


bool oEvent::fillControls(gdioutput &gdi, const string &name, int type)
{	
	oControlList::iterator it;	
	syncronizeList(oLControlId);
  Controls.sort();
	gdi.clearList(name);
	string b;
	char bf[16];
	for (it=Controls.begin(); it != Controls.end(); ++it){

		if(!it->Removed){
			b.clear();

      if(type==0) {
			  if(it->Status==oControl::StatusOK)
				  b+="[ok]";
			  else b+="[!!]";

			  //sprintf(bf, "%d", it->Number); 
			  sprintf_s(bf, 16, " %s", it->codeNumbers(' ').c_str());
			  b+=bf;

  			
			  if(it->Name.length()>0)
				  b+="\t("+it->Name+")";
      }
      else if(type==1) {
        sprintf_s(bf, 16, lang.tl("Kontroll %s").c_str(), it->codeNumbers(' ').c_str());
			  b=bf;
        
			  if(!it->Name.empty())
				  b+=" ("+it->Name+")";
      }
			gdi.addItem(name, b, it->Id);
		}
	}
	return true;
}

bool oEvent::fillControlTypes(gdioutput &gdi, const string &name)
{	
	oControlList::iterator it;	
	syncronizeList(oLControlId);

	gdi.clearList(name);
	set<int> sicodes;

	for (it=Controls.begin(); it != Controls.end(); ++it){
    if(!it->Removed) {
      for (int k=0;k<it->nNumbers;k++)
        sicodes.insert(it->Numbers[k]);
    }
	}

  set<int>::iterator sit;
  char bf[32];
  gdi.addItem(name, lang.tl("Check"), oPunch::PunchCheck);
  gdi.addItem(name, lang.tl("Start"), oPunch::PunchStart);
  gdi.addItem(name, lang.tl("Ml"), oPunch::PunchFinish);

  for (sit = sicodes.begin(); sit!=sicodes.end(); ++sit) {
    sprintf_s(bf, lang.tl("Kontroll %s").c_str(), itos(*sit).c_str());
    gdi.addItem(name, bf, *sit);
  }
	return true;
}

int oControl::getTimeAdjust() const
{
  return getDCI().getInt("TimeAdjust");
}

string oControl::getTimeAdjustS() const
{
  return getTimeMS(getTimeAdjust());
}

void oControl::setTimeAdjust(int v)
{
  getDI().setInt("TimeAdjust", v);
}

void oControl::setTimeAdjust(const string &s)
{
  setTimeAdjust(convertAbsoluteTimeMS(s));
}


void oControl::startCheckControl()
{
  //Mark all numbers as unchecked.
  for (int k=0;k<nNumbers;k++)
    checkedNumbers[k]=false;
}
 
string oControl::getInfo() const
{
  return getName();
}

void oControl::addUncheckedPunches(vector<int> &mp) const
{
  if(controlCompleted())
    return;

  for (int k=0;k<nNumbers;k++)
    if(!checkedNumbers[k]) {
      mp.push_back(Numbers[k]);

      if(Status!=StatusMultiple)
        return;
    }
}

int oControl::getMissingNumber() const
{
  for (int k=0;k<nNumbers;k++)
    if (!checkedNumbers[k])
      return Numbers[k];

  assert(false);
  return Numbers[0];//This should not happen
}

bool oControl::controlCompleted() const
{
  if (Status==StatusOK) {
    //Check if any number is used.
    for (int k=0;k<nNumbers;k++)
      if(checkedNumbers[k])
        return true;

    //Return true only if there is no control
    return nNumbers==0;
  }
  else if (Status==StatusMultiple) {
    //Check if al numbers are used.
    for (int k=0;k<nNumbers;k++)
      if(!checkedNumbers[k])
        return false;

    return true;
  }
  else return true;
}
