/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License fro more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include <vector>
#include <set>
#include <cassert>
#include <algorithm>

#include "oEvent.h"
#include "gdioutput.h"
#include "oDataContainer.h"

#include "random.h"
#include "SportIdent.h"

#include "oFreeImport.h"

#include "meos.h"
#include "meos_util.h"
#include "RunnerDB.h"
#include "localizer.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>

//Version of database
int oEvent::dbVersion = 53;

oEvent::oEvent(gdioutput &gdi):oBase(0), gdibase(gdi)
{
  hMod=0;
	ZeroTime=0;

  sqlCounterRunners=0;
	sqlCounterClasses=0;
	sqlCounterCourses=0;
	sqlCounterControls=0;
	sqlCounterClubs=0;
	sqlCounterCards=0;
	sqlCounterPunches=0;
	sqlCounterTeams=0;

	SYSTEMTIME st;
	GetLocalTime(&st);

	char bf[64];
	sprintf_s(bf, 64, "%d-%02d-%02d", st.wYear, st.wMonth, st.wDay);
	
	Date=bf;
	ZeroTime=st.wHour*3600;
	oe=this;

  runnerDB = new RunnerDB(this);

  char cp[64];
  DWORD size=64;
  GetComputerName(cp, &size);
  clientName=cp;

	HasDBConnection=false;
	msSyncronizeList=0;
	msSyncronizeRead=0;
	msSyncronizeUpdate=0;
	msOpenDatabase=0;
  msRemove=0;
  msMonitor=0;

  msGetErrorState=0;
  msResetConnection=0;
  msReConnect=0;

	//These object must be initialized on creation of any oObject,
	//but we need to create (dummy) objects to get the sizeof their 
	//oData[]-sets...

  // --- REMEBER TO UPDATE dvVersion when these are changed.

  oEventData=new oDataContainer(sizeof(oData));
	oEventData->initData(oData, sizeof(oData));
  oEventData->addVariableInt("CardFee", oDataContainer::oIS16, "Hyravgift");
  oEventData->addVariableInt("EliteFee", oDataContainer::oIS16, "Elitavgift");
  oEventData->addVariableInt("EntryFee", oDataContainer::oIS16, "Anmlningsavgift");
  oEventData->addVariableInt("YouthFee", oDataContainer::oIS16, "Ungdomsavgift");
  oEventData->addVariableString("Account", 30, "Konto");
  oEventData->addVariableDate("PaymentDue", "Sista betalningsdatum");

  oEventData->addVariableString("Organizer", 32, "Arrangr");
  oEventData->addVariableString("Street", 32, "Adress");
  oEventData->addVariableString("Address", 32, "Postadress");
  oEventData->addVariableString("EMail", 40, "E-post");
  oEventData->addVariableString("Homepage", 40, "Hemsida");
 
  oEventData->addVariableInt("UseEconomy", oDataContainer::oIS8U, "Ekonomi");
  oEventData->addVariableInt("UseSpeaker", oDataContainer::oIS8U, "Speaker");
  
	oDataContainer dummy(0);
	oClubData=oRunnerData=oClassData=oCourseData=oControlData=&dummy;
	oTeamData=&dummy;

	oClub oc(this);
	oClubData=new oDataContainer(sizeof (oc.oData));
	oClubData->addVariableInt("District", oDataContainer::oIS8U, "Distriktskod");
	
	oClubData->addVariableString("CareOf", 31, "c/o");
	oClubData->addVariableString("Street", 41, "Gata");
	oClubData->addVariableString("City", 23, "Stad");
	oClubData->addVariableString("ZIP", 11, "Postkod");
	oClubData->addVariableString("EMail", 41, "E-post");
	oClubData->addVariableString("Phone", 23, "Telefon");
	oClubData->addVariableString("Nationality", 3, "Nationalitet");

	oRunner or(this);
	oRunnerData=new oDataContainer(sizeof (or.oData));
	oRunnerData->addVariableInt("Fee", oDataContainer::oIS32, "Anm. avgift");
	oRunnerData->addVariableInt("CardFee", oDataContainer::oIS32, "Brickhyra");
	oRunnerData->addVariableInt("Paid", oDataContainer::oIS32, "Betalat");
	oRunnerData->addVariableInt("BirthYear", oDataContainer::oIS32, "Fdelser");	
	oRunnerData->addVariableInt("Bib", oDataContainer::oIS16U, "Nummerlapp");
	oRunnerData->addVariableInt("Rank", oDataContainer::oIS16U, "Ranking");
	oRunnerData->addVariableInt("VacRank", oDataContainer::oIS16U, "Vak. ranking");

	oRunnerData->addVariableDate("EntryDate", "Anm. datum");
	
	oRunnerData->addVariableString("Sex", 1, "Kn");
	oRunnerData->addVariableString("Nationality", 3, "Nationalitet");

	oControl ocn(this);
	oControlData=new oDataContainer(sizeof (ocn.oData));
  oControlData->addVariableInt("TimeAdjust", oDataContainer::oIS32, "Tidsjustering");

	oCourse ocr(this);
	oCourseData=new oDataContainer(sizeof (ocr.oData));
  oCourseData->addVariableInt("NumberMaps", oDataContainer::oIS16, "Kartor");
  oCourseData->addVariableString("StartName", 16, "Start");
	
	oClass ocl(this);
	oClassData=new oDataContainer(sizeof (ocl.oData));

	oClassData->addVariableInt("LowAge", oDataContainer::oIS8U, "Undre lder");
	oClassData->addVariableInt("HighAge", oDataContainer::oIS8U, "vre lder");
  oClassData->addVariableInt("HasPool", oDataContainer::oIS8U, "Banpool");
  oClassData->addVariableInt("AllowQuickEntry", oDataContainer::oIS8U, "Direktanmlan");
  
	oClassData->addVariableString("ClassType", 16, "Klasstyp");
	oClassData->addVariableString("Sex", 1, "Kn");
	oClassData->addVariableString("StartName", 16, "Start");
	oClassData->addVariableInt("StartBlock", oDataContainer::oIS8U, "Block");
  oClassData->addVariableInt("NoTiming", oDataContainer::oIS8U, "Ej tidtagning");

	oTeam ot(this);
	oTeamData=new oDataContainer(sizeof(ot.oData));
	oTeamData->addVariableInt("Fee", oDataContainer::oIS32, "Anm. avgift");
	oTeamData->addVariableInt("Paid", oDataContainer::oIS32, "Betalat");
	oTeamData->addVariableDate("EntryDate", "Anm. datum");
	oTeamData->addVariableString("Nationality", 3, "Nationalitet");
	oTeamData->addVariableInt("Bib", oDataContainer::oIS16U, "Nummerlapp");
	
  currentClientCS = 0;
}

oEvent::~oEvent()
{
	//Clean up things in the right order.
	clear();
  delete runnerDB;
  runnerDB=0;

  if (hMod) {
    HasDBConnection=false;
    msSyncronizeList=0;
    msSyncronizeRead=0;
    msSyncronizeUpdate=0;
    msOpenDatabase=0;
    msRemove=0;

    msGetErrorState=0;
    msResetConnection=0;
    msReConnect=0;
    FreeLibrary(hMod);
    hMod=0;
  }

	delete oEventData;
	delete oRunnerData;
	delete oClubData;
	delete oControlData;
	delete oCourseData;
	delete oClassData;
	delete oTeamData;	
  return;
}

pControl oEvent::addControl(int Id, int Number, const string &Name)
{
	if(Id<=0)
		Id=getFreeControlId();
  else
    qFreeControlId = max (qFreeControlId, Id);

	oControl c(this);
	c.set(Id, Number, Name);
	Controls.push_back(c);

  oe->updateTabs();
	return &Controls.back();
}

int oEvent::getNextControlNumber() const
{
  int c = 31;
  for (oControlList::const_iterator it = Controls.begin(); it!=Controls.end(); ++it)
    c = max(c, it->maxNumber()+1);

  return c;
}

pControl oEvent::addControl(const oControl &oc)
{
  if(oc.Id<=0)
		return 0;

  if (getControl(oc.Id, false))
    return 0;
    
  qFreeControlId = max (qFreeControlId, Id);

	Controls.push_back(oc);
	return &Controls.back();
}

const int legHashConstant = 100000;

int oEvent::getControlIdFromPunch(int time, int type, int card, bool markClassChanged)
{
  pRunner r=getRunnerByCard(card);

  if (markClassChanged && r && r->Class)
    r->Class->sqlChanged = true;

  if (r!=0) 
    r = r->tParentRunner ? r->tParentRunner : r;
  
  int race = 0;
  // Try to figure out to which (multi)runner punch belong
  // by looking between which start and finnish the time is
  if (r && r->getNumMulti()) {
    if (r->Class)
      r->Class->sqlChanged = true; //Mark class as changed.

    pRunner r2 = r;
    while (r2 && r2->Card && race<r->getNumMulti()) {
      if (time >= r2->getStartTime() && time <= r2->getFinishTime())
        break;
      
      race++;
      r2=r->getMultiRunner(race);
    }
  }

  if (type!=oPunch::PunchFinish) {  
    pCourse c = r ? r->getCourse(): 0;

    if (c!=0) {
      for (int k=0; k<c->nControls; k++) {
        pControl ctrl=c->getControl(k);
        if(ctrl && ctrl->hasNumber(type)) {
          pFreePunch p = getPunch(race, ctrl->getId(), card);
          if(!p || (p && abs(p->Time-time)<60)) {
            //Code controlId and runner race number into code
            return ctrl->getId() + race*legHashConstant;
          }          
        }
      }
    }
  }
  
  return type + race*legHashConstant; 
}

pFreePunch oEvent::addFreePunch(int time, int type, int card)
{	
  int itype = getControlIdFromPunch(time, type, card, true);

	oFreePunch ofp(this);			
	ofp.setData(card, time, type);
	punches.push_back(ofp);	
	pFreePunch fp=&punches.back();
  punchIndex[itype][card]=fp;

  return fp;
}

pFreePunch oEvent::addFreePunch(oFreePunch &fp)
{								
  int itype = getControlIdFromPunch(fp.Time, fp.Type, fp.CardNo, true);

	punches.push_back(fp);	
	pFreePunch fpz=&punches.back();
  punchIndex[itype][fp.CardNo]=fpz;
  return fpz;
}

void oEvent::rebuildPunchIndex()
{
}

void oEvent::removeFreePunch(int id)
{
	oFreePunchList::iterator it;	

	for (it=punches.begin(); it != punches.end(); ++it) {
    if(it->Id==Id) {
      punchIndex[it->Type][it->CardNo]=0;
      punches.erase(it);
			return;
    }
	}
}

pFreePunch oEvent::getPunch(int Id) const
{
	oFreePunchList::const_iterator it;	

	for (it=punches.begin(); it != punches.end(); ++it) {
		if(it->Id==Id)
			return const_cast<pFreePunch>(&*it);
	}
	return 0;
}

pFreePunch oEvent::getPunch(int runnerRace, int type, int card) const
{
  map<int, map<int, pFreePunch> >::const_iterator it1;
    
  it1=punchIndex.find(type + runnerRace*legHashConstant);

  if (it1!=punchIndex.end()) {
    map<int, pFreePunch>::const_iterator it2;
    const map<int, pFreePunch> &cIndex=it1->second;

    it2=cIndex.find(card);

    if(it2!=cIndex.end())
      return it2->second;
    else return 0;
  }
	return 0;
}

pControl oEvent::getControl(int Id, bool create)
{
	oControlList::iterator it;	

	for (it=Controls.begin(); it != Controls.end(); ++it) {
		if(it->Id==Id)
			return &*it;
	}

  if(!create || Id<=0)
    return 0;

	//Not found. Auto add...
	return addControl(Id, Id, "");
}

bool oEvent::existControl(int Id)
{
	oControlList::iterator it;	

	for (it=Controls.begin(); it != Controls.end(); ++it){
		if(it->Id==Id)
			return true;
	}
	return false;
}

bool oEvent::existCourse(int Id)
{
	oCourseList::iterator it;	

	for (it=Courses.begin(); it != Courses.end(); ++it){
		if(it->Id==Id)
			return true;
	}
	return false;
}

bool oEvent::existClass(int Id)
{
	oClassList::iterator it;	

	for (it=Classes.begin(); it != Classes.end(); ++it){
		if(it->Id==Id)
			return true;
	}
	return false;
}

bool oEvent::existClub(int Id)
{
	oClubList::iterator it;	

	for (it=Clubs.begin(); it != Clubs.end(); ++it){
		if(it->Id==Id)
			return true;
	}
	return false;
}


bool oEvent::existRunner(int Id)
{
	oRunnerList::iterator it;	

	for (it=Runners.begin(); it != Runners.end(); ++it){
		if(it->Id==Id)
			return true;
	}
	return false;
}



bool oEvent::writeControls(xmlparser &xml)
{
	oControlList::iterator it;	

	xml.StartTag("ControlList");

	for (it=Controls.begin(); it != Controls.end(); ++it)
		it->write(xml);

	xml.EndTag();
 
	return true;
}

bool oEvent::writeCourses(xmlparser &xml)
{
	oCourseList::iterator it;	

	xml.StartTag("CourseList");

	for (it=Courses.begin(); it != Courses.end(); ++it)
		it->Write(xml);

	xml.EndTag();
 
	return true;
}

bool oEvent::writeClasses(xmlparser &xml)
{
	oClassList::iterator it;	

	xml.StartTag("ClassList");

	for (it=Classes.begin(); it != Classes.end(); ++it)
		it->Write(xml);

	xml.EndTag();
 
	return true;
}

bool oEvent::writeClubs(xmlparser &xml)
{
	oClubList::iterator it;	

	xml.StartTag("ClubList");

	for (it=Clubs.begin(); it != Clubs.end(); ++it)
		it->Write(xml);

	xml.EndTag();
 
	return true;
}

bool oEvent::writeRunners(xmlparser &xml)
{
	oRunnerList::iterator it;	

	xml.StartTag("RunnerList");

	for (it=Runners.begin(); it != Runners.end(); ++it)
    if(!it->tDuplicateLeg) //Duplicates is written by the ruling runner.
		  it->Write(xml);

	xml.EndTag();
 
	return true;
}


bool oEvent::writePunches(xmlparser &xml)
{
	oFreePunchList::iterator it;	

	xml.StartTag("PunchList");
	for (it=punches.begin(); it != punches.end(); ++it)
		it->Write(xml);
	xml.EndTag();
 
	return true;
}

//Write free cards not owned by a runner.
bool oEvent::writeCards(xmlparser &xml)
{
	oCardList::iterator it;	

	xml.StartTag("CardList");

  for (it=Cards.begin(); it != Cards.end(); ++it) {
    if(it->tOwner==0)
		  it->Write(xml);
  }

	xml.EndTag(); 
	return true;
}

bool oEvent::save()
{
	if(empty())
		return true;

	if (!CurrentFile[0]) 
    throw std::exception("Felaktigt filnamn");
	
	int f=0;
  _sopen_s(&f, CurrentFile, _O_RDONLY, _SH_DENYNO, _S_IWRITE);
	
	char fn1[260];
	char fn2[260];

	if(f!=-1) {
		_close(f);
		sprintf_s(fn1, MAX_PATH, "%s.bu%d", CurrentFile, 5);
		remove(fn1);

		for(int k=10;k>0;k--) {
			sprintf_s(fn1, MAX_PATH, "%s.bu%d", CurrentFile, k-1);
			sprintf_s(fn2, MAX_PATH, "%s.bu%d", CurrentFile, k);
			rename(fn1, fn2);
		}

		rename(CurrentFile, fn1);
	}

	return save(CurrentFile);
}

bool oEvent::save(const char *file)
{
	xmlparser xml;

	if(!xml.OpenOutput(file, "meosdata"))
    throw std::exception((string("Fel. Kan inte ppna: ") + file).c_str());

	xml.Write("Name", Name);
	xml.Write("Date", Date);
	xml.Write("ZeroTime", ZeroTime);
	xml.Write("Id", Id);
	xml.Write("Updated", Modified.GetStamp());

	oEventData->write(oData, xml);

	writeControls(xml);
	writeCourses(xml);
	writeClasses(xml);
	writeClubs(xml);
	writeRunners(xml);
	writeTeams(xml);
	writePunches(xml);
  writeCards(xml);

	xml.CloseOut();

	//updateRunnerDatabase();
	return true;
}

bool oEvent::open(int Id)
{
	list<CompetitionInfo>::iterator it;

	for (it=cinfo.begin(); it!=cinfo.end(); ++it) {		
  	if (it->Server.empty()) {
      if(Id==it->Id) {
        CompetitionInfo ci=*it; //Take copy
				return open(ci.FullPath.c_str());
      }
    }
		else if(!it->Server.empty()) {			
      if(Id==(10000000+it->Id)) {			
        CompetitionInfo ci=*it; //Take copy
				return readSyncronize(ci);
      }
		}
	}

	return false;
}

bool oEvent::open(const char *file, bool Import)
{
	xmlparser xml;

	if(!xml.Read(file))	{
		MessageBox(NULL, xml.GetError(), NULL, MB_OK);
		return false;
	}

	//This generates a new file name
	newCompetition("-");
	
  if (!Import) {
		strcpy_s(CurrentFile, MAX_PATH, file); //Keep new file name, if imported
 
    _splitpath_s(CurrentFile, NULL, 0, NULL,0, CurrentNameId, 64, NULL, 0);
    int i=0;
    while (CurrentNameId[i]) {
      if(CurrentNameId[i]=='.') {
        CurrentNameId[i]=0;
        break;
      }
      i++;
    }
  }
  return open(xml);
}

bool oEvent::open(const xmlparser &xml) {
	const xmlobject *xo;
	
	xo=xml.GetObject("Date");
	if(xo) Date=xo->Get();

	xo=xml.GetObject("Name");
	if(xo) Name=xo->Get();
  tOriginalName.clear();
	
  xo=xml.GetObject("ZeroTime");
	if(xo) ZeroTime=atoi(xo->Get().c_str());

	xo=xml.GetObject("Id");
	if(xo) Id=atoi(xo->Get().c_str());

	xo=xml.GetObject("oData");

	if(xo)
		oEventData->set(oData, xo);

	//Get controls
	xo=xml.GetObject("ControlList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Control")){
					oControl c(this);
					c.set(&*it);

					if(!existControl(c.Id) && c.Id>0)
						Controls.push_back(c);
					//else gdi.alert("Control duplicate");
				}
			}
		}
	}

	//Get courses
	xo=xml.GetObject("CourseList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Course")){
					oCourse c(this);
					c.Set(&*it);
					if(!existCourse(c.Id) && c.Id>0)
            addCourse(c);
				}
			}
		}
	}

	//Get classes
	xo=xml.GetObject("ClassList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Class")){
					oClass c(this);
					c.Set(&*it);
					if(!existClass(c.Id) && c.Id>0)
						Classes.push_back(c);
					//else gdi.alert("Class duplicate");
				}
			}
		}
	}

	//Get clubs
	xo=xml.GetObject("ClubList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Club")){
					oClub c(this);
					c.Set(&*it);
          if(c.Id>0)
						addClub(c);//Clubs.push_back(c);
				}
			}
		}
	}

	//Get runners
	xo=xml.GetObject("RunnerList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Runner")){
					oRunner r(this);
          r.Set(&*it);
          if (r.Id>0)
            addRunner(r);
          else if (r.Card)
            r.Card->tOwner=0;
				}
			}
		}
	}

	//Get teams
	xo=xml.GetObject("TeamList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Team")){
					oTeam t(this);
					t.set(&*it);
					if(t.Id>0){
						Teams.push_back(t);
						Teams.back().apply(false);
					}
				}
			}
		}
	}	
	
	xo=xml.GetObject("PunchList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Punch")){
					oFreePunch p(this);
					p.Set(&*it);
          addFreePunch(p);
				}
			}
		}
	}

	xo=xml.GetObject("CardList");
	if(xo){
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Card")){
					oCard c(this);
					c.Set(&*it);
          assert(c.Id>=0);
					Cards.push_back(c);
				}
			}
		}
	}

	xo=xml.GetObject("Updated");
	if(xo) Modified.SetStamp(xo->Get());

  adjustTeamMultiRunners();
	updateFreeId();
	reEvaluateAll(false);
	return true;
}


bool oEvent::openRunnerDatabase(char* filename)
{
	char file[260];
	GetUserFile(file, filename);
  
  char fold[260];
  char fclub[260];
  char frunner[260];
  strcpy_s(fold, file);
  strcat_s(fold, ".xml");

  strcpy_s(fclub, file);
  strcat_s(fclub, ".clubs");

  strcpy_s(frunner, file);
  strcat_s(frunner, ".persons");

  if (fileExist(fclub) && fileExist(frunner)) {
    runnerDB->loadClubs(fclub);
    runnerDB->loadRunners(frunner);
  }
  else if (fileExist(fold)) {
    // Read old format file to migrate.
	  xmlparser xml;

	  if(!xml.Read(fold))
		  return false;
  	
	  const xmlobject *xo;

	  //Get runners
	  xo = xml.GetObject("RunnerList");  
    const xmlList *xl=xo ? xo->GetObjects() : 0;

	  if (!xl)
      return false;
    
	  xmlList::const_iterator it;

	  for (it=xl->begin(); it != xl->end(); ++it) {
		  if (it->Is("Runner")) {
			  oRunner r(this);
			  r.Set(&*it);
        runnerDB->addCR(r.CardNo, r.Name, r.ClubName);                
		  }
	  }

    //runnerDB->compactifyClubs();
    // Save new file format
    runnerDB->saveClubs(fclub);
    runnerDB->saveRunners(frunner);
  }

	return true;
}

pRunner oEvent::DBLookUp(int CardNo)
{
	static oRunner sRunner = oRunner(this, 0);
  sRunner = oRunner(this, 0);
  if(runnerDB->getRunner(CardNo, sRunner.Name, sRunner.ClubName))
	  return &sRunner;
  else
    return 0;
}

bool oEvent::saveRunnerDatabase(char *filename)
{  
  char file[260];
	GetUserFile(file, filename);

  char fclub[260];
  char frunner[260];
  strcpy_s(fclub, file);
  strcat_s(fclub, ".clubs");

  strcpy_s(frunner, file);
  strcat_s(frunner, ".persons");

  runnerDB->saveClubs(fclub);
  runnerDB->saveRunners(frunner);
	return true;
}

void oEvent::updateRunnerDatabase()
{
  if(Name=="!TESTTVLING")
    return;

	oRunnerList::iterator it;	

	for (it=Runners.begin(); it != Runners.end(); ++it){
    if(it->Card && it->Card->CardNo==it->CardNo && 
        it->getDI().getInt("CardFee")==0)
			updateRunnerDatabase(&*it);
	}
}

void oEvent::updateRunnerDatabase(pRunner r)
{
	if(!r->CardNo)
		return;
  runnerDB->updateAdd(r->CardNo, r->Name, r->Club);
}

pCourse oEvent::addCourse(const string &pname, int plengh)
{	
	oCourse c(this);
	c.Length=plengh;
	c.Name=pname;

  return addCourse(c);
}

pCourse oEvent::addCourse(const oCourse &oc)
{
  if (oc.Id==0)
    return 0;
  else {
    pCourse pOld=getCourse(oc.getId());
    if (pOld)
      return 0;
  }
	Courses.push_back(oc);
  qFreeCourseId=max(qFreeCourseId, oc.getId());

  pCourse pc = &Courses.back();

  if (!pc->existInDB())
    pc->syncronize();
  courseIdIndex[oc.Id] = pc;
	return pc;
}

pRunner oEvent::addRunner(const string &pname, int ClubId, int ClassId, bool autoAdd)
{	
	oRunner r(this);
	r.Name = pname;
  r.Club = ClubId ? getClub(ClubId) : 0;
  r.Class = ClassId ? getClass(ClassId) : 0;

  pRunner pr = addRunner(r);
  if (autoAdd)
    autoAddTeam(pr);
  return pr;
}

void oEvent::autoAddTeam(pRunner pr) 
{
  if (pr && pr->Class) {
    pClass pc = pr->Class;
    if (pc->isSingleRunnerMultiStage()) {
      //Auto create corresponding team
      pTeam t = addTeam(pr->getName(), pr->getClubId(), pc->getId());
      t->setRunner(0, pr, true);
    }
  }
}

void oEvent::autoRemoveTeam(pRunner pr) 
{
  if (pr && pr->Class) {
    pClass pc = pr->Class;
    if (pc->isSingleRunnerMultiStage()) {
      if (pr->tInTeam) {
        removeTeam(pr->tInTeam->getId());
      }
    }
  }
}

pRunner oEvent::addRunner(const string &pname, const string &pclub, int ClassId, bool autoAdd)
{
	oRunner r(this);
	r.Name=pname;

	r.Club=getClubCreate(0, pclub);
  r.Class=ClassId ? getClass(ClassId):0;

	pRunner pr = addRunner(r);

  if (autoAdd)
    autoAddTeam(pr);
  return pr;
}

pRunner oEvent::addRunnerId(int rid, const string &pname, const string &pclub, int ClassId, bool autoAdd)
{
	pRunner old=getRunner(rid, 0);

	if(old) return old;

	oRunner r(this, rid);
	r.Name=pname;
	qFreeRunnerId=max(rid, qFreeRunnerId);

	r.Club=getClubCreate(0,pclub);
	r.Class=getClass(ClassId);

  pRunner pr = addRunner(r);
  if (autoAdd)
    autoAddTeam(pr);
  return pr;
}

pRunner oEvent::addRunner(const oRunner &r) 
{
  Runners.push_back(r);	
	pRunner pr=&Runners.back();

  if (pr->Card)
    pr->Card->tOwner = pr;

  if (HasDBConnection) {
    if (!pr->existInDB())
      pr->syncronize();
  }

  oe->updateTabs();
  runnerById[pr->Id] = pr;
  return pr;
}

int oEvent::getFreeCourseId()
{
	qFreeCourseId++;
	return qFreeCourseId;
}

int oEvent::getFreeControlId()
{
	qFreeControlId++;
	return qFreeControlId;
}

string oEvent::getAutoCourseName() const
{
	char bf[32];
  sprintf_s(bf, lang.tl("Bana %d").c_str(), Courses.size()+1);
	return bf;
}

int oEvent::getFreeClassId()
{
	qFreeClassId++;
	return qFreeClassId;
}

int oEvent::getFirstClassId() const {
  return Classes.empty() ? 0 : Classes.front().getId();
}

int oEvent::getFreeCardId()
{
	qFreeCardId++;
	return qFreeCardId;
}

int oEvent::getFreePunchId()
{
	qFreePunchId++;
	return qFreePunchId;
}

string oEvent::getAutoClassName() const
{
	char bf[32];
  sprintf_s(bf, 32, lang.tl("Klass %d").c_str(), Classes.size()+1);
	return bf;
}

string oEvent::getAutoTeamName() const
{
	char bf[32];
  sprintf_s(bf, 32, lang.tl("Lag %d").c_str(), Teams.size()+1);
	return bf;
}

string oEvent::getAutoRunnerName() const
{
	char bf[32];
  sprintf_s(bf, 32, lang.tl("Deltagare %d").c_str(), Runners.size()+1);
	return bf;
}

int oEvent::getFreeClubId()
{
	qFreeClubId++;
	return qFreeClubId;
}

int oEvent::getFreeRunnerId()
{
	qFreeRunnerId++;
	return qFreeRunnerId;
}

void oEvent::updateFreeId(oBase *obj)
{	
	if(typeid(*obj)==typeid(oRunner)){		
		qFreeRunnerId=max(obj->Id, qFreeRunnerId);
	}
	else if(typeid(*obj)==typeid(oClass)){
		qFreeClassId=max(obj->Id, qFreeClassId);
	}
	else if(typeid(*obj)==typeid(oCourse)){
		qFreeCourseId=max(obj->Id, qFreeCourseId);
	}
	else if(typeid(*obj)==typeid(oControl)){
		qFreeControlId=max(obj->Id, qFreeControlId);
	}
	else if(typeid(*obj)==typeid(oClub)){
		qFreeClubId=max(obj->Id, qFreeClubId);
	}
	else if(typeid(*obj)==typeid(oCard)){
		qFreeCardId=max(obj->Id, qFreeCardId);
	}
	else if(typeid(*obj)==typeid(oFreePunch)){
		qFreePunchId=max(obj->Id, qFreePunchId);
	}
	else if(typeid(*obj)==typeid(oTeam)){
		qFreeTeamId=max(obj->Id, qFreeTeamId);
	}
	/*else if(typeid(*obj)==typeid(oEvent)){
		qFree
	}*/
}

void oEvent::updateFreeId()
{
	{
		oRunnerList::iterator it;	
		qFreeRunnerId=0;
		for (it=Runners.begin(); it != Runners.end(); ++it)
			qFreeRunnerId=max(qFreeRunnerId, it->Id);
	}
	{
		oClassList::iterator it;	
		qFreeClassId=0;
		for (it=Classes.begin(); it != Classes.end(); ++it)
			qFreeClassId=max(qFreeClassId, it->Id);
	}
	{
		oCourseList::iterator it;	
		qFreeCourseId=0;
		for (it=Courses.begin(); it != Courses.end(); ++it)
			qFreeCourseId=max(qFreeCourseId, it->Id);
	}
	{
		oControlList::iterator it;	
		qFreeControlId=0;
		for (it=Controls.begin(); it != Controls.end(); ++it)
			qFreeControlId=max(qFreeControlId, it->Id);
	}
	{
		oClubList::iterator it;	
		qFreeClubId=0;
		for (it=Clubs.begin(); it != Clubs.end(); ++it)
			qFreeClubId=max(qFreeClubId, it->Id);
	}
	{
		oCardList::iterator it;	
		qFreeCardId=0;
		for (it=Cards.begin(); it != Cards.end(); ++it)
			qFreeCardId=max(qFreeCardId, it->Id);
	}
	{
		oFreePunchList::iterator it;	
		qFreePunchId=0;
		for (it=punches.begin(); it != punches.end(); ++it)
			qFreePunchId=max(qFreePunchId, it->Id);
	}

	{
		oTeamList::iterator it;	
		qFreeTeamId=0;
		for (it=Teams.begin(); it != Teams.end(); ++it)
			qFreeTeamId=max(qFreeTeamId, it->Id);
	}
}

/*
void oEvent::Notify(string s) const
{
	MessageBox(NULL, s.c_str(), Name.c_str(), MB_OK);
}

void oEvent::Notify(const char *s, int d) const
{
	char bf[256];
	sprintf_s(bf, 256, "%s %d", s, d);
	Notify(bf);
}
*/

int ClassInfo::sSortOrder=0;

bool ClassInfo::operator <(ClassInfo &ci)
{
	if(sSortOrder==0)
		return nRunners>ci.nRunners;
	else
		return FirstStart<ci.FirstStart;
}

struct ClassBlockInfo{
	int FirstControl;
	int nRunners;
	int Depth;

	int FirstStart;

	bool operator<(ClassBlockInfo &ci);
};

bool ClassBlockInfo::operator <(ClassBlockInfo &ci)
{
	return Depth<ci.Depth;
}


bool IsFree(vector<int> *StartField, int nFields, int FirstPos, int PosInterval, int nEntries, int Type)
{
	//Check if free at all...
	for(int k=0; k<nEntries; k++){
		bool HasFree=false;
		for(int f=0;f<nFields;f++){
			int t=StartField[f][FirstPos+k*PosInterval];
			if(t==0)
				HasFree=true;
			else if(t==Type)
				return false;//Type of course occupied. Cannot put it here;
		}

		if(!HasFree) return false;//No free start position.
	}

	/*
	//Check if there is another paralell and not allowed start.
	
	for(f=0;f<nFields;f++){
		if(f!=thisField){
			for(int k=0; k<nEntries; k++){
				if( StartField[f][FirstPos+k*PosInterval]== 0)
					return false;
			}
		}
	}
*/

	return true;
}

bool InsertStart(vector<int> *StartField, int nFields, int FirstPos, int PosInterval, int nEntries, int Type)
{	
	for(int k=0; k<nEntries; k++){
		bool HasFree=false;	

		for(int f=0;f<nFields && !HasFree;f++){
			if(StartField[f][FirstPos+k*PosInterval]==0){
				StartField[f][FirstPos+k*PosInterval]=Type;
				HasFree=true;
			}
		}

		if(!HasFree) return false;//No free start position. Fail.
	}

	/*
	//Check if there is another paralell and not allowed start.
	
	for(f=0;f<nFields;f++){
		if(f!=thisField){
			for(int k=0; k<nEntries; k++){
				if( StartField[f][FirstPos+k*PosInterval]== 0)
					return false;
			}
		}
	}
*/

	return true;
}


/*int BaseInterval=60;
	int MinClassInterval=60*2;
	int MaxClassInterval=60*8;
	int nFields=3; //Number of max paralell starts
*/
void oEvent::optimizeStartOrder(gdioutput &gdi, int nVacances, int BaseInterval, int MinClassInterval, int MaxClassInterval, int nFields, 	list<ClassInfo> &cInfo, bool UseFirstControl)
{
	if(Classes.size()==0)
		return;

	//ClassInfo *ci=new ClassInfo[Classes.size()];
	cInfo.clear();

	oClassList::iterator c_it;

	for (c_it=Classes.begin(); c_it != Classes.end(); ++c_it){
		ClassInfo ci;
		memset(&ci, 0, sizeof(ci));	
		ci.ClassId=c_it->getId();

		pCourse pc=c_it->getCourse();

		if(pc){
			if(UseFirstControl && pc->nControls>0)
				ci.Unique=1000000+pc->Controls[0]->Numbers[0];
			else
				ci.Unique=pc->getId();
			
			ci.CourseId=pc->getId();
		}
		else ci.Unique=ci.ClassId;

		ci.nRunners=c_it->getNumRunners()+nVacances;

		if(ci.nRunners>0)
			cInfo.push_back(ci);
	}

	if(cInfo.empty()) return; //No runners

	cInfo.sort();

	list<ClassInfo>::iterator it;


	int MaxSize=MinClassInterval*cInfo.begin()->nRunners;
	
	if(MaxSize==0) return; //No runners

	for(it=cInfo.begin(); it!=cInfo.end(); ++it){
		int quotient=MaxSize/(it->nRunners*BaseInterval);
		
		if(quotient*BaseInterval>MaxClassInterval)
			quotient=MaxClassInterval/BaseInterval;

		it->Interval=quotient;
	}

	vector<int> StartField[64];

	for(int m=0;m<nFields;m++)
		StartField[m].resize(3000, 0);

	int alternator=0;

	for(it=cInfo.begin(); it!=cInfo.end(); ++it){
		int MinPos=10000;
		int MinInterval=it->Interval;


		for(int i=MinClassInterval/BaseInterval; i<=it->Interval; i++){
			int Position=alternator%4;		
			while(!IsFree(StartField, nFields, Position, i, it->nRunners, it->Unique))
				Position++;

			if(Position<=MinPos){
				MinPos=Position;
				MinInterval=i;
			}
		}

		InsertStart(StartField, nFields, MinPos, MinInterval, it->nRunners, it->Unique);

		it->FirstStart=MinPos;
		it->Interval=MinInterval;
		alternator++;
	}

	int nr;
	int T=0;
	int sum=0;
	gdi.addString("", 1, "Antal startande per block:");
	string str="";
	int empty=4;
	while(empty>0){
		nr=0;

		for(int k=0;k<nFields;k++){
			if(StartField[k][T])
				nr++;
		}
		T++;
		sum+=nr;
		if(nr!=0) empty=4;
		else empty--;
	
		char bf[20];
		sprintf_s(bf, 20, "%d ", nr);
		str+=bf;
	}

	gdi.addStringUT(10, str);
	gdi.dropLine();
		
/*
	list<ClassInfo>::iterator ciit=cInfo.begin();


	//Tv-gaffling

	while(ciit!=cInfo.end()){
		list<ClassInfo> block;
		block.push_back(*ciit);
		++ciit;

		while(ciit!=cInfo.end() && ciit->FirstControl==block.back().FirstControl){			
			block.push_back(*ciit);			
			++ciit;
		}

		//Analysera blocket
		ClassBlockInfo cbi;

		cbi.nClasses=block.size();
		cbi.nRunners=0;

		int left=0;
		int right=0;
		
		while(!block.empty())
		{
			ClassInfo ci=block.front();
			block.pop_front();

			if(left>right)
				right+=ci.nRunners;
			else
				left+=ci.nRunners;

			cbi.nRunners+=ci.nRunners;
		}

		if(ciit!=cInfo.end()) ++ciit;
	}
*/
}

void oEvent::drawRemaining(bool useSOFTMethod, bool placeAfter)
{
  DrawType drawType = placeAfter ? remainingAfter : remainingBefore;

  for (oClassList::iterator it = Classes.begin(); it !=  Classes.end(); ++it) {
    drawList(it->getId(), 0, 0, 0, 0, useSOFTMethod, drawType);
  }
}

void oEvent::drawList(int ClassID, int leg, int FirstStart, 
                      int Interval, int Vacances, 
                      bool useSOFTMethod, DrawType drawType)
{
	autoSyncronizeLists(false);

	oRunnerList::iterator it;		

	int VacantClubId=getVacantClub();

  pClass pc=getClass(ClassID);

  if(!pc)
    throw std::exception("Klass saknas");

  if (Vacances>0 && pc->getClassType()==oClassRelay)
    throw std::exception("Vakanser stds ej i stafett");

  if (Vacances>0 && leg>0)
    throw std::exception("Det gr endast att stta in vakanser p strcka 1");
  
  vector<pRunner> runners;
  runners.reserve(Runners.size());

  if (drawType == drawAll) {
    if (leg==0) {
      //Only remove vacances on leg 0.
      vector<int> toRemove;
      //Remove old vacances
      for (it=Runners.begin(); it != Runners.end(); ++it) {
        if(it->Class && it->Class->Id==ClassID) {
          if(it->getClubId()==VacantClubId) {
            toRemove.push_back(it->getId());
          }
        }
      }

      while (!toRemove.empty()) {
        removeRunner(toRemove.back());
        toRemove.pop_back();
      }

	    while (Vacances>0) {
		    pRunner r=addRunner(_VacantName, VacantClubId, ClassID, true);
		    r->syncronize();
        r->apply(false);
		    Vacances--;
	    }
    }

	  for (it=Runners.begin(); it != Runners.end(); ++it)
      if(!it->isRemoved() && it->Class && it->Class->Id==ClassID && it->legToRun()==leg ) {
        runners.push_back(&*it);
      }
  }
  else {
    // Find first/last start in class and interval:
    int first=7*24*3600, last = 0;
    set<int> cinterval;
    int baseInterval = 10*60;

    for (it=Runners.begin(); it != Runners.end(); ++it)
      if (!it->isRemoved() && it->getClassId()==ClassID) {
        int st = it->getStartTime();
        if (st>0) {
          first = min(first, st);
          last = max(last, st);
          cinterval.insert(st);
        }
        else
          runners.push_back(&*it);
      }


    // Find start interval
    int t=0;
    for (set<int>::iterator sit = cinterval.begin(); sit!=cinterval.end();++sit) {
      if ( (*sit-t) > 0)
        baseInterval = min(baseInterval, (*sit-t));
      t = *sit;
    }
 
    if (drawType == remainingBefore)
      FirstStart = first - runners.size()*baseInterval;
    else
      FirstStart = last + baseInterval;

    Interval = baseInterval;

    if (last == 0 || FirstStart<=0 ||  baseInterval == 10*60) {
      // Fallback if incorrect specification.
      FirstStart = 3600;
      Interval = 2*60;
    }

  }

  if(runners.empty()) 
    return;

  vector<int> stimes(runners.size());	
	for(unsigned k=0;k<stimes.size(); k++)
		stimes[k]=FirstStart+Interval*k;

  if (useSOFTMethod)
    drawSOFTMethod(runners);
  else 
	  permute(stimes);

  for(unsigned k=0;k<stimes.size(); k++) {
    runners[k]->setStartTime(stimes[k]);
    runners[k]->setStartNo(k+1);
	  runners[k]->syncronize();
  }
}

void getLargestClub(map<int, vector<pRunner> > &clubRunner, vector<pRunner> &largest)
{
  size_t maxClub=0;
  for (map<int, vector<pRunner> >::iterator it = 
               clubRunner.begin(); it!=clubRunner.end(); ++it) {
    maxClub = max(maxClub, it->second.size());
  }

  for (map<int, vector<pRunner> >::iterator it = 
               clubRunner.begin(); it!=clubRunner.end(); ++it) {
    if (it->second.size()==maxClub) {
      swap(largest, it->second);
      clubRunner.erase(it);
      return;
    }
  }
}

void getRange(int size, vector<int> &p)
{
  p.resize(size);
  for (size_t k=0;k<p.size();k++)
    p[k] = k;
}

void oEvent::drawSOFTMethod(vector<pRunner> &runners, bool handleBlanks)
{
  if (runners.empty())
    return;

  //Group runners per club
  map<int, vector<pRunner> > clubRunner;

  for (size_t k=0;k<runners.size();k++) {
    int clubId = runners[k] ? runners[k]->getClubId() : -1;
    clubRunner[clubId].push_back(runners[k]);
  }
  
  vector< vector<pRunner> > runnerGroups(1);

  // Find largest club
  getLargestClub(clubRunner, runnerGroups[0]);

  int largeSize = runnerGroups[0].size();
  int ngroups = (runners.size()+largeSize-1) / largeSize;
  runnerGroups.resize(ngroups);

  while (!clubRunner.empty()) {
    // Find the smallest available group
    unsigned small = runners.size()+1;
    int cgroup = -1;
    for (size_t k=1;k<runnerGroups.size();k++)
      if (runnerGroups[k].size()<small) {
        cgroup = k;
        small = runnerGroups[k].size();
      }

    // Add the largest remaining group to the smallest.
    vector<pRunner> largest;
    getLargestClub(clubRunner, largest);
    runnerGroups[cgroup].insert(runnerGroups[cgroup].end(), largest.begin(), largest.end());
  }

  unsigned maxGroup=runnerGroups[0].size();

  //Permute the first group
  vector<int> pg(maxGroup);
  getRange(pg.size(), pg);
  permute(pg);
  vector<pRunner> pr(maxGroup);
  for (unsigned k=0;k<maxGroup;k++)
    pr[k] = runnerGroups[0][pg[k]];
  runnerGroups[0] = pr;

  //Find the largest group
  for (size_t k=1;k<runnerGroups.size();k++) 
    maxGroup = max(maxGroup, runnerGroups[k].size()); 

  if (handleBlanks) {
    //Give all groups same size (fill with 0)
    for (size_t k=1;k<runnerGroups.size();k++) 
      runnerGroups[k].resize(maxGroup); 
  }

  // Apply algorithm recursivly to groups with several clubs
  for (size_t k=1;k<runnerGroups.size();k++) 
    drawSOFTMethod(runnerGroups[k]);

  // Permute the order of groups
  vector<int> p(runnerGroups.size());
  getRange(p.size(), p);
  permute(p);

  // Write back result
  int index = 0;
  for (unsigned level = 0; level<maxGroup; level++) {
    for (size_t k=0;k<runnerGroups.size();k++) {
      int gi = p[k];
      if (level<runnerGroups[gi].size() && (runnerGroups[gi][level]!=0  || !handleBlanks)) 
        runners[index++] = runnerGroups[gi][level];
    }
  }

  if (handleBlanks)
    runners.resize(index);
}

int oEvent::getVacantClub()
{
	pClub pc=getClubCreate(cVacantId, "Vakant");
	pc->Id=cVacantId;
	return cVacantId;
}

void oEvent::drawListClumped(int ClassID, int FirstStart, int Interval, int Vacances)
{
  pClass pc=getClass(ClassID);

  if(!pc)
    throw std::exception("Klass saknas");

  if (Vacances>0 && pc->getClassType()!=oClassIndividual)
    throw std::exception("Lottningsmetoden stds ej i den hr klassen.");

	oRunnerList::iterator it;		
	int nRunners=0;

	autoSyncronizeLists(false);
	int VacantClubId=getVacantClub();

	while (Vacances>0)	{
		pRunner r=addRunner(_VacantName, VacantClubId, ClassID, true);
		r->syncronize();
		Vacances--;
	}

	for (it=Runners.begin(); it != Runners.end(); ++it)
		if(it->Class && it->Class->Id==ClassID) nRunners++;

	if(nRunners==0) return;

	int *stimes=new int[nRunners];	


	//Number of start groups
	//int ngroups=(nRunners/5)
	int ginterval;

	if(nRunners>=Interval)
		ginterval=10;
	else if(Interval/nRunners>60){
		ginterval=40;
	}
	else if(Interval/nRunners>30){
		ginterval=20;
	}
	else if(Interval/nRunners>20){
		ginterval=15;	
	}
	else if(Interval/nRunners>10){
		ginterval=1;	
	}
	else ginterval=10;

	int nGroups=Interval/ginterval+1; //15 s. per interval.
	int k;

	if(nGroups>0){

		int MaxRunnersGroup=max((2*nRunners)/nGroups, 4)+GetRandomNumber(2);
		int *sgroups=new int[nGroups];	

		for(k=0;k<nGroups; k++)
			sgroups[k]=FirstStart+((ginterval*k+2)/5)*5;

		if(nGroups>5){
			//Remove second group...
			sgroups[1]=sgroups[nGroups-1];
			nGroups--;
		}
		
		if(nGroups>9 && ginterval<60 && (GetRandomBit() || GetRandomBit() || GetRandomBit())){
			//Remove third group...
			sgroups[2]=sgroups[nGroups-1];
			nGroups--;

			if(nGroups>13 &&  ginterval<30 && (GetRandomBit() || GetRandomBit() || GetRandomBit())){
				//Remove third group...
				sgroups[3]=sgroups[nGroups-1];
				nGroups--;


				int ng=4; //Max two minutes pause
				while(nGroups>10 && (nRunners/nGroups)<MaxRunnersGroup && ng<8 && (GetRandomBit() || GetRandomBit())){
					//Remove several groups...
					sgroups[ng]=sgroups[nGroups-1];
					nGroups--;
					ng++;
				}
			}
		}

		//Permute some of the groups (not first and last group)
		if(nGroups>5){
			permute(sgroups+2, nGroups-2);

			//Remove some random groups (except first and last).
			for(k=2;k<nGroups; k++){
				if((nRunners/nGroups)<MaxRunnersGroup && nGroups>5){					
					sgroups[k]=sgroups[nGroups-1];
					nGroups--;
				}				
			}
		}

		//Premute all groups;
		permute(sgroups, nGroups);

		int *counters=new int[nGroups];
		memset(counters, 0, sizeof(int)*nGroups);
		
		stimes[0]=FirstStart;
		stimes[1]=FirstStart+Interval;

		for(k=2;k<nRunners; k++){
			int g=GetRandomNumber(nGroups);

			if(counters[g]<=2 && GetRandomBit()){
				//Prefer already large groups
				g=(g+1)%nGroups;
			}

			if(counters[g]>MaxRunnersGroup){
				g=(g+3)%nGroups;
			}

			if(sgroups[g]==FirstStart){
				//Avoid first start
				if(GetRandomBit() || GetRandomBit())
					g=(g+1)%nGroups;
			}
			
			if(counters[g]>MaxRunnersGroup){
				g=(g+2)%nGroups;
			}
			
			if(counters[g]>MaxRunnersGroup){
				g=(g+2)%nGroups;
			}

			stimes[k]=sgroups[g];
			counters[g]++;
		}

		delete[] sgroups;
		delete[] counters;

	}
	else{
		for(k=0;k<nRunners; k++) stimes[k]=FirstStart;
	}

	
	permute(stimes, nRunners);

	k=0;

	for (it=Runners.begin(); it != Runners.end(); ++it)
		if(it->Class && it->Class->Id==ClassID){
			it->setStartTime(stimes[k++]);
			it->StartNo=k;
			it->syncronize();
		}

	delete[] stimes;
}

void oEvent::permute(vector<int> &vec)
{
  if (vec.empty())
    return;

  permute(&vec.front(), vec.size());
}


void oEvent::permute(int *array, int size)
{
	int size1=size-1;

	int m=0;

	while(m<=size1) {
		if (GetRandomBit()) {
			int t=array[size1];
			array[size1]=array[m];
			size1--;
			array[m]=t;
		}
		else m++;
	}

	int p1=m;
	int p2=size-m;

	if(p1>1) permute(array, p1);

	if(p2>1) permute(&array[p1], p2);
}

pCard oEvent::allocateCard(pRunner owner)
{
	oCard c(this);
  c.tOwner=owner;
	Cards.push_back(c);
	pCard newCard=&Cards.back();
	return newCard;
}

bool oEvent::sortRunners(SortOrder so)
{
	CurrentSortOrder=so;
	Runners.sort();
	return true;
}

bool oEvent::sortTeams(SortOrder so, int leg)
{
  oTeamList::iterator it;

  if(so==ClassResult) {
	  for (it=Teams.begin(); it != Teams.end(); ++it) {
      int lg=leg;
      if(it->Class && lg>=int(it->Class->getNumStages()))
        lg=it->Class->getLastStageIndex();

		  it->_sortstatus=it->getLegStatus(lg);
		  it->_sorttime=it->getLegRunningTime(lg);

      // Ensure number of restarts has effect on result
      it->_sorttime += it->tNumRestarts*24*3600;
	  }
    Teams.sort(oTeam::compareResult);
  }
  else if(so==ClassStartTime) {
	  for (it=Teams.begin(); it != Teams.end(); ++it) {
		  it->_sortstatus=0;
      it->_sorttime=it->getLegStartTime(leg);
      if (it->_sorttime<=0)
        it->_sortstatus=1;
	  }
    Teams.sort(oTeam::compareResult);
  }


	return true;
}

void oEvent::calculateSplitResults(int controlId)
{
	oRunnerList::iterator it;

  for (it=Runners.begin(); it!=Runners.end(); ++it) {
    it->getSplitTime(controlId, it->tStatus, it->tRT);
    if(it->tStatus==StatusOK && it->Status>StatusOK)
      it->tStatus=it->Status;
  }

  Runners.sort(oRunner::sortSplit);
	int cClassId=-1;
	int cPlace=0;
	int vPlace=0;
	int cTime=0;

	for (it=Runners.begin(); it != Runners.end(); ++it){
		if(it->getClassId()!=cClassId){
			cClassId=it->getClassId();
			cPlace=0;
			vPlace=0;
			cTime=0;
      it->Class->tLegLeaderTime=9999999;
		}
	
		if (it->tStatus==StatusOK) {
			cPlace++;

      if(it->Class)
        it->Class->tLegLeaderTime=min(it->tRT, it->Class->tLegLeaderTime);


			if(it->tRT>cTime)
				vPlace=cPlace;

			cTime=it->tRT;

			it->tPlace=vPlace;
		}
		else
			it->tPlace=99000+it->Status;
	}
}

void oEvent::calculateResults()
{
	sortRunners(ClassResult);
	oRunnerList::iterator it;

	int cClassId=-1;
	int cPlace=0;
	int vPlace=0;
	int cTime=0;
  int cDuplicateLeg=0;
  bool useResults = false;

	for (it=Runners.begin(); it != Runners.end(); ++it) {
    if (it->getClassId()!=cClassId || it->tDuplicateLeg!=cDuplicateLeg) {
			cClassId=it->getClassId();
      useResults = it->Class ? !it->Class->getNoTiming() : false;
			cPlace=0;
			vPlace=0;
			cTime=0;
      cDuplicateLeg = it->tDuplicateLeg;
		}
	
		if(it->Status==StatusOK){
			cPlace++;

			if(it->getRunningTime()>cTime)
				vPlace=cPlace;

			cTime=it->getRunningTime();

      if (useResults)
			  it->tPlace = vPlace;
      else
        it->tPlace = 0;
		}
		else
			it->tPlace=99000+it->Status;
	}
}

bool oEvent::calculateTeamResults(int leg)
{
	//sortRunners(ClassResult);
	oTeamList::iterator it;
	bool hasRunner=false;

	for (it=Teams.begin(); it != Teams.end(); ++it) {
    if(leg==-1 || leg<int(it->Runners.size()) )
      hasRunner=true;

    int lg=leg;
    if(it->Class && lg>=int(it->Class->getNumStages()))
      lg=it->Class->getLastStageIndex();

		it->_sortstatus=it->getLegStatus(lg);
		it->_sorttime=it->getLegRunningTime(lg);
	}

  if(!hasRunner)
    return false;

	Teams.sort(oTeam::compareResult);

	int cClassId=0;
	int cPlace=0;
	int vPlace=0;
	int cTime=0;

	for (it=Teams.begin(); it != Teams.end(); ++it){
		if(it->Class && it->Class->Id!=cClassId){
			cClassId=it->Class->Id;
			cPlace=0;
			vPlace=0;
			cTime=0;
		}

    int sleg;
    if (leg==-1)
      sleg=it->Runners.size()-1;
    else
      sleg=leg;

    if(it->_sortstatus==StatusOK){
			cPlace++;

			if(it->_sorttime>cTime)
				vPlace=cPlace;

			cTime=it->_sorttime;

			it->_places[sleg]=vPlace;
		}
		else
      it->_places[sleg]=99000+it->_sortstatus;
	}
  return true;
}


void oEvent::calculateTeamResults()
{
	for(int i=0;i<maxRunnersTeam;i++)
		if(!calculateTeamResults(i))
      return;
}


string oEvent::getZeroTime() const
{
  return getAbsTime(0);
}

void oEvent::setZeroTime(string m)
{
	int hour=atoi(m.c_str());
	int minute=atoi(m.substr(m.find_first_of(':')+1).c_str());
	int second=atoi(m.substr(m.find_last_of(':')+1).c_str());
	DWORD nZeroTime=hour*3600+minute*60+second;

  if (nZeroTime!=ZeroTime) {
    updateChanged();
    ZeroTime=nZeroTime;
  }
}

void oEvent::setName (const string &m) 
{
  if (m!=Name) {
    Name=m;
    updateChanged();
  }
}

string oEvent::getTitleName() const
{
  if(isClient())
    return oe->getName() + " (p server)";
  else
    return oe->getName() + " (lokalt)";
}

void oEvent::setDate(const string &m)
{
 if (m!=Date) {
    Date=m;
    updateChanged();
  }
}

string oEvent::getAbsTime(DWORD time) const
{
	DWORD t=ZeroTime+time;

	if(int(t)<0)
		return "00:00:00";

	char bf[256];
	sprintf_s(bf, 256, "%02d:%02d:%02d", (t/3600)%24, (t/60)%60, t%60);
	return bf;
}

string oEvent::getAbsTimeHM(DWORD time) const
{
	DWORD t=ZeroTime+time;

	if(int(t)<0)
		return "00:00";

	char bf[256];
	sprintf_s(bf, 256, "%02d:%02d", (t/3600)%24, (t/60)%60);
	return bf;
}

//Absolute time string to absolute time int (used by cvs-parser)
int oEvent::convertAbsoluteTime(const string &m)
{
	if(m.empty() || m[0]=='-')
		return -1;

  int len=m.length();
  for (int k=0;k<len;k++) {
    BYTE b=m[k];
    if ( !(b==':' || b==' ' || (b>='0' && b<='9')) )
      return -1;
  }

	int hour=atoi(m.c_str());

	if(hour<0 || hour>23)
		return -1;

	int minute=0;
	int second=0;

	int kp=m.find_first_of(':');
	
	if(kp>0)
	{
		string mtext=m.substr(kp+1);
		minute=atoi(mtext.c_str());

		if(minute<0 || minute>60)
			minute=0;

		kp=mtext.find_last_of(':');

		if(kp>0) {
			second=atoi(mtext.substr(kp+1).c_str());

			if(second<0 || second>60)
				second=0;
		}
	}
	int t=hour*3600+minute*60+second;

	if(t<0)	return 0;

	return t;
}

int oEvent::getRelativeTime(const string &m) const
{
	int atime=convertAbsoluteTime(m);

	if(atime>=0 && atime<3600*24){
		int rtime=atime-ZeroTime;
		
		if(rtime<=0) 
      rtime+=3600*24;

    //Don't allow times just before zero time.
    if(rtime>3600*20)
      return -1;

		return rtime;
	}
	else return -1;
}

int oEvent::getRelativeTimeFrom12Hour(const string &m) const
{
	int atime=convertAbsoluteTime(m);

	if (atime>=0 && atime<3600*24) {
		int rtime=atime-(ZeroTime % (3600*12));
		
		if(rtime<=0) 
      rtime+=3600*12;

    //Don't allow times just before zero time.
    if(rtime>3600*20)
      return -1;

		return rtime;
	}
	else return -1;
}


void oEvent::removeRunner(int Id)
{
	oRunnerList::iterator it;	

  pRunner r=getRunner(Id, 0);

  if (r==0)
    return;

  r = r->tParentRunner ? r->tParentRunner : r;

  //Remove a singe runner team
  autoRemoveTeam(r);

  set<int> toRemove;
  for (size_t k=0;k<r->multiRunner.size();k++)
    if (r->multiRunner[k])
      toRemove.insert(r->multiRunner[k]->getId());
  toRemove.insert(Id);

	for (it=Runners.begin(); it != Runners.end();){
    oRunner &cr = *it;
    if(toRemove.count(cr.getId())> 0){
			if(msRemove) 
        msRemove(&cr);
      toRemove.erase(cr.getId());
      runnerById.erase(cr.getId());
      
      // Reset team runner (this should not happen)
      if (it->tInTeam) {
        if (it->tInTeam->Runners[it->tLeg]==&*it)
          it->tInTeam->Runners[it->tLeg] = 0;
      }
      
      oRunnerList::iterator next = it;
      ++next;

			Runners.erase(it);
      if (toRemove.empty()) {
        oe->updateTabs();
			  return;
      }
      else
      it = next;
		}
    else
      ++it;
	}
  oe->updateTabs();
}

void oEvent::removeCourse(int Id)
{
	oCourseList::iterator it;	

	for (it=Courses.begin(); it != Courses.end(); ++it){
		if(it->Id==Id){
			if(msRemove) 
        msRemove(&*it);
      
			Courses.erase(it);
      courseIdIndex.erase(Id);
			return;
		}
	}	
}

void oEvent::removeClass(int Id)
{
	oClassList::iterator it;	

	for (it=Classes.begin(); it != Classes.end(); ++it){
		if(it->Id==Id){
			if(msRemove) msRemove(&*it);
			Classes.erase(it);
      updateTabs();
			return;
    }
	}	
}

void oEvent::removeControl(int Id)
{
	oControlList::iterator it;	

	for (it=Controls.begin(); it != Controls.end(); ++it){
		if(it->Id==Id){
			if(msRemove) msRemove(&*it);
			Controls.erase(it);
			return;
		}
	}	
}

void oEvent::removeClub(int Id)
{
	oClubList::iterator it;	

	for (it=Clubs.begin(); it != Clubs.end(); ++it){
		if (it->Id==Id) {
			if(msRemove) msRemove(&*it);
			Clubs.erase(it);
      clubIdIndex.erase(Id);
			return;
		}
	}	
}

bool oEvent::isCourseUsed(int Id) const
{
	oClassList::const_iterator it;	

	for (it=Classes.begin(); it != Classes.end(); ++it){
		if(it->isCourseUsed(Id))
			return true;
	}	

	oRunnerList::const_iterator rit;

	for (rit=Runners.begin(); rit != Runners.end(); ++rit){
		pCourse pc=rit->getCourse();
		if(pc && pc->Id==Id)	
			return true;
	}
	return false;
}

bool oEvent::isClassUsed(int Id) const
{
	//Search runners
  oRunnerList::const_iterator it;
	for (it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->getClassId()==Id)	
			return true;
	}	

	//Search teams
	oTeamList::const_iterator tit;
	for (tit=Teams.begin(); tit != Teams.end(); ++tit){		
		if(tit->getClassId()==Id)	
			return true;
	}
	return false;
}

bool oEvent::isClubUsed(int Id) const
{
	//Search runners
  oRunnerList::const_iterator it;
	for (it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->getClubId()==Id)	
			return true;
	}	

	//Search teams
	oTeamList::const_iterator tit;
	for (tit=Teams.begin(); tit != Teams.end(); ++tit){		
		if(tit->getClubId()==Id)	
			return true;
	}

	return false;
}

bool oEvent::isRunnerUsed(int Id) const
{
	//Search teams
  oTeamList::const_iterator tit;
	for (tit=Teams.begin(); tit != Teams.end(); ++tit){		
    if(tit->isRunnerUsed(Id)) {	
      if (tit->Class && tit->Class->isSingleRunnerMultiStage())
        //Don't report single-runner-teams as blocking
        continue;
      return true; 
    }
	}

	return false;
}

bool oEvent::isControlUsed(int Id) const
{
	oCourseList::const_iterator it;	

	for (it=Courses.begin(); it != Courses.end(); ++it){
		
		for(int i=0;i<it->nControls;i++)
			if(it->Controls[i] && it->Controls[i]->Id==Id)
				return true;
	}	
	return false;
}

bool oEvent::classHasResults(int Id) const
{
	oRunnerList::const_iterator it;

	for (it=Runners.begin(); it != Runners.end(); ++it)
		if(it->getClassId()==Id && (it->getCard() || it->FinishTime))	
      return true;
	
	return false;
}

bool oEvent::classHasTeams(int Id) const
{
  oTeamList::const_iterator it;

  for (it=Teams.begin(); it != Teams.end(); ++it)
    if(it->getClassId()==Id)
      return true;
  
  return false;
}

/*
void oEvent::generateStartlist(gdioutput &gdi, int leg, GUICALLBACK cb, 
                               const vector<int> &classId)
{
	sortRunners(ClassStartTime);

	oRunnerList::iterator it;

	//BIB, START, NAME, CLUB, RANK, SI
	int dx[6]={0, 0, 70, 300, 470, 470};

	bool withrank=hasRank();
	bool withbib=hasBib();
	int i;

	if(withrank) dx[5]+=50;
	if(withbib) for(i=1;i<6;i++) dx[i]+=40;
		
	int y=gdi.getCY(); 
	int x=gdi.getCX();
	int lh=gdi.getLineHeight();
 
	gdi.addStringUT(2, lang.tl("Startlista") + MakeDash(" - ") + getName());
	y+=lh/2;

	
	gdi.addStringUT(1, getDate());
	y+=lh;
	char bf[256];

	y+=(3*lh)/2;

	
	int Id=0;

	for(it=Runners.begin(); it != Runners.end(); ++it){	
    if (it->isRemoved() || it->legToRun()!=leg)
      continue;
		if(classId.size()>0 && count(classId.begin(), classId.end(), it->getClassId())==0)
			continue;

		if(it->getClassId()!=Id){
			Id=it->getClassId();
			y+=lh/2;

			gdi.addStringUT(y, x+dx[0], 1, it->getClass());

			y+=lh+lh/3;	
		}

		oDataInterface DI=it->getDI();

		if(withbib){
			char bib[8];
      int b=it->getBib();

			if(b>0){
				sprintf_s(bib, "%d", b);
				gdi.addStringUT(y, x+dx[0], 0, bib);
			}
		}

		gdi.addStringUT(y, x+dx[1], 0, it->getStartTimeS());

		_itoa_s(it->Id, bf, 256, 10);
		gdi.addstring(bf, y, x+dx[2], 0, it->Name, dx[3]-dx[2]-4, cb);
		gdi.addString("", y, x+dx[3], 0, it->getClub(), dx[4]-dx[3]-4);
		
		if(it->CardNo>0){
			sprintf_s(bf, 256, "%d", it->CardNo);
			gdi.addString("", y, x+dx[5], 0, bf);
		}

		if(withrank){
			int r=DI.getInt("Rank");
			if(r>0){
				gdi.addString("", y, x+dx[4], 0, FormatRank(r));
			}
		}

		y+=lh;
	}
	gdi.UpdateScrollbars();
}*/

void oEvent::generateInForestList(gdioutput &gdi, GUICALLBACK cb, GUICALLBACK cb_nostart)
{
  sortTeams(ClassStartTime, 0);
	int y=gdi.getCY();
	int x=gdi.getCX();
	int lh=gdi.getLineHeight();

  oTeamList::iterator it;
	gdi.addStringUT(2, lang.tl("Kvar-i-skogen") + MakeDash(" - ") + getName());
	y+=lh/2;

	gdi.addStringUT(1, getDate());

  gdi.dropLine();

  y+=3*lh;
  int id=0;
  int nr=0;
  char bf[128];

  for(it=Teams.begin(); it!=Teams.end(); ++it) {
    if(it->Status==StatusUnknown) {

      if (id!=it->getClassId()) {
        if(nr>0) {
          sprintf_s(bf, lang.tl("Antal: %d").c_str(), nr);
          gdi.addStringUT(y, x, 0, bf);
          y+=lh;
          nr=0;
        }
        y+=lh;
        id=it->getClassId();
        gdi.addStringUT(y, x, 1, it->getClass());
        y+=lh;
      }
      gdi.addStringUT(y, x, 0, it->getClass());
      nr++;
      gdi.addStringUT(y, x+100, 0, it->Name);
      y+=lh;
    }
  } 

  if(nr>0) {
    sprintf_s(bf, "Antal: %d", nr);
    gdi.addStringUT(y, x, 0, bf);
    y+=lh;
  }

  {
	  sortRunners(ClassStartTime);

	  oRunnerList::iterator it;

	  int dx[4]={0, 70, 350, 470};
	  int y=gdi.getCY();
	  int x=gdi.getCX();
	  int lh=gdi.getLineHeight();

    y+=lh;
	  char bf[256];

	  y=gdi.getCY();

	  for(it=Runners.begin(); it != Runners.end(); ++it){	
      if (it->skip())
        continue;

		  if(it->Status==StatusUnknown) {
			  gdi.addStringUT(y, x+dx[0], 0, it->getStartTimeS());
			  gdi.addStringUT(y, x+dx[1], 0, it->Name+" ("+it->getClub()+")", dx[2]-dx[1]-4, cb);
			  _itoa_s(it->Id, bf, 256, 10);
			  //gdi.addString(bf, y, x+dx[2], 0, "Startade inte", dx[3]-dx[2]-4, cb_nostart);

			  gdi.addStringUT(y, x+dx[3], 0, it->getClass());
			  y+=lh;
		  }
	  }
  }
	gdi.UpdateScrollbars();
}

void oEvent::generateMinuteStartlist(gdioutput &gdi)
{
	sortRunners(SortByStartTime);
	oRunnerList::iterator it;

	int dx[4]={0, 70, 300, 470};
	int y=gdi.getCY();
	int x=gdi.getCX();	
	int lh=gdi.getLineHeight();

  vector<int> blocks;
  vector<string> starts;
  getStartBlocks(blocks, starts);

  char bf[256];
  for (size_t k=0;k<blocks.size();k++) {
    gdi.dropLine();
    gdi.addStringUT(boldLarge, lang.tl("Minutstartlista") +  MakeDash(" - ") + getName());
    if (!starts[k].empty()) {
      sprintf_s(bf, lang.tl("%s, block: %d").c_str(), starts[k].c_str(), blocks[k]);
      gdi.addStringUT(fontMedium, bf);
    }
    else if (blocks[k]!=0) {
      sprintf_s(bf, lang.tl("Startblock: %d").c_str(),  blocks[k]);
      gdi.addStringUT(fontMedium, bf);
    }
   
    y = gdi.getCY();
  	
	  int LastStartTime=-1;
	  for (it=Runners.begin(); it != Runners.end(); ++it) {		
      if (it->Class && it->Class->getBlock()!=blocks[k])
        continue;

      if (!it->Class && blocks[k]!=0)
        continue;

		  if (LastStartTime!=it->StartTime) {
			  y+=lh/2;
			  LastStartTime=it->StartTime;
        gdi.addStringUT(y, x+dx[0], boldText, it->getStartTimeS());
  		  y+=lh;
		  }

      if (it->getCardNo()>0)
        gdi.addStringUT(y, x+dx[0], fontMedium, itos(it->getCardNo()));
		  
      gdi.addStringUT(y, x+dx[1], fontMedium, it->Name, dx[2]-dx[1]-4);
		  gdi.addStringUT(y, x+dx[2], fontMedium, it->getClub(), dx[3]-dx[2]-4);
		  gdi.addStringUT(y, x+dx[3], fontMedium, it->getClass());
		  y+=lh;
	  }
  }
	gdi.refresh();
}

void oEvent::generateResultlistFinishTime(gdioutput &gdi, bool PerClass, GUICALLBACK cb)
{
	calculateResults();

	if(PerClass)
		sortRunners(SortByClassFinishTime);
	else
		sortRunners(SortByFinishTime);

	oRunnerList::iterator it;

	int dx[5]={0, 40, 300, 490, 550};
	int y=gdi.getCY();
	int x=gdi.getCX();
	int lh=gdi.getLineHeight();

	gdi.addStringUT(2, lang.tl("Resultat") + MakeDash(" - ") + getName());
	y+=lh/2;

	
	gdi.addStringUT(1, getDate());
	y+=lh;
	y+=(3*lh)/2;

	int FirstStart=getFirstStart();
	
	int Id=0;
	int place=0;
	int order=1;
	int lasttime=0;
	char bf[256];

	for(it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->Status!=0)	{
			if(PerClass && it->getClassId()!=Id){
				//Next class
				Id=it->getClassId();
				y+=lh/2;

				gdi.addStringUT(y, x+dx[0], 1, it->getClass());

				FirstStart=getFirstStart(Id);
				
				order=1;
				lasttime=0;
				y+=lh+lh/3;	
			}

			if(it->FinishTime>lasttime) {
				place=order;
				lasttime=it->FinishTime;
			}
			order++;

			if(it->getAge()<100) {
				RECT rc;
				rc.left=x;
				rc.right=x+dx[3];
				rc.top=y;
				rc.bottom=y+lh;
				gdi.addRectangle(rc);
			}	
			gdi.addStringUT(y, x+dx[1], 0, it->Name, dx[2]-dx[1]-4, cb);
			gdi.addStringUT(y, x+dx[2], 0, it->getClub(), dx[3]-dx[2]-4);
		
			
			if(it->getStatus()==StatusOK){
				_itoa_s(place, bf, 256, 10);
				gdi.addStringUT(y, x+dx[0], 0, string(bf)+".");

				char tstr[32];
				int rt=it->FinishTime-FirstStart;

				if(rt>=3600)
					sprintf_s(tstr, 32, "%d:%02d:%02d", rt/3600,(rt/60)%60, rt%60);
				else
					sprintf_s(tstr, 32, "%d:%02d", (rt/60), rt%60);

				gdi.addStringUT(y, x+dx[3], 0, string(tstr));

				if(PerClass)
					gdi.addStringUT(y, x+dx[4], 0, string("(")+it->getPlaceS()+", "+it->getRunningTimeS()+")");
				else
					gdi.addStringUT(y, x+dx[4], 0, string("(")+it->getClass()+": "+it->getPlaceS()+", "+it->getRunningTimeS()+")");

			}
			else
				gdi.addStringUT(y, x+dx[3], 0, it->getStatusS());

			y+=lh;
		}
	}
	//gdi.UpdatePos(x+700,y+40,0,0);
	gdi.UpdateScrollbars();
}

void oEvent::generateResultlistFinishTime(const string &file, bool PerClass)
{
	calculateResults();

	if(PerClass)
		sortRunners(SortByClassFinishTime);
	else
		sortRunners(SortByFinishTime);


	oRunnerList::iterator it;

	ofstream fout(file.c_str());

	fout << "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n" <<
		"\"http://www.w3.org/TR/html4/loose.dtd\">\n\n";
	
	fout << "<html>\n<head>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n";

	fout << "<title>" << "Resultat &ndash; " + getName() << "</title>\n</head>\n\n";

	fout << "<body bgcolor=\"#FFFFFF\" text=\"#000000\" link=\"#FF0000\" vlink=\"#AA0000\" alink=\"#0000DD\">\n";


	fout << "<h2>" << "Resultat &ndash; " + getName() << "</h2>\n";

	char bf1[256];
	char bf2[256];
    GetTimeFormat(LOCALE_USER_DEFAULT, 0, NULL, NULL, bf2, 256);
	GetDateFormat(LOCALE_USER_DEFAULT, 0, NULL, NULL, bf1, 256);

	fout << "Skapad av <i>MeOS</i>: " << bf1 << " "<< bf2 << "\n";

	int FirstStart=getFirstStart();

	int Id=0;
	int place=0;
	int order=1;
	int lasttime=0;
	char bf[256];

	fout << "<table>\n";
	
	fout << "<tr>";
	fout << "<td width=\"20\">&nbsp;</td>";
	fout << "<td width=\"200\">&nbsp;</td>";
	fout << "<td width=\"200\">&nbsp;</td>";
	fout << "<td width=\"80\">&nbsp;</td>";
	fout << "</tr>";

	for(it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->Status!=0){
			if(PerClass && it->getClassId()!=Id){

				//Next class
				Id=it->getClassId();
				
				if(Id)
					fout << "<tr><td colspan=\"4\">&nbsp;</td></tr>\n";

				Id=it->getClassId();
		
				fout << "<tr><td colspan=\"4\"><b>" << it->getClass() << "</b></td></tr>\n";
	
				FirstStart=getFirstStart(Id);
				
				order=1;
				lasttime=0;			
			}

			if(it->FinishTime>lasttime)
			{
				place=order;
				lasttime=it->FinishTime;
			}
			order++;
	

			fout << "<tr>\n";

			if(it->getStatus()==StatusOK){
				_itoa_s(place, bf, 256, 10);
				fout << "<td>" <<  string(bf)+"." << "</td>\n";
			}
			else
				fout << "<td>" << "&nbsp;" << "</td>\n";

			fout << "<td>" <<  it->Name << "</td>\n";
			fout << "<td>" <<  it->getClub() << "</td>\n";

	
			if(it->getStatus()==StatusOK)	{
				
				char tstr[32];
				int rt=it->FinishTime-FirstStart;

				if(rt>=3600)
					sprintf_s(tstr, 32, "%d:%02d:%02d", rt/3600,(rt/60)%60, rt%60);
				else
					sprintf_s(tstr, 32, "%d:%02d", (rt/60), rt%60);

				fout << "<td>" <<  tstr << "</td>\n";

				
				
				if(PerClass)
					fout << "<td>" << string("(")+"<b>"+it->getPlaceS()+"</b>, "+it->getRunningTimeS()+")</td>\n";
				else
					fout << "<td>" << string("(")+it->getClass()+": <b>"+it->getPlaceS()+"</b>, "+it->getRunningTimeS()+")</td>\n";



			}			else
			{
				fout << "<td>" <<  it->getStatusS() << "</td>\n";
				fout << "<td>" <<  string("(")+it->getClass()+")" << "</td>\n";
			}
			fout << "</tr>\n";
		
		}
	}
	fout << "</table>\n";

	fout << "</body></html>\n";
}


bool oEvent::empty() const
{
  return Name.empty();
}

void oEvent::clearListedCmp()
{
  cinfo.clear();
}


bool oEvent::enumerateCompetitions(const char *file, const char *filetype)
{
	WIN32_FIND_DATA fd;
	
	char dir[MAX_PATH];
	//char buff[MAX_PATH];
	char FullPath[MAX_PATH];
	
	strcpy_s(dir, MAX_PATH, file);

	if(dir[strlen(file)-1]!='\\')
		strcat_s(dir, MAX_PATH, "\\");
	
	strcpy_s(FullPath, MAX_PATH, dir);
	
	strcat_s(dir, MAX_PATH, filetype);
	
	HANDLE h=FindFirstFile(dir, &fd);
	
	if(h==INVALID_HANDLE_VALUE)
		return false;
	
	bool more=true;//=FindNextFile(h, &fd);
	
	int id=1;

	list<CompetitionInfo> saved;

	list<CompetitionInfo>::iterator it;

	for (it=cinfo.begin(); it!=cinfo.end(); ++it) {
		if(it->Server.length()>0)
			saved.push_back(*it);
	}

	cinfo=saved;
	

	while (more) {
		if(fd.cFileName[0]!='.') //Avoid .. and .
		{
			char FullPathFile[MAX_PATH];
			strcpy_s(FullPathFile, MAX_PATH, FullPath);
			strcat_s(FullPathFile, MAX_PATH, fd.cFileName);

			CompetitionInfo ci;

			ci.FullPath=FullPathFile;
			ci.Name="";
			ci.Date="2007-01-01";
			ci.Id=id++;

			SYSTEMTIME st;
			FileTimeToSystemTime(&fd.ftLastWriteTime, &st);
			ci.Modified=convertSystemTime(st);
			xmlparser xp;

			xp.GetOnlyData();
			xp.Read(FullPathFile);
			//xmlobject *xo=xp.GetObject("meosdata");
			const xmlobject *date=xp.GetObject("Date");

			if(date) ci.Date=date->Get();

			const xmlobject *name=xp.GetObject("Name");

			if(name) ci.Name=name->Get();

			cinfo.push_front(ci);
		}
		more=FindNextFile(h, &fd)!=0;
	}
	
	FindClose(h);

	return true;
}


bool oEvent::fillCompetitions(gdioutput &gdi, const string &name, int type)
{	
	cinfo.sort();
	cinfo.reverse();
  list<CompetitionInfo>::iterator it;

	gdi.clearList(name);
	string b;
	char bf[128];
	for (it=cinfo.begin(); it!=cinfo.end(); ++it) {
		if (it->Server.length()==0) {
      if (type==0 || type==1) {
			  sprintf_s(bf, 128, "[%s] %s", it->Date.c_str(), it->Name.c_str()); 
			  gdi.addItem(name, bf, it->Id);
      }
		}
		else if (type==0 || type==2) {
      if (type==0)
        sprintf_s(bf, 128, lang.tl("Server: [%s] %s").c_str(), it->Date.c_str(), it->Name.c_str()); 
      else
        sprintf_s(bf, 128, "[%s] %s", it->Date.c_str(), it->Name.c_str()); 
      
			gdi.addItem(name, bf, 10000000+it->Id);
		}
	}

	return true;
}

void tabAutoKillMachines();

void oEvent::checkDB()
{
  if(HasDBConnection) {
    vector<string> err;
    int k=checkChanged(err);

#ifdef _DEBUG
    if(k>0) {
      char bf[256];
      sprintf_s(bf, "Databasen innehller %d osynkroniserade ndringar.", k);
      string msg(bf);
      for(int i=0;i < min<int>(err.size(), 10);i++)
        msg+=string("\n")+err[i];

      MessageBox(0, msg.c_str(), "Varning/Fel", MB_OK);
    }
#endif   
  }
  updateTabs();
}

void clearSpeakerData();

void oEvent::clear()
{
  checkDB();

  if(HasDBConnection && msMonitor)
    msMonitor(0);

	HasDBConnection=false;
  tOriginalName.clear();
  Id=0;

	ZeroTime=0;
	Name.clear();

  //Make sure no daemon is hunting us.
  tabAutoKillMachines();

	//Order of destruction is extreamly important...
  runnerById.clear();
  Runners.clear();
	Teams.clear();

	Classes.clear();
	Courses.clear();
	courseIdIndex.clear();

  Controls.clear();
	
	Cards.clear();
	Clubs.clear();
  clubIdIndex.clear();
  
  punchIndex.clear();
	punches.clear();
	
	updateFreeId();
	
	strcpy_s(CurrentNameId, "");
	strcpy_s(CurrentFile, "");

  sqlCounterRunners=0;
	sqlCounterClasses=0;
	sqlCounterCourses=0;
	sqlCounterControls=0;
	sqlCounterClubs=0;
	sqlCounterCards=0;
	sqlCounterPunches=0;
	sqlCounterTeams=0;

  sqlUpdateControls.clear();
  sqlUpdateCards.clear();
  sqlUpdateClubs.clear();
  sqlUpdateClasses.clear();
  sqlUpdateCourses.clear();
  sqlUpdateRunners.clear();
  sqlUpdatePunches.clear();
  sqlUpdateTeams.clear();
  
  //Reset speaker data structures.
#ifndef MEOSDB  
  clearSpeakerData();
#else
  throw std::exception("Bad call");
#endif
}

bool oEvent::deleteCompetition()
{
	if (!empty() && !HasDBConnection) {
		rename(CurrentFile, (string(CurrentFile)+".removed").c_str());
		return true;
	}
	else return false;
}

void oEvent::newCompetition(const string &name)
{
	clear();
	ZeroTime=0;

	SYSTEMTIME st;
	GetLocalTime(&st);

	char bf[64];
	sprintf_s(bf, 64, "%d-%02d-%02d", st.wYear, st.wMonth, st.wDay);
	
	Date=bf;
	ZeroTime=st.wHour*3600;

	Name=name;
  tOriginalName.clear();
	oEventData->initData(oData, sizeof(oData));

  GetEventDI().setInt("CardFee", 25);
  GetEventDI().setInt("EliteFee", 120);
  GetEventDI().setInt("EntryFee", 80);
  GetEventDI().setInt("YouthFee", 40);
    
	char file[260];
	char filename[64];
	sprintf_s(filename, 64, "meos_%d%02d%02d_%02d%02d%02d_%X.meos", 
		st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);	
	
	//strcpy_s(CurrentNameId, filename);
	GetUserFile(file, filename);

	strcpy_s(CurrentFile, MAX_PATH, file);
	_splitpath_s(CurrentFile, NULL, 0, NULL,0, CurrentNameId, 64, NULL, 0);
  int i=0;
  while (CurrentNameId[i]) {
    if(CurrentNameId[i]=='.') { 
      CurrentNameId[i]=0;
      break;
    }
    i++;
  }

  oe->updateTabs();
}

void oEvent::exportTeamSplits(xmlparser &xml)
{
  bool ClassStarted=false;
	int Id=-1;
  bool skipClass=false;

  sortTeams(ClassResult, -1);
  for(oTeamList::iterator it=Teams.begin(); it != Teams.end(); ++it) {				
		if(it->getClassId()!=Id) {
			if(ClassStarted) xml.EndTag();
			
      if(!it->Class || it->Class->getClassType()!=oClassRelay) {
        skipClass=true;
        ClassStarted=false;
        continue;
      }

      skipClass=false;
			xml.StartTag("ClassResult");
			ClassStarted=true;
			Id=it->getClassId();

			xml.Write("ClassShortName", it->getClass());
		}

    if(skipClass)
      continue;

    xml.StartTag("TeamResult"); {
      /*    
    <TeamName>Sundsvalls OK</TeamName>
    <BibNumber></BibNumber>
    <StartTime>
     <Clock clockFormat="HH:MM:SS">10:00:00</Clock>
    </StartTime>
    <FinishTime>
     <Clock clockFormat="HH:MM:SS">11:45:52</Clock>
    </FinishTime>
    <Time>
     <Time timeFormat="HH:MM:SS">01:45:52</Time>
    </Time>
    <ResultPosition>1</ResultPosition>
    <TeamStatus value="OK"></TeamStatus>
      */
      xml.Write("TeamName", it->getName());
      xml.Write("BibNumber", it->getStartNo());

		  xml.StartTag("StartTime");
			  xml.Write("Clock", "clockFormat", "HH:MM:SS", it->getStartTimeS());
		  xml.EndTag();
		  xml.StartTag("FinishTime");
			  xml.Write("Clock", "clockFormat", "HH:MM:SS", it->getFinishTimeS());
		  xml.EndTag();
		
      xml.Write("Time", "timeFormat", "HH:MM:SS", formatTimeHMS(it->getRunningTime()));
		  xml.Write("ResultPosition", it->getPlaceS());
		  xml.Write("TeamStatus", "value", it->getIOFStatusS());
		
      for (size_t k=0;k<it->Runners.size();k++) {
        if(!it->Runners[k])
          continue;
        pRunner r=it->Runners[k];

        xml.StartTag("PersonResult"); {

          string sex;

          if(sex.empty())  xml.StartTag("Person");
          else xml.StartTag("Person", "sex", sex); {

            xml.StartTag("PersonName");
              xml.Write("Family", r->getFamilyName());
			        xml.Write("Given", "sequence", "1", r->getGivenName());					
		        xml.EndTag();

		        xml.Write("PersonId", it->Id);
          } xml.EndTag();
      			
          xml.StartTag("Club"); {
				    xml.Write("ClubId", 1);
            xml.Write("ShortName", r->getClub());
				    xml.Write("CountryId", "value", r->getDI().getString("Nationality"));
          } xml.EndTag();

          xml.StartTag("Result"); {
            xml.Write("TeamSequence", k+1);
			      xml.StartTag("StartTime");
				      xml.Write("Clock", "clockFormat", "HH:MM:SS", formatTimeHMS(r->getStartTime()));
			      xml.EndTag();
			      xml.StartTag("FinishTime");
              xml.Write("Clock", formatTimeHMS(r->getFinishTime()));
			      xml.EndTag();
    			
			      xml.Write("Time", formatTimeHMS(r->getRunningTime()));
			      xml.Write("ResultPosition", r->getPlaceS());

			      xml.Write("CompetitorStatus", "value", r->getIOFStatusS());
    			
            const vector<int> &sp=r->getSplitTimes();

			      pCourse pc=r->getCourse();

            if(pc) {
              xml.StartTag("CourseVariation");
              xml.Write("CourseVariationId", pc->getId());
              xml.Write("CourseLength", "unit", "m", pc->getLengthS());
              xml.EndTag();
            }
			      pCourse pcourse=pc;
			      if(pcourse && r->getStatus()>0 && r->getStatus()!=StatusDNS) {
				      char bf[16];

              for (int k=0;k<pcourse->nControls;k++)	{
					      _itoa_s(k+1, bf, 16, 10);
					      xml.StartTag("SplitTime", "sequence", bf);
					      xml.Write("ControlCode", pcourse->Controls[k]->Numbers[0]);
					      if(unsigned(k)<sp.size() && sp[k]>0)
                  xml.Write("Time", "clockFormat", "HH:MM:SS", getAbsTime((sp[k]-r->StartTime)-ZeroTime));
					      else
					        xml.Write("Time", "--:--:--");

					      xml.EndTag();
              } //Loop over splits
			      }
          } xml.EndTag();
        } xml.EndTag();
      } //Loop over team members
      
    }  xml.EndTag();    
  }
}

bool oEvent::exportIOFSplits(const char *file)
{
	xmlparser xml;

	if(!xml.OpenOutput(file, "ResultList"))
    throw std::exception((string("Kan inte ppna: ")+file).c_str());

  reEvaluateAll(true);
	calculateResults();
  calculateTeamResults();

	xml.Write("IOFVersion", "version", "2.0.3");

  xml.StartTag("Event");
    xml.Write("Name", getName());
    xml.StartTag("StartDate");
      xml.Write("Date", "dateFormat", "YYYY-MM-DD", getDate());
    xml.EndTag();
  xml.EndTag();

	bool ClassStarted=false;
	int Id=-1;
  bool skipClass=false;

	for(oTeamList::iterator it=Teams.begin(); it != Teams.end(); ++it){				
		if (it->Runners.size()>=2 && it->Runners[0] && it->Runners[1]) {
			if(it->getClassId()!=Id) {
				if(ClassStarted) xml.EndTag();
				
        if(!it->Class || it->Class->getClassType()!=oClassPatrol) {
          skipClass=true;
          ClassStarted=false;
          continue;
        }

        skipClass=false;
				xml.StartTag("ClassResult");
				ClassStarted=true;
				Id=it->getClassId();

				xml.Write("ClassShortName", it->getClass());
			}

      if(skipClass)
        continue;

			xml.StartTag("PersonResult");

      string sex;

      if(sex.empty())
        xml.StartTag("Person");
      else
        xml.StartTag("Person", "sex", sex);

				  xml.StartTag("PersonName");
            xml.Write("Family", it->Runners[0]->getFamilyName());
					  xml.Write("Given", "sequence", "1", it->Runners[0]->getGivenName());					
				  xml.EndTag();

				  xml.Write("PersonId", it->Id);
        xml.EndTag();
				
				xml.StartTag("Club");
					xml.Write("ClubId", 1);
          xml.Write("ShortName", it->Runners[1]->getName());
					//xml.Write(<CountryId value="SWE"/>
					xml.Write("CountryId", "value", it->getDI().getString("Nationality"));
				xml.EndTag();

				xml.StartTag("Result");
					xml.StartTag("StartTime");
						xml.Write("Clock", "clockFormat", "HH:MM:SS", it->getStartTimeS());
					xml.EndTag();
					xml.StartTag("FinishTime");
						xml.Write("Clock", it->getLegFinishTimeS(-1));
					xml.EndTag();
				
					xml.Write("Time", it->getLegRunningTimeS(-1));
					xml.Write("ResultPosition", it->getLegPlaceS(-1));

					xml.Write("CompetitorStatus", "value", it->Runners[0]->getIOFStatusS());
				
          const vector<int> &sp=it->Runners[0]->getSplitTimes();

					pCourse pc=it->Runners[0]->getCourse();
					if(pc)	xml.Write("CourseLength", "unit", "m", pc->getLengthS());

					pCourse pcourse=pc;
					if(pcourse && it->getLegStatus(-1)>0 && it->getLegStatus(-1)!=StatusDNS) {
						char bf[16];

            for (int k=0;k<pcourse->nControls;k++)	{
							_itoa_s(k+1, bf, 16, 10);
							xml.StartTag("SplitTime", "sequence", bf);
							xml.Write("ControlCode", pcourse->Controls[k]->Numbers[0]);
							if(unsigned(k)<sp.size() && sp[k]>0)
								xml.Write("Time", "clockFormat", "HH:MM:SS", getAbsTime((sp[k]-it->StartTime)-ZeroTime));
							else
							  xml.Write("Time", "--:--:--");
	
							xml.EndTag();
						}
					}
				xml.EndTag();
			xml.EndTag();
		}
	}


  exportTeamSplits(xml);

  skipClass=false;
  Id=-1;

	for (oRunnerList::iterator it=Runners.begin(); 
        it != Runners.end(); ++it) {		
		//if(it->Status!=0)
		{
			if(it->getClassId()!=Id) {
				if(ClassStarted) xml.EndTag();
		
        if(!it->Class || it->Class->getClassType()==oClassPatrol
                      || it->Class->getClassType()==oClassRelay) {
          skipClass=true;
          ClassStarted=false;
          continue;
        }

				xml.StartTag("ClassResult");
				ClassStarted=true;
        skipClass=false;
				Id=it->getClassId();

				xml.Write("ClassShortName", it->getClass());
			}
      
      if(skipClass)
        continue;

			xml.StartTag("PersonResult");

      string sex=it->getDI().getString("Sex");

      if(sex=="K" || sex=="W")
        sex="F";

      if(sex.empty())
        xml.StartTag("Person");
      else
        xml.StartTag("Person", "sex", sex);
				  xml.StartTag("PersonName");
            xml.Write("Family", it->getFamilyName());
					  xml.Write("Given", "sequence", "1", it->getGivenName());					
				  xml.EndTag();

				  xml.Write("PersonId", it->Id);
        xml.EndTag();
				
				xml.StartTag("Club");
					xml.Write("ClubId", it->getClubId());
					xml.Write("ShortName", it->getClub());
					//xml.Write(<CountryId value="SWE"/>
					xml.Write("CountryId", "value", it->getDI().getString("Nationality"));
				xml.EndTag();

				xml.StartTag("Result");
					xml.StartTag("StartTime");
						xml.Write("Clock", "clockFormat", "HH:MM:SS", it->getStartTimeS());
					xml.EndTag();
					xml.StartTag("FinishTime");
						xml.Write("Clock", it->getFinishTimeS());
					xml.EndTag();
				
					xml.Write("Time", it->getRunningTimeS());
					xml.Write("ResultPosition", it->getPlaceS());

					xml.Write("CompetitorStatus", "value", it->getIOFStatusS());
				
          const vector<int> &sp=it->getSplitTimes();

//          int splits=it->GetSplitTimes(sp);

					pCourse pc=it->getCourse();
					if(pc)	xml.Write("CourseLength", "unit", "m", pc->getLengthS());

					pCourse pcourse=it->getCourse();
					if (pcourse && it->getStatus()>0 && it->getStatus()!=StatusDNS) {
						char bf[16];

            for(int k=0;k<pcourse->nControls;k++)	{
							_itoa_s(k+1, bf, 16, 10);
							xml.StartTag("SplitTime", "sequence", bf);
							xml.Write("ControlCode", pcourse->Controls[k]->Numbers[0]);
							if(unsigned(k)<sp.size() && sp[k]>0)
								xml.Write("Time", "clockFormat", "HH:MM:SS", getAbsTime((sp[k]-it->StartTime)-ZeroTime));
							else
							  xml.Write("Time", "--:--:--");
	
							xml.EndTag();
						}
					}
				xml.EndTag();
			xml.EndTag();
		}
	}

	xml.EndTag();
	
  if (!xml.EndTag())
    throw std::exception("Kunde inte spara strcktidsfilen.");

	xml.CloseOut();
	return true;
}

void oEvent::reEvaluateClass(int ClassId, bool DoSync)
{
	if(DoSync)
		autoSyncronizeLists(false);

	oRunnerList::iterator it;

  pClass pc=getClass(ClassId);
  if(pc)
    pc->resetLeaderTime();

	vector<int> mp;
	for(it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->Class && it->Class->getId()==ClassId){
			it->evaluateCard(mp);

			if(DoSync)
				it->syncronize(true);

			mp.clear();
		}
	}
}

void oEvent::reEvaluateCourse(int CourseId, bool DoSync)
{
	oRunnerList::iterator it;
	
	if(DoSync)
		autoSyncronizeLists(false);

	vector<int> mp;
  set<int> classes;
	for(it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->getCourse() && it->getCourse()->getId()==CourseId){
      classes.insert(it->getClassId());
		}
	}

  for(set<int>::iterator sit=classes.begin();sit!=classes.end(); ++sit)
    reEvaluateClass(*sit, false);
}

void oEvent::reEvaluateAll(bool DoSync)
{	
	if(DoSync)
		autoSyncronizeLists(false);

  for(oClassList::iterator it=Classes.begin();it!=Classes.end();++it)  
    it->resetLeaderTime();

  oRunnerList::iterator it;
	vector<int> mp;
	for (it=Runners.begin(); it != Runners.end(); ++it) {		
		if (!it->isRemoved()) {
			it->evaluateCard(mp);

			if(DoSync)
				it->syncronize(true);

			mp.clear();
		}
	}
}

void oEvent::reEvaluateChanged()
{	
  for(oClassList::iterator it=Classes.begin();it!=Classes.end();++it)  
    it->resetLeaderTime();

  oRunnerList::iterator it;
	vector<int> mp;
	for (it=Runners.begin(); it != Runners.end(); ++it) {		
		if (!it->isRemoved()) {
      if (it->wasSQLChanged())
			  it->evaluateCard(mp);
      else
        it->storeTimes();

  		mp.clear();
		}
	}
}

void oEvent::reCalculateLeaderTimes(int classId)
{
  for (oClassList::iterator it=Classes.begin(); it != Classes.end(); ++it) {	
    if (!it->isRemoved() && (classId==it->getId() || classId==0))
      it->resetLeaderTime();
  }
	
  for (oRunnerList::iterator it=Runners.begin(); it != Runners.end(); ++it) {		
		if (!it->isRemoved() && (classId==0 || classId==it->getClassId())) {
      it->storeTimes();
		}
	}  
}

string oEvent::getCurrentTimeS()
{
	SYSTEMTIME st;
	GetLocalTime(&st);

	char bf[64];
	sprintf_s(bf, 64, "%02d:%02d:%02d", st.wHour, st.wMinute, st.wSecond);
	return bf;
}

pClass oEvent::findBestClass(const SICard &card, int &Distance) const
{
	Distance=-1000;
	oClassList::const_iterator it;	

	const oClass *rclass=0; //Best match class

	for (it=Classes.begin(); it != Classes.end(); ++it) {
		pCourse pc=it->getCourse();

		if(pc) {
			int d=pc->distance(card);

			if(d>=0) {
				if(Distance<0) Distance=1000;

				if(d<Distance) {
					Distance=d;
					rclass=&*it;
				}
			}
			else {
				if(Distance<0 && d>Distance) {
					Distance=d;
					rclass=&*it;
				}
			}
		}
	}

	return const_cast<pClass>(rclass);
}

void oEvent::convertTimes(SICard &sic) const
{

	if(sic.CheckPunch.Code!=-1){
		if(sic.CheckPunch.Time<ZeroTime)
			sic.CheckPunch.Time+=(24*3600);

		sic.CheckPunch.Time-=ZeroTime;
	}

	if(sic.StartPunch.Code!=-1){
		if(sic.StartPunch.Time<ZeroTime)
			sic.StartPunch.Time+=(24*3600);
		
		sic.StartPunch.Time-=ZeroTime;
	}

	for(unsigned k=0;k<sic.nPunch;k++){
		if(sic.Punch[k].Code!=-1){
			if(sic.Punch[k].Time<ZeroTime)
				sic.Punch[k].Time+=(24*3600);
			
			sic.Punch[k].Time-=ZeroTime;
		}
	}
	
	
	if(sic.FinishPunch.Code!=-1){
		if(sic.FinishPunch.Time<ZeroTime)
			sic.FinishPunch.Time+=(24*3600);
	
		sic.FinishPunch.Time-=ZeroTime;
	}	
}

int oEvent::getFirstStart(int ClassId)
{
	oRunnerList::iterator it=Runners.begin();
	int MinTime=3600*24;

	while(it!=Runners.end()){
		if(ClassId==0 || it->getClassId()==ClassId)
			if(it->StartTime<MinTime && it->Status==StatusOK && it->StartTime!=0)
				MinTime=it->StartTime;

		++it;
	}
	
	if(MinTime==3600*24)
		MinTime=0;

	return MinTime;
}

bool oEvent::exportONattResults(gdioutput &gdi, const string &file)
{
	ofstream fout(file.c_str());

	calculateResults();

	sortRunners(SortByClassFinishTime);

	oRunnerList::iterator it;

	int Id=0;

	int ClassNr=1;
	bool warn=false;
	char bf[256];

	int FirstStart=0;

	for(it=Runners.begin(); it != Runners.end(); ++it){		
		if(it->Status!=0 && it->Status!=StatusDNS){
			if(it->getClassId()!=Id){
				//Next class
				Id=it->getClassId();			
				FirstStart=getFirstStart(Id);
				
				if(it->getClass()=="Lng")
					ClassNr=1;
				else if(it->getClass()=="Mellan")
					ClassNr=2;
				else if(it->getClass()=="Kort")
					ClassNr=3;
				else{
					if(!warn)
						gdi.alert("Varning: Funktionen anpassad fr klassnamn Lng, Mellan och Kort");
					warn=true;
					ClassNr=Id;
				}
			}

			fout << ClassNr << ";";

			int st=it->StartTime-FirstStart;
			sprintf_s(bf, 64, "%d.%02d;", st/60, st%60);
			fout << bf;

			if(it->getStatus()==StatusOK){
				int ft=it->FinishTime-FirstStart;
				sprintf_s(bf, 64, "%d.%02d;", ft/60, ft%60);
				fout << bf;
			}
			else fout << ";";

			fout << it->getName() << ";";
			fout << it->getClub() << endl;

		}
	}

	return true;
}


bool oEvent::hasRank() const
{	
	oRunnerList::const_iterator it;	

	for (it=Runners.begin(); it != Runners.end(); ++it){
		if(it->getDCI().getInt("Rank")>0)
			return true;
	}	
	return false;
}

bool oEvent::hasBib() const
{
	oRunnerList::const_iterator it;	

	for (it=Runners.begin(); it != Runners.end(); ++it){
		if(it->getBib()>0)
			return true;
	}	
	return false;
}


void oEvent::addBib(int ClassId, int leg, int FirstNumber)
{
  if( !classHasTeams(ClassId) ) {
	  sortRunners(ClassStartTime);
	  oRunnerList::iterator it;

	  if(FirstNumber>0) {
		  for (it=Runners.begin(); it != Runners.end(); ++it) {	
        if( (ClassId==0 || it->getClassId()==ClassId) && it->legToRun()==leg) {
          it->setBib(FirstNumber++);
			  }
		  }
	  }
	  else {
		  for(it=Runners.begin(); it != Runners.end(); ++it){	
        if(ClassId==0 || it->getClassId()==ClassId) {
				  it->getDI().setInt("Bib", 0);//Update only bib
        }
		  }
	  }
  }
  else {
    sortTeams(ClassStartTime, 0);

    oTeamList::iterator it;
    if(FirstNumber>0) {
		  for (it=Teams.begin(); it != Teams.end(); ++it) {	
			  if(ClassId==0 || it->getClassId()==ClassId)
          it->setBib(FirstNumber++);         
		  }
	  }
	  else {
		  for(it=Teams.begin(); it != Teams.end(); ++it){	
			  if(ClassId==0 || it->getClassId()==ClassId)
				  it->getDI().setInt("Bib", 0); //Update only bib
		  }
	  }
  }
}

void oEvent::checkOrderIdMultipleCourses(int ClassId)
{
	sortRunners(ClassStartTime);
	int order=1;
	oRunnerList::iterator it;

	//Find first free order
	for(it=Runners.begin(); it != Runners.end(); ++it){	
		if(ClassId==0 || it->getClassId()==ClassId){
			it->syncronize();//Ensure we are up-to-date
			order=max(order, it->StartNo);
		}
	}

	//Assign orders
	for(it=Runners.begin(); it != Runners.end(); ++it){	
		if(ClassId==0 || it->getClassId()==ClassId)
			if(it->StartNo==0){
				it->StartNo=++order;
				it->updateChanged(); //Mark as changed.
				it->syncronize(); //Sync!
			}
	}
}

void oEvent::adjustTeamMultiRunners()
{
  oTeamList::iterator it;
  for (it=Teams.begin(); it != Teams.end(); ++it) {	
    it->adjustMultiRunners(true);       
  }
}

void oEvent::generatePreReport(gdioutput &gdi)
{
	CurrentSortOrder=SortByName;
	Runners.sort();
	
	oRunnerList::iterator r_it;
	oTeamList::iterator t_it;

	//BIB, START, NAME, CLUB, RANK, SI
	int dx[6]={0, 0, 70, 300, 470, 470};

	bool withrank=hasRank();
	bool withbib=hasBib();
	int i;

	if(withrank) dx[5]+=50;
	if(withbib) for(i=1;i<6;i++) dx[i]+=40;
		
	int y=gdi.getCY(); 
	int x=gdi.getCX();
	int lh=gdi.getLineHeight();
 
	gdi.addStringUT(2, lang.tl("Rapport infr") +": " + getName());
	
	gdi.addStringUT(1, getDate());
	gdi.dropLine();
	char bf[256];

	list<pRunner> no_card;
	list<pRunner> no_start;
	list<pRunner> no_class;
	list<pRunner> no_course;
	list<pRunner> no_club;

	for (r_it=Runners.begin(); r_it != Runners.end(); ++r_it){	
		if( r_it->getClubId()!=cVacantId) {
			if(r_it->getCardNo()==0)
				no_card.push_back(&*r_it);
			if(r_it->getStartTime()==0)
				no_start.push_back(&*r_it);
			if(r_it->getClubId()==0)
				no_club.push_back(&*r_it);
		}

		if(r_it->getClassId()==0)
			no_class.push_back(&*r_it);
		else if(r_it->getCourse()==0)
			no_course.push_back(&*r_it);
	}
	
	list<pRunner> si_duplicate;

	if(Runners.size()>1){
		Runners.sort(oRunner::CompareSINumber);
		
		r_it=Runners.begin();		
		while (++r_it != Runners.end()){	
			oRunnerList::iterator r_it2=r_it;
			r_it2--;

			if(r_it2->getCardNo() && r_it2->getCardNo()==r_it->getCardNo()){
				
				if(si_duplicate.size()==0 || si_duplicate.back()->getId()!=r_it2->getId())
					si_duplicate.push_back(&*r_it2);

				si_duplicate.push_back(&*r_it);
			}
		}
	}

	const string Ellipsis="[ ... ]";

 	sprintf_s(bf, lang.tl("Lpare utan klass: %d.").c_str(), no_class.size());
	gdi.addStringUT(1, bf);
	i=0;

	while(!no_class.empty() && ++i<20){
		pRunner r=no_class.front();
		no_class.pop_front();
		sprintf_s(bf, "%s (%s)", r->getName().c_str(), r->getClub().c_str());
		gdi.addStringUT(0, bf);
	}
	if(!no_class.empty()) gdi.addStringUT(1, Ellipsis);

	gdi.dropLine();
  sprintf_s(bf, lang.tl("Lpare utan bana: %d.").c_str(), no_course.size());
	gdi.addStringUT(1, bf);
	i=0;

	while(!no_course.empty() && ++i<20){
		pRunner r=no_course.front();
		no_course.pop_front();
		sprintf_s(bf, "%s: %s (%s)", r->getClass().c_str(), r->getName().c_str(), r->getClub().c_str());
		gdi.addStringUT(0, bf);
	}
	if(!no_course.empty()) gdi.addStringUT(1, Ellipsis);


	gdi.dropLine();
	sprintf_s(bf, lang.tl("Lpare utan klubb: %d.").c_str(), no_club.size());
	gdi.addStringUT(1, bf);
	i=0;

	while(!no_club.empty() && ++i<20){
		pRunner r=no_club.front();
		no_club.pop_front();
		sprintf_s(bf, "%s: %s", r->getClass().c_str(), r->getName().c_str());
		gdi.addStringUT(0, bf);
	}
	if(!no_club.empty()) gdi.addStringUT(1, Ellipsis);

	gdi.dropLine();
	sprintf_s(bf, lang.tl("Lpare utan starttid: %d.").c_str(), no_start.size());
	gdi.addStringUT(1, bf);
	i=0;

	while(!no_start.empty() && ++i<20){
		pRunner r=no_start.front();
		no_start.pop_front();
		sprintf_s(bf, "%s: %s (%s)", r->getClass().c_str(), r->getName().c_str(), r->getClub().c_str());
		gdi.addStringUT(0, bf);
	}
	if(!no_start.empty()) gdi.addStringUT(1, Ellipsis);

	gdi.dropLine();
	sprintf_s(bf, lang.tl("Lpare utan SI-bricka: %d.").c_str(), no_card.size());
	gdi.addStringUT(1, bf);
	i=0;

	while(!no_card.empty() && ++i<20){
		pRunner r=no_card.front();
		no_card.pop_front();
		sprintf_s(bf, "%s: %s (%s)", r->getClass().c_str(), r->getName().c_str(), r->getClub().c_str());
		gdi.addStringUT(0, bf);
	}
	if(!no_card.empty()) gdi.addStringUT(1, Ellipsis);


	gdi.dropLine();
	sprintf_s(bf, lang.tl("SI-dubbletter: %d.").c_str(), si_duplicate.size());
	gdi.addStringUT(1, bf);
	i=0;

	while(!si_duplicate.empty() && ++i<20){
		pRunner r=si_duplicate.front();
		si_duplicate.pop_front();
		sprintf_s(bf, "%s: %s (%s) SI=%d", r->getClass().c_str(), 
              r->getName().c_str(), r->getClub().c_str(), r->getCardNo());
		gdi.addStringUT(0, bf);
	}
	if(!si_duplicate.empty()) gdi.addStringUT(1, Ellipsis);

	
	//List all competitors not in a team.
	for (r_it=Runners.begin(); r_it != Runners.end(); ++r_it)
		r_it->_objectmarker=0; 
	
	for (t_it=Teams.begin(); t_it != Teams.end(); ++t_it){		
		pClass pc=getClass(t_it->getClassId());
		 
		if(pc){
			for(unsigned i=0;i<pc->getNumStages();i++){
				pRunner r=t_it->getRunner(i);
				if(r) r->_objectmarker++;
			}
		}		
	}

	gdi.dropLine();
	gdi.addString("", 1, "Lpare som frekommer i flera lag (=katastrof!)");

	for (r_it=Runners.begin(); r_it != Runners.end(); ++r_it){
		if(r_it->_objectmarker>1){
			sprintf_s(bf, "%s: %s (%s)", r_it->getClass().c_str(), r_it->getName().c_str(), r_it->getClub().c_str());
			gdi.addStringUT(0, bf);
		}
	}
	sortRunners(ClassStartTime);	
	
	gdi.dropLine();
	gdi.addString("", 1, "Individuella deltagare");
	
	y=gdi.getCY(); 
	int tab[5]={0, 100, 350, 420, 550};
	for (r_it=Runners.begin(); r_it != Runners.end(); ++r_it){
		if(r_it->_objectmarker==0){ //Only consider runners not in a team.
			gdi.addStringUT(y, x+tab[0], 0, r_it->getClass(), tab[1]-tab[0]);
			gdi.addStringUT(y, x+tab[1], 0, r_it->getName()+" ("+r_it->getClub()+")", tab[2]-tab[1]);
			gdi.addStringUT(y, x+tab[2], 0, itos(r_it->getCardNo()), tab[3]-tab[2]);
			gdi.addStringUT(y, x+tab[3], 0, r_it->getCourseName(), tab[4]-tab[3]);
			y+=lh;	
			pCourse pc=r_it->getCourse();
			
			if(pc){
				gdi.addStringUT(y, x+tab[1], 0, pc->getCourseReadable());
				y+=lh;
			}

		}
	}

	gdi.dropLine();
	gdi.addStringUT(1, "Lag");
	
	for (t_it=Teams.begin(); t_it != Teams.end(); ++t_it){		
		pClass pc=getClass(t_it->getClassId());
	
		gdi.addStringUT(0, t_it->getClass() + ": " + t_it->getName() + "  " +t_it->getStartTimeS());

		if(pc){
			for(unsigned i=0;i<pc->getNumStages();i++){
				pRunner r=t_it->getRunner(i);
				if(r){
					gdi.addStringUT(0, r->getName()+ " SI: " +itos(r->getCardNo()));

					pCourse pcourse=r->getCourse();
			
					if(pcourse){
						gdi.addStringUT(0, pcourse->getCourseReadable());
						y+=lh;
					}
				}
				else
					gdi.addString("", 0, "Lpare saknas");
			}
		}	
		gdi.dropLine();
	}

	gdi.UpdateScrollbars();
}

void oEvent::fillStatus(gdioutput &gdi, const string &name)
{
	gdi.clearList(name);
  gdi.addItem(name, "-", StatusUnknown);
	gdi.addItem(name, lang.tl("OK"), StatusOK);
	gdi.addItem(name, lang.tl("Ej start"), StatusDNS);
	gdi.addItem(name, lang.tl("Felst."), StatusMP);
	gdi.addItem(name, lang.tl("Utg."), StatusDNF);
	gdi.addItem(name, lang.tl("Disk."), StatusDQ);
	gdi.addItem(name, lang.tl("Maxtid"), StatusMAX);
}

int oEvent::getPropertyInt(const char *name, int def)
{
  if(eventProperties.count(name)==1)
    return atoi(eventProperties[name].c_str());
  else return def;
}

const string &oEvent::getPropertyString(const char *name, const string &def)
{
  if(eventProperties.count(name)==1)
    return eventProperties[name];
  else return def;
}

void oEvent::setProperty(const char *name, int prop)
{
  eventProperties[name]=itos(prop);
}

void oEvent::setProperty(const char *name, const string &prop)
{
  eventProperties[name]=prop;
}


bool compareClubClassTeamName(const oRunner &a, const oRunner &b)
{
  if(a.Club==b.Club) {
    if(a.Class==b.Class) {
      if(a.tInTeam==b.tInTeam)
        return a.Name<b.Name;
      else if(a.tInTeam) {
        if(b.tInTeam)
          return a.tInTeam->getStartNo() < b.tInTeam->getStartNo();
        else return false;
      }
      return b.tInTeam!=0;
    }
    else 
      return a.getClass()<b.getClass();
  }
  else
    return a.getClub()<b.getClub();
  
}

void oEvent::assignCardInteractive(gdioutput &gdi, GUICALLBACK cb)
{
  gdi.fillDown();
  gdi.dropLine(1);
  gdi.addString("", 2, "Tilldelning av hyrbrickor");

  Runners.sort(compareClubClassTeamName);

  oRunnerList::iterator it;
  pClub lastClub=0;

  int k=0;
	for (it=Runners.begin(); it != Runners.end(); ++it) {	  

    if(it->skip() || it->getCardNo() || it->isVacant() || it->needNoCard())
      continue;

    if(it->Club!=lastClub) {
      lastClub=it->Club;
      gdi.dropLine(0.5);
      gdi.addString("", 1, it->getClub());
    }

    string r;
    if(it->Class)
      r+=it->getClass()+", ";

    if(it->tInTeam) {
      char bf[1024];
      sprintf_s(bf, "%d %s, ", it->tInTeam->getStartNo(), 
                    it->tInTeam->getName().c_str());
      r+=bf;
    }

    r+=it->Name+":";
    gdi.fillRight();
    gdi.pushX();
    gdi.addStringUT(0, r);
    char id[24];
    sprintf_s(id, "*%d", k++);

    gdi.addInput(max(gdi.getCX(), 450), gdi.getCY()-4, 
                 id, "", 10, cb).setExtra(LPVOID(it->getId()));

    gdi.popX();
    gdi.dropLine(1.6);
    gdi.fillDown();
	}

  if (k==0) 
    gdi.addString("", 0, "Ingen lpare saknar bricka");
}

void oEvent::calcUseStartSeconds()
{
  tUseStartSeconds=false;
  oRunnerList::iterator it;	
	for (it=Runners.begin(); it != Runners.end(); ++it)
    if( it->getStartTime()>0 && 
        (it->getStartTime()+ZeroTime)%60!=0 ) {
      tUseStartSeconds=true;
      return;
    } 
}

const string &oEvent::formatStatus(RunnerStatus status) 
{
  const static string stats[7]={"?", "OK", "Ej start", "Felst.", "Utg.", "Disk.", "Maxtid"};
	switch(status)	{
	case StatusOK:
		return lang.tl(stats[1]);
	case StatusDNS:
		return lang.tl(stats[2]);
	case StatusMP:
		return lang.tl(stats[3]);
	case StatusDNF:
		return lang.tl(stats[4]);
	case StatusDQ:
		return lang.tl(stats[5]);
	case StatusMAX:
		return lang.tl(stats[6]);
	}

	return stats[0];
}

void oEvent::generateTestCard(SICard &sic) const
{
  sic.clear();

  if(Runners.empty())
    return;

  oRunnerList::const_iterator it;
  
  int rNo=rand()%Runners.size();
  
  it=Runners.begin();

  while(rNo-->0)
    ++it;
  
  const oRunner *r=0;
  while(r==0 && it!=Runners.end()) {
    if(it->CardNo && !it->Card && it->getCourse()) {
      // For team runners, we require start time to get right order
      if(!it->tInTeam || it->StartTime>0)
        r=&*it;
    }
    ++it;
  }
  --it;
  while(r==0 && it!=Runners.begin()) {
    if(it->CardNo && !it->Card && it->getCourse()) {
      // For team runners, we require start time to get right order
      if(!it->tInTeam || it->StartTime>0)
        r=&*it;
    }
    --it;
  }

  if (r) {
    pCourse pc=r->getCourse();

    if(pc) {
      sic.CardNumber=r->CardNo;
      int s = sic.StartPunch.Time = r->StartTime>0 ? r->StartTime+ZeroTime : ZeroTime+3600+rand()%(3600*3);
      int f=sic.FinishPunch.Time=s+(30+pc->getLength()/200)*60+rand()%(60*20);

      if(rand()%40==0 || r->StartTime>0)
        sic.StartPunch.Code=-1;
      
      if(rand()%50==0)
        sic.FinishPunch.Code=-1;

      if(rand()%40==0)
        sic.CardNumber++;

      sic.nPunch=0;
      double dt=1./double(pc->nControls+1);

      for(int k=0;k<pc->nControls;k++) {      
        if (rand()%100!=50) {
          sic.Punch[sic.nPunch].Code=pc->getControl(k)->Numbers[0];
          double cc=(k+1)*dt;
		      sic.Punch[sic.nPunch].Time=int(f*cc+s*(1.-cc));	
          sic.nPunch++;
        }
      }
    }
  }
}

pCourse oEvent::generateTestCourse(int nCtrl)
{
  char bf[64];
  static int sk=0;
  sprintf_s(bf, lang.tl("Bana %d").c_str(), ++sk);
  pCourse pc=addCourse(bf, 4000+(rand()%1000)*10);

  int i=0;
  for (;i<nCtrl/3;i++) 
    pc->addControl(rand()%(99-32)+32);
  
  i++;
  pc->addControl(50)->setName("Radio 1");

  for (;i<(2*nCtrl)/3;i++) 
    pc->addControl(rand()%(99-32)+32);

  i++;
  pc->addControl(150)->setName("Radio 2");

  for (;i<nCtrl-1;i++) 
    pc->addControl(rand()%(99-32)+32);
  pc->addControl(100)->setName("Frvarning");

  return pc;
}


pClass oEvent::generateTestClass(int nlegs, int nrunners, 
                               char *name, const string &start)
{   
  pClass cls=addClass(name);

  if (nlegs==1 && nrunners==1) {
    int nCtrl=rand()%15+5;
    if(rand()%10==1)
      nCtrl+=rand()%40;
    cls->setCourse(generateTestCourse(nCtrl));
  }
  else if (nlegs==1 && nrunners==2) {
    setupRelay(*cls, PPatrol, 2, start);
    int nCtrl=rand()%15+10;
    pCourse pc=generateTestCourse(nCtrl);
    cls->addStageCourse(0, pc->getId());
    cls->addStageCourse(1, pc->getId());
  }
  else if (nlegs>1 && nrunners==2) {
    setupRelay(*cls, PTwinRelay, nlegs, start);
    int nCtrl=rand()%8+10;
    int cid[64];
    for (int k=0;k<nlegs;k++)
      cid[k]=generateTestCourse(nCtrl)->getId();

    for (int k=0;k<nlegs;k++)
      for (int j=0;j<nlegs;j++)
        cls->addStageCourse(k, cid[(k+j)%nlegs]);
  }
  else if (nlegs>1 && nrunners==nlegs) {
    setupRelay(*cls, PRelay, nlegs, start);
    int nCtrl=rand()%8+10;
    int cid[64];
    for (int k=0;k<nlegs;k++)
      cid[k]=generateTestCourse(nCtrl)->getId();

    for (int k=0;k<nlegs;k++)
      for (int j=0;j<nlegs;j++)
        cls->addStageCourse(k, cid[(k+j)%nlegs]);
  }
  else if (nlegs>1 && nrunners==1) {
    setupRelay(*cls, PHunting, 2, start);
    cls->addStageCourse(0, generateTestCourse(rand()%8+10)->getId());
    cls->addStageCourse(1, generateTestCourse(rand()%8+10)->getId());
  }
  return cls;
}


void oEvent::generateTestCompetition(int nClasses, int nRunners, 
                                     bool generateTeams)
{ 
  oe->newCompetition("!TESTTVLING");
  oe->setZeroTime("05:00:00");
  bool useSOFTMethod = true;
  vector<string> gname;
  //gname.reserve(RunnerDatabase.size());  
  vector<string> fname;
  //fname.reserve(RunnerDatabase.size());

  runnerDB->getAllNames(gname, fname);
/*  oRunnerList::iterator it;
  for(it=RunnerDatabase.begin(); it!=RunnerDatabase.end(); ++it){
    if (!it->getGivenName().empty())
      gname.push_back(it->getGivenName());

    if (!it->getFamilyName().empty())
      fname.push_back(it->getFamilyName());
  }
*/
  int nClubs=30;
  char bf[128];
  int startno=1;
  for(int k=0;k<nClubs;k++) {
    sprintf_s(bf, "Klubb %d", k);
    addClub(bf, k+1);
  }

  int now=getRelativeTime(getCurrentTimeS());
  string start=getAbsTime(now+60*3-(now%60));

  for (int k=0;k<nClasses;k++) {
    pClass cls=0;

    if (!generateTeams) {
      int age=0;
      if(k<7)
        age=k+10;
      else if(k==7)
        age=18;
      else if(k==8)
        age=20;
      else if(k==9)
        age=21;
      else 
        age=30+(k-9)*5;
        
      sprintf_s(bf, "HD %d", age);
      cls=generateTestClass(1,1, bf, "");
    }
    else {
      sprintf_s(bf, "Klass %d", k);
      int nleg=k%5+1;
      int nrunner=k%3+1;
      nrunner = nrunner == 3 ? nleg:nrunner;

      nleg=3;
      nrunner=3;
      cls=generateTestClass(nleg, nrunner, bf, start);
    }

    int classesLeft=(nClasses-k);
    int nRInClass=nRunners/classesLeft;

    if(classesLeft>2 && nRInClass>3)
      nRInClass+=int(nRInClass*0.7)-rand()%int(nRInClass*1.5);
    
    if (cls->getNumDistinctRunners()==1) {
      for (int i=0;i<nRInClass;i++) {
        pRunner r=addRunner(gname[rand()%gname.size()]+" "+fname[rand()%fname.size()],
          rand()%nClubs+1, cls->getId(), true);

        r->setStartNo(startno++);
        r->setCardNo(500001+Runners.size()*97+rand()%97, false);
        r->apply(false);
      }
      nRunners-=nRInClass;
      if (k%5!=5)
        drawList(cls->getId(), 0, getRelativeTime(start), 10, 3, useSOFTMethod, drawAll); 
      else
        cls->Name += " ppen";
    }
    else {
      int dr=cls->getNumDistinctRunners();
      for (int i=0;i<nRInClass;i++) {
        pTeam t=addTeam("Lag " + fname[rand()%fname.size()], rand()%nClubs+1, cls->getId());
        t->setStartNo(startno++);

        for (int j=0;j<dr;j++) {
          pRunner r=addRunner(gname[rand()%gname.size()]+" "+fname[rand()%fname.size()], 0, 0, true);
          r->setCardNo(500001+Runners.size()*97+rand()%97, false);
          t->setRunner(j, r, false);
        }
      }
      nRunners-=nRInClass;

      if( cls->getStartType(0)==STDrawn )
        drawList(cls->getId(), 0, getRelativeTime(start), 20, 3, useSOFTMethod, drawAll); 
    }
  }
}

void oEvent::getFreeImporter(oFreeImport &fi)
{
  if(!fi.isLoaded())
    fi.load();

  fi.init(Runners, Clubs, Classes);
}


void oEvent::fillFees(gdioutput &gdi, const string &name)
{	
	gdi.clearList(name);

  char bf[24];
  int f;
  f=GetEventDI().getInt("EliteFee");
  sprintf_s(bf, "%d kr", f);
  gdi.addItem(name, bf, f);

  f=GetEventDI().getInt("EntryFee");
  sprintf_s(bf, "%d kr", f);
  gdi.addItem(name, bf, f);

  f=GetEventDI().getInt("YouthFee");
  sprintf_s(bf, "%d kr", f);
  gdi.addItem(name, bf, f);
}

void oEvent::fillLegNumbers(gdioutput &gdi, const string &name)
{	
	oClassList::iterator it;	
  syncronizeList(oLClassId);

	gdi.clearList(name);
	string b;
	char bf[64];
  int maxmulti=0;
	for (it=Classes.begin(); it != Classes.end(); ++it) 
    if (!it->Removed) 
      maxmulti=max<unsigned>(maxmulti, it->getNumStages());
 
  if (maxmulti==0)
    gdi.disableInput(name.c_str());
  else {
    
    for (int k=0;k<maxmulti;k++) {
      sprintf_s(bf, lang.tl("Strcka %d").c_str(), k+1);
      gdi.addItem(name, bf, k);
    }
    gdi.addItem(name, lang.tl("Sista strckan"), 1000);

    gdi.selectFirstItem(name);
  }
}

void oEvent::generateTableData(const string &tname, Table *table) 
{
  if (tname == "runners") {
    generateRunnerTableData(table);
    return;
  }
  else if (tname == "classes") {
    generateClassTableData(table);
    return;
  }
  else if (tname == "clubs") {
    generateClubTableData(table);
    return;
  }
  else if (tname == "cards") {
    generateCardTableData(table);
    return;
  }
  throw std::exception("Wrong table name");
}

void oEvent::generateFees(const string &classType, const string &lateLimit,
                          int BaseFee, int HighFee)
{
  oRunnerList::iterator it;

	for (it=Runners.begin(); it != Runners.end(); ++it) {
    if (!it->skip()) {
      pClass pc = it->Class;
      oDataInterface di = it->getDI();
      if (di.getInt("Fee")>0)
        continue;
      
      if ( (pc && pc->getType()==classType) || (pc==0 && classType.empty())) {
        string date = di.getDate("EntryDate");

        if (date <= lateLimit)
          di.setInt("Fee", BaseFee);
        else
          di.setInt("Fee", HighFee);
      }
    }
	}
}

#ifndef MEOSDB 
void createTabs(bool onlyMain, bool skipTeam, bool skipSpeaker, 
                bool skipEconomy, bool skipLists, bool skipRunners, bool skipControls);

void oEvent::updateTabs() const
{
  bool hasTeam = !Teams.empty();

  for (oClassList::const_iterator it = Classes.begin(); 
                  !hasTeam && it!=Classes.end(); ++it) {
    if (it->getNumStages()>1)
      hasTeam = true;
  }

  bool hasRunner = !Runners.empty() || !Classes.empty();
  bool hasLists = !Runners.empty() || !Teams.empty();

  createTabs(empty(), !hasTeam, GetEventDCI().getInt("UseSpeaker")==0, 
              !useEconomy(), !hasLists, !hasRunner, Controls.empty());
}

#else
void oEvent::updateTabs() const 
{
}
#endif

bool oEvent::useEconomy() const
{
  return GetEventDCI().getInt("UseEconomy")!=0;
}

bool oEvent::hasMultiRunner() const
{
  for (oClassList::const_iterator it = Classes.begin(); it!=Classes.end(); ++it) {
    if(it->hasMultiCourse() && it->getNumDistinctRunners() != it->getNumStages())
      return true;
  }

  return false;
}
