/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

// oEvent.cpp: implementation of the oEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <vector>
#include "oEvent.h"
#include "gdioutput.h"
#include "oDataContainer.h"
#include "csvparser.h"
#include "oFreeImport.h"

#include "random.h"
#include "SportIdent.h"
#include "RunnerDB.h"

#include "meos.h"

#include <io.h>
#include <fcntl.h>


string conv_is(int i)
{
	char bf[256];
//	if(i==0)
//		return "";
	//else
	 if(_itoa_s(i, bf, 10)==0)
		return bf;
	 return "";
}


int ConvertStatusToOE(int i)
{
	switch(i)
	{
	    case StatusOK:
			return 0;	    	
	    case StatusDNS:  // Ej start
			return 1;	    	
	    case StatusDNF:  // Utg.
			return 2;	    	
	    case StatusMP:  // Felst.
			return 3;
	    case StatusDQ: //Disk
			return 4;  	    	
	    case StatusMAX: //Maxtid 
			return 5;
	} 
	return 1;//Ej start...?!
}

/*
	enum {OEstno=0, OEcard=1, OEid=2, OEsurname=3, OEfirstname=4,
			OEbirth=5, OEsex=6, OEstart=9,  OEfinish=10, OEstatus=12,
			OEclubno=13, OEclub=15, OEnat=16, OEclassno=17, OEclass=18,
			OErent=35, OEfee=36, OEpaid=37, OEcourseno=38, OEcourse=39,
			OElength=40};
*/
bool oEvent::exportOEcsv(const char *filename)
{
  throw std::exception("Wrong method");
  csvparser csv;

  if(!csv.OpenOutput(filename))
    return false;

  csv.OutputRow("Startnr;Bricka;Databas nr.;Efternamn;Frnamn;r;K;Block;ut;Start;Ml;Tid;Status;Klubb nr.;Namn;Ort;Land;Klass nr.;Kort;Lng;Num1;Num2;Num3;Text1;Text2;Text3;Adr. namn;Gata;Rad 2;Post nr.;Ort;Tel;Fax;E-post;Id/Club;Hyrd;Startavgift;Betalt;Bana nr.;Bana;km;Hm;Bana kontroller");

  char bf[256];
  string row;
 	oRunnerList::iterator it;	
	for (it=Runners.begin(); it != Runners.end(); ++it) {
    row.clear();

    sprintf_s(bf, "%d;", it->getStartNo());
    row.append(bf);
    
    sprintf_s(bf, "%d;", it->getCardNo());
    row.append(bf);

    sprintf_s(bf, "%d;", it->getId());
    row.append(bf);

    sprintf_s(bf, "\"%s\";", it->getFamilyName().c_str());
    row.append(bf);

    sprintf_s(bf, "\"%s\";", it->getGivenName().c_str());
    row.append(bf);

    sprintf_s(bf, ";"); //Birth
    row.append(bf);

    sprintf_s(bf, ";"); //Sex
    row.append(bf);

    sprintf_s(bf, ";"); //7
    row.append(bf);

    sprintf_s(bf, ";"); //8
    row.append(bf);

    sprintf_s(bf, "%s;", it->getStartTimeS().c_str()); 
    row.append(bf);

    sprintf_s(bf, "%s;", it->getFinishTimeS().c_str()); 
    row.append(bf);

    sprintf_s(bf, "%s;", it->getRunningTimeS().c_str()); 
    row.append(bf);

    sprintf_s(bf, "%d;", it->getStatus()); 
    row.append(bf);

    sprintf_s(bf, "%d;", it->getClubId()); 
    row.append(bf);

    sprintf_s(bf, ";"); //14
    row.append(bf);

    sprintf_s(bf, "\"%s\";", it->getClub().c_str()); 
    row.append(bf);

    sprintf_s(bf, ";"); //Nat
    row.append(bf);

    sprintf_s(bf, "%d;", it->getClassId()); 
    row.append(bf);

    sprintf_s(bf, "\"%s\";", it->getClass().c_str()); 
    row.append(bf);

    for (int k=19;k<=37;k++)
      row.append(";");

    sprintf_s(bf, "%d;", it->getCourseId()); 
    row.append(bf);

    pCourse pc=it->getCourse();

    if(pc) {
      sprintf_s(bf, "\"%s\";", pc->getName().c_str()); 
      row.append(bf);

      sprintf_s(bf, "%g;", double(pc->getLength())/1000); 
      row.append(bf);
    }
    else
      row.append(";;");

    row.append(";");

    csv.OutputRow(row);
  }
  csv.CloseOutput();

  return true;
}


bool oEvent::exportOECSV(const char *file)
{
	csvparser csv;

	if(!csv.OpenOutput(file))
		return false;
	
	calculateResults();

	oRunnerList::iterator it;

	csv.OutputRow("Startnr;Bricka;Databas nr.;Efternamn;Frnamn;r;K;Block;ut;Start;Ml;Tid;Status;Klubb nr.;Namn;Ort;Land;Klass nr.;Kort;Lng;Num1;Num2;Num3;Text1;Text2;Text3;Adr. namn;Gata;Rad 2;Post nr.;Ort;Tel;Fax;E-post;Id/Club;Hyrd;Startavgift;Betalt;Bana nr.;Bana;km;Hm;Bana kontroller");
	
	char bf[256];
	for(it=Runners.begin(); it != Runners.end(); ++it){		
		vector<string> row;
		row.resize(43);
		oDataInterface di=it->getDI();

		row[0]=conv_is(it->getId());
		row[1]=conv_is(it->getCardNo());
		row[2]=conv_is(it->getId());
		row[3]=it->getFamilyName();
		row[4]=it->getGivenName();
		row[5]=conv_is(di.getInt("BirthYear"));
		row[6]=di.getString("Sex");
		
		row[9]=it->getStartTimeS();
		if(row[9]=="-") row[9]="";

		row[10]=it->getFinishTimeS();
		if(row[10]=="-") row[10]="";

		row[11]=it->getRunningTimeS(); 
		if(row[11]=="-") row[11]="";

		row[12]=conv_is(ConvertStatusToOE(it->getStatus()));
		row[13]=conv_is(it->getClubId());
		
		row[15]=it->getClub();
		row[16]=di.getString("Nationality");
		row[17]=conv_is(it->getClassId());
		row[18]=it->getClass();
		row[19]=it->getClass(); 

		row[35]=conv_is(di.getInt("CardFee"));
		row[36]=conv_is(di.getInt("Fee"));
		row[37]=conv_is(di.getInt("Paid"));
		
		pCourse pc=it->getCourse();
		if(pc){
			row[38]=conv_is(pc->getId());
			row[39]=pc->getName();
			if(pc->getLength()>0){
				sprintf_s(bf, "%d.%d", pc->getLength()/1000, pc->getLength()%1000);
				row[40]=bf;
			}
			//row[41]=conv_is(pc->getDI().getInt("Climbing"));

			row[42]=conv_is(pc->nControls);
		}

		csv.OutputRow(row);

	}

	csv.CloseOutput();

	return true;
}
/*bool oEvent::ImportXML_EntryData(const char *file)
{
	xmlparser xml;
	DWORD tc=GetTickCount();

	if(xml.Read(file))
	{
		DWORD tc_time=GetTickCount()-tc;
		char bf[128];
		sprintf_s(bf, "Time=%d ms", tc_time);
		Notify(bf);

		const xmlobject *xo=xml.GetObject("EntryList");

		if(xo){			
			const xmlList *xl=xo->GetObjects();

			if(xl){
				xmlList::const_iterator it;

				for(it=xl->begin(); it != xl->end(); ++it){
					if(it->Is("ClubEntry")){
						cxmlobject xentry=it->GetObject("Entry");
						int ClubId=it->GetObjectInt("ClubId");
						
						AddXMLEntry(xentry, ClubId);
					}
				}
			}
		}


		xo=xml.GetObject("ClassData");

		if(xo){			
			const xmlList *xl=xo->GetObjects();

			if(xl){
				xmlList::const_iterator it;

				for(it=xl->begin(); it != xl->end(); ++it){
					if(it->Is("Class")){						
						AddXMLClass(&*it);
					}
				}
			}
		}

		xo=xml.GetObject("ClubList");

		if(xo){			
			const xmlList *xl=xo->GetObjects();

			if(xl){
				xmlList::const_iterator it;

				for(it=xl->begin(); it != xl->end(); ++it){
					if(it->Is("Club")){						
						AddXMLClub(&*it);
					}
				}
			}
		}

		return true;
	}
	else return false;
}
*/

struct XMLData
{
	oEvent *oe;
	xmlobject *xml;
};

DWORD ThreadXMLProc(LPVOID data)
{
	XMLData &xd=*(XMLData *)data;
    return xd.oe->processXMLData(xd.xml);
}

bool oEvent::importXMLData(const char *file)
{
	xmlparser xml;
	//DWORD tc=GetTickCount();
	if(!xml.Open(file)){
    throw std::exception(xml.GetError());
		return false;
	}

	DWORD tc=GetTickCount();

	bool Forc=false;
	HANDLE hThread=0;

	char ncpu[16];
	GetEnvironmentVariable("NUMBER_OF_PROCESSORS", ncpu, 16);

	if(atoi(ncpu)>1)
		Forc=true;

	while(xml.Read(200)){
		xmlobject *xmlbase=xml.CloneTakeBase();
		
		if(xml.Failed() || !xmlbase){
      throw std::exception(xml.GetError());
			return false;
		}

		if(hThread){//We can only allow one process to access memory at one time... So wait.
  		WaitForSingleObject(hThread, INFINITE);
			CloseHandle(hThread);
			hThread=0;
		}

		if(Forc){
			XMLData xd;
			xd.oe=this;
			xd.xml=xmlbase;
			DWORD tid;
			hThread=CreateThread(NULL, 0, LPTHREAD_START_ROUTINE(ThreadXMLProc), &xd, 0, &tid);
		}
		else
			processXMLData(xmlbase);	
	}


	if(hThread){
		WaitForSingleObject(hThread, INFINITE);
		CloseHandle(hThread);
		hThread=0;
	}

	DWORD tc_time=GetTickCount()-tc;
	char bf[128];
	sprintf_s(bf, "Time=%d ms", tc_time);
	//Notify(bf);

	return true;
}

int oEvent::processXMLData(xmlobject *xml)
{
	const xmlobject *xo=xml->GetObject("EntryList");

	if(xo){			
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("ClubEntry")){
					cxmlobject xentry=it->GetObject("Entry");
					int ClubId=it->GetObjectInt("ClubId");
					
					addXMLEntry(xentry, ClubId);
				}
			}
		}
	}


	xo=xml->GetObject("ClassData");

	if(xo){			
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Class")){						
					addXMLClass(&*it);
				}
			}
		}
	}

	xo=xml->GetObject("ClubList");

	if(xo){			
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Club")){						
					addXMLClub(&*it, false);
				}
			}
		}
	}

	xo=xml->GetObject("RankList");

	if(xo){			
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Competitor")){						
					addXMLRank(&*it);
				}
			}
		}
	}

	delete xml;

	return 0;
}

bool oEvent::addXMLCompetitorDB(cxmlobject xentry, int clubId)
{
	if(!xentry) return false;
	
	cxmlobject person=xentry->GetObject("Person");	
	if(!person) return false;

	cxmlobject pname=person->GetObject("PersonName");
	if(!pname) return false;

  int cardno=0;
	cxmlobject card=xentry->GetObject("CCard");
	if(card) cardno = card->GetObjectInt("CCardId");

  if(!cardno)
    return false;

  const string &given=pname->GetObjectString("Given");
  const string &family=pname->GetObjectString("Family");

  if(given.empty() || family.empty())
    return false;

  //RunnerDatabase.push_back(oRunner(this));
  //pRunner r=&RunnerDatabase.back();

	string name(given+" "+family);

	const string &sexStr = person->GetObjectString("sex");
  char sex = sexStr.empty() ? 0:sexStr[0];
  
  cxmlobject bd=person->GetObject("BirthDate");
  int birth = bd ? bd->GetObjectInt("Date") : 0;
  
	cxmlobject nat=person->GetObject("Nationality");
	
  char national[3]={0,0,0};
  if (nat) {
    const string &natStr =  nat->GetObjectString("CountryId");
    if (natStr.length()>=3)
      memcpy(national, natStr.c_str(), 3);
  }

  RunnerDBEntry *rde = runnerDB->getRunner(cardno);
 
  if (!rde)
    rde = runnerDB->addRunner(name.c_str(), clubId, cardno);

  rde->setName(name.c_str());
  rde->clubNo = clubId;
  rde->birthYear = birth;
  rde->sex = sex;
  memcpy(rde->national, national, 3);

	return true;
}

bool oEvent::addXMLEntry(cxmlobject xentry, int ClubId)
{
	if(!xentry) return false;

	char bf[256];
	
	cxmlobject person=xentry->GetObject("Person");	
	if(!person) return false;

	cxmlobject pname=person->GetObject("PersonName");
	if(!pname) return false;

	int pid=person->GetObjectInt("PersonId");

	pRunner r=0;

	if(pid)
		r=getRunner(pid, 0);
	
	if(!r){
    if ( pid > 0) {
      oRunner or(this, pid);
		  r = addRunner(or);
    }
    else {
      oRunner or(this);
		  r = addRunner(or);
    }
	}	

	r->setName(pname->GetObjectString("Given")+" "+pname->GetObjectString("Family"));

	oDataInterface DI=r->getDI();

	DI.setString("Sex", person->GetObjectString("sex"));
	cxmlobject bd=person->GetObject("BirthDate");

	if(bd) DI.setInt("BirthYear", bd->GetObjectInt("Date"));

	cxmlobject card=xentry->GetObject("CCard");
	if(card) r->setCardNo(card->GetObjectInt("CCardId"), false);

	cxmlobject nat=person->GetObject("Nationality");

	if(nat) DI.setString("Nationality", nat->GetObjectString("CountryId"));

	cxmlobject eclass=xentry->GetObject("EntryClass");

	if(eclass){
		int cid=eclass->GetObjectInt("ClassId");
		sprintf_s(bf, "Klass %d", cid);
		pClass pc=getClassCreate(cid, bf);//Create class if not found
		//r.SetClassId(Id);
		r->Class=pc;
	}

	if (ClubId) {
		sprintf_s(bf, "Klubb %d", ClubId);
		r->Club=getClubCreate(ClubId, bf);
	}

	cxmlobject edate=xentry->GetObject("EntryDate");

	if(edate) DI.setDate("EntryDate", edate->GetObjectString("Date"));

	return true;
}

bool addXMLPerson(cxmlobject xentry, oWordList &givenNames, 
                                     oWordList &familyNames)
{
	if(!xentry) return false;
	
	cxmlobject person=xentry->GetObject("Person");	
	if(!person) return false;

	cxmlobject pname=person->GetObject("PersonName");
	if(!pname) return false;

  givenNames.insert(pname->GetObjectString("Given").c_str());
  familyNames.insert(pname->GetObjectString("Family").c_str());

	return true;
}

void importXMLNames(const xmlobject *xml, oWordList &givenNames, 
                                  oWordList &familyNames)
{
  if(!xml)
    return;

	const xmlobject *xo=xml->GetObject("CompetitorList");

	if(xo){			
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Competitor")){
          addXMLPerson(&*it, givenNames, familyNames);					
				}
			}
		}
	}
}

void addXMLClub(const xmlobject *xclub, oWordList &clubs)
{
	if(!xclub)
		return;

	string Name=xclub->GetObjectString("Name");
	
	if(Name.empty() || !IsCharAlphaNumeric(Name[0]))
		return;

  clubs.insert(Name.c_str());
	return;
}

void importXMLAllClubs(const xmlobject *xml, oWordList &clubs)
{
  if(!xml)
    return;
	const xmlobject *xo=xml->GetObject("ClubList");

	if(xo){			
		const xmlList *xl=xo->GetObjects();

		if(xl){
			xmlList::const_iterator it;

			for(it=xl->begin(); it != xl->end(); ++it){
				if(it->Is("Club")){
          addXMLClub(&*it, clubs);					
				}
			}
		}
	}
}

bool oEvent::importXMLNames(const char *file,
                            oFreeImport &fi, string &info) const
{
  xmlparser xml;
	info.clear();
  DWORD tc=GetTickCount(), t;
  
  if(!xml.Read(file)){
    throw std::exception(xml.GetError());
	  return false;
  }

  char bf[128];
  t=GetTickCount()-tc;
  sprintf_s(bf, "XML read: %d.%ds ", t/1000, (t/100)%10);
  info+=bf;
  tc=GetTickCount();
  if(xml.GetObject("meosdata")) {
    oEvent oNew(gdibase);
    oNew.open(xml);

    fi.init(oNew.Runners, oNew.Clubs, oNew.Classes);
  }
  else {
    ::importXMLNames(xml.GetObject("CompetitorList"), fi.givenDB, fi.familyDB);
    ::importXMLAllClubs(xml.GetObject("ClubList"), fi.clubDB);
  }

  t=GetTickCount()-tc;
  sprintf_s(bf, "Import: %d.%ds ", t/1000, (t/100)%10);
  info+=bf;

	return true;
}



void oEvent::importXML_IOF_Data(const char *clubfile,
                              const char *competitorfile) 
{
  xmlparser xml_club;
  xmlparser xml_cmp;

  if (clubfile && clubfile[0]) {
    gdibase.addString("",0,"Lser klubbar...");
    gdibase.refresh();

    if(!xml_club.Read(clubfile))
      throw std::exception(xml_club.GetError());


    gdibase.addString("",0,"Lgger till klubbar...");
    gdibase.refresh();
    
    const xmlobject *xo=xml_club.GetObject("ClubList");
    const xmlList *xl=xo ? xo->GetObjects(): 0;

	  if (xl!=0) {
		  xmlList::const_iterator it;

		  for (it=xl->begin(); it != xl->end(); ++it)
			  if (it->Is("Club")) 
				  addXMLClub(&*it, true);
	  }
  }

  if (competitorfile && competitorfile[0]) {
    gdibase.addString("",0,"Lser lpare...");
    gdibase.refresh();
    
    if (!xml_cmp.Read(competitorfile))
      throw std::exception(xml_cmp.GetError());

    gdibase.addString("",0,"Lgger till lpare...");
    gdibase.refresh();

	  const xmlobject *xo=xml_cmp.GetObject("CompetitorList");

    const xmlList *xl=xo ? xo->GetObjects() : 0;

	  if (xl!=0) {
		  xmlList::const_iterator it;

		  for (it=xl->begin(); it != xl->end(); ++it) {
			  if(it->Is("Competitor")){
				  cxmlobject xentry=it->GetObject("Competitor");
				  int ClubId=it->GetObjectInt("ClubId");
          addXMLCompetitorDB(xentry, ClubId);
			  }
		  }
	  }
  }
  //tmpEvent.saveRunnerDatabase("runnerdb.xml");
}


/*
 <Class lowAge="1" highAge="18" sex="K" actualForRanking="Y">
    <ClassId type="nat" idManager="SWE">6</ClassId>
    <Name>D18 Elit</Name>
    <ClassShortName>D18 Elit</ClassShortName>
    <ClassTypeId>E</ClassTypeId>
    <SubstituteClass>
      <ClassId> 600 </ClassId> 
    </SubstituteClass>
    <NotQualifiedSubstituteClass> 
      <ClassId> 600 </ClassId> 
    </NotQualifiedSubstituteClass>
    <EntryFee>
      <EntryFeeId> A </EntryFeeId>
      <Name> Adult </Name>
      <Amount currency="SEK"> 65 </Amount> 
    </EntryFee>
    <ModifyDate> 
      <Date> 2001-10-18 </Date> 
    </ModifyDate>
  </Class>
*/

bool oEvent::addXMLClass(cxmlobject xclass)
{
	if(!xclass)
		return false;
	
	int classid=xclass->GetObjectInt("ClassId");
	string Name=xclass->GetObjectString("Name");

	pClass pc=0;

	if(classid)
		pc=getClassCreate(classid, Name);

	if(!pc)
		pc=addClass(Name);
 
	pc->setName(Name);
	oDataInterface DI=pc->getDI();

	DI.setInt("LowAge", xclass->GetObjectInt("lowAge"));
	DI.setInt("HighAge", xclass->GetObjectInt("highAge"));
	DI.setString("Sex", xclass->GetObjectString("sex"));
	DI.setString("ClassType", xclass->GetObjectString("ClassTypeId"));

	return true;
}


bool oEvent::addXMLClub(cxmlobject xclub, bool savetoDB)
{
	if(!xclub)
		return false;
	
	int clubid=xclub->GetObjectInt("ClubId");
	string Name=xclub->GetObjectString("Name");
  int district = xclub->GetObjectInt("OrganisationId");

	if(Name.length()==0 || !IsCharAlphaNumeric(Name[0]))
		return false;

	cxmlobject address=xclub->GetObject("Address");

	if(!address) return false;

	if(address->GetObjectString("street").length()==0 &&
     address->GetObjectString("careOf").length()==0)
		return 0;

	pClub pc=0;

  if ( !savetoDB ) {
	  if(clubid)
		  pc = getClubCreate(clubid, Name);

    if(!pc) return false;
  }
  else {
    pc = new oClub(this);
    pc->Id = clubid;
  }

	pc->setName(Name);	
	oDataInterface DI=pc->getDI();

	DI.setString("CareOf", address->GetObjectString("careOf"));
	DI.setString("Street", address->GetObjectString("street"));
	DI.setString("City", address->GetObjectString("city"));
	DI.setString("ZIP", address->GetObjectString("zipCode"));
  DI.setInt("District", district);

	cxmlobject tele=xclub->GetObject("Tele");

	if(tele){
		DI.setString("EMail", tele->GetObjectString("mailAddress"));
		DI.setString("Phone", tele->GetObjectString("phoneNumber"));	
	}

	cxmlobject country=xclub->GetObject("Country");

	if(country)
		DI.setString("Nationality", tele->GetObjectString("CountryId"));	
	
  if (savetoDB) {
    runnerDB->importClub(*pc);
    delete pc;
  }

	return true;
}


bool oEvent::addXMLRank(cxmlobject xrank)
{
	if(!xrank)
		return false;
	
	cxmlobject person=0;//xrank->GetObject("Person");
	cxmlobject club=0;//xrank->GetObject("Club");
	cxmlobject rank=0;
	cxmlobject vrank=0;

	const xmlList *x=xrank->GetObjects();
		
	xmlList::const_iterator cit=x->begin();

	while(cit!=x->end()){

		if(cit->Is("Person"))
			person=&*cit;
		else if(cit->Is("Club"))
			club=&*cit;
		else if(cit->Is("Rank")){
			if(cit->GetObjectString("Name")=="Swedish Ranking List")
				rank=&*cit;
			else if(cit->GetObjectString("Name")=="Swedish Vacancy List")
				vrank=&*cit;
		}
		++cit;
	}

	if(!person) return false;

	pRunner r=getRunner(person->GetObjectInt("PersonId"), 0);

	if(!r){
		cxmlobject pname=person->GetObject("PersonName");

		if(!pname) return false;

		string name=pname->GetObjectString("Given")+" "+pname->GetObjectString("Family");
		
		if(!club)
			r=getRunnerByName(name);
		else
			r=getRunnerByName(name, club->GetObjectString("ShortName"));
	}


	if(!r) return false; //No runner here!

	oDataInterface DI=r->getDI();

	if(rank)
		DI.setInt("Rank", rank->GetObjectInt("RankPosition"));

	if(vrank)
		DI.setInt("VacRank", vrank->GetObjectInt("RankPosition"));
	return true;
}
