/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"
#include "oListInfo.h"
#include "oEvent.h"
#include "gdioutput.h"
#include "meos_util.h"
#include <cassert>
#include <cmath>
#include "Localizer.h"

oPrintPost::oPrintPost()
{
  dx=0;
  dy=0;
  format=0;
  type=lString;
  index=-1;
  fixedWidth=0;
}

oPrintPost::oPrintPost(EPostType type_, const string &text_, 
                       int format_, int dx_, int dy_, int index_)
{
  dx=dx_;
  dy=dy_;
  text=text_;
  format=format_;
  type=type_;
  index=index_;
  fixedWidth=0;
}

oListParam::oListParam()
{
  listCode = EStdResultList; //Just need a default
  cb = 0;
  legNumber = 0;
  useControlIdResult = 0;
  filterMaxPer = 0;
  pageBreak = false;
  showInterTimes = false;
}

oListInfo::oListInfo(void)
{
  listType=EBaseTypeRunner;
  listSubType=EBaseTypeRunner;
  calcResults=false;
  listPostFilter.resize(_EFilterMax+1, 0);
  fixedType = false;

  supportClasses = true;
  supportLegs = false;
  supportExtra = false;
}

oListInfo::~oListInfo(void)
{
}


bool oEvent::formatListString(const oPrintPost &pp, string &out, const oListParam &par,
                              const pTeam t, const pRunner r, const pClub c,
                              const pClass pc, oCounter &counter) const
{
  
  char bf[512];
  const string *sptr=0;
  bf[0]=0;

  switch ( pp.type ) {
    case lClassName:
      sptr=pc ? &pc->Name : 0;
      break;
    case lClassLength:
      if(pc) strcpy_s(bf, pc->getLength().c_str());
      break;
    case lClassStartName:
      if(pc) strcpy_s(bf, pc->getDI().getString("StartName").c_str());
      break;
    case lClassResultFraction:
      if(pc) {
        int total, finished,  dns;
        oe->getNumClassRunners(pc->getId(), par.legNumber, total, finished, dns);
        sprintf_s(bf, "(%d/%d)", finished, total);
      }
      break;
    case lCmpName:
      sptr=&this->Name;
      break;
    case lCmpDate:
      strcpy_s(bf, this->getDate().c_str());      
      break;
    case lRunnerClub:
      sptr=(r && r->Club) ? &r->Club->Name : 0;
      break;
    case lRunnerFinish:
      if(r) strcpy_s(bf, r->getFinishTimeS().c_str());
      break;
    case lRunnerStart:
      if(r) strcpy_s(bf, r->getStartTimeCompact().c_str());
      break;
    case lRunnerName:
      sptr = r ? &r->Name : 0;
      break;
    case lRunnerTime:
      if(r) strcpy_s(bf, r->getRunningTimeS().c_str());
      break;
    case lRunnerTimeStatus:
      if(r) {
        if(r->getStatus()==StatusOK && pc && !pc->getNoTiming())
          strcpy_s(bf, r->getRunningTimeS().c_str() );
        else
          strcpy_s(bf, r->getStatusS().c_str() );
      }
      break;
    case lRunnerTempTimeStatus:
      if(r) {
        if(r->tStatus==StatusOK && pc && !pc->getNoTiming())
          strcpy_s(bf, formatTime(r->tRT).c_str() );
        else
          strcpy_s(bf, formatStatus(r->tStatus).c_str() );
      }
      break;
    case lRunnerPlace:
      if(r) strcpy_s(bf, r->getPrintPlaceS().c_str() );
      break;
    case lRunnerTimeAfter:
      if (r && pc) {
        int tleg=r->tLeg>=0 ? r->tLeg:0;
        if (r->Status==StatusOK &&  pc && !pc->getNoTiming() 
              && r->getRunningTime() > pc->getBestLegTime(tleg)) {
          int after=r->getRunningTime()-pc->getBestLegTime(tleg);
          sprintf_s(bf, "+%d:%02d", after/60, after%60);
        }
      }
      break;
    case lRunnerTempTimeAfter:
      if (r && pc) {
        if (r->tStatus==StatusOK &&  pc && !pc->getNoTiming() 
              && r->tRT>pc->tLegLeaderTime) {
          int after=r->tRT-pc->tLegLeaderTime;
          sprintf_s(bf, "+%d:%02d", after/60, after%60);
        }
      }
      break;

    case lRunnerCard:
      if(r && r->CardNo > 0)
        sprintf_s(bf, "%d", r->CardNo);
    case lRunnerRank:
      if(r) {
        int rank=r->getDI().getInt("Rank");
        if(rank>0)
          strcpy_s(bf, FormatRank(rank).c_str());          
      }
      break;
    case lRunnerCourse:
      if(r && r->getCourse()) 
        sptr = &r->getCourse()->getName();
      break;
    case lRunnerBib:
      if(r) {
        int bib=r->getBib();
        if(bib>0)
          sprintf_s(bf, "%d", bib);
      }
      break;
    case lRunnerUMMasterPoint:
      if(r) {
        int total, finished, dns;
        oe->getNumClassRunners(pc->getId(), par.legNumber, total, finished, dns);
        int percent = int(floor(0.5+double((100*(total-dns-r->getPlace()))/double(total-dns))));
        if (r->getStatus()==StatusOK)
          sprintf_s(bf, "%d",  percent);
        else
          sprintf_s(bf, "0");
      }
      break;
    case lTeamName:
      sptr = t ? &t->Name : 0;
      break;
    case lTeamStart:
      if(t && unsigned(pp.index)<t->Runners.size() && t->Runners[pp.index])
        strcpy_s(bf, t->Runners[pp.index]->getStartTimeCompact().c_str());
      break;
    case lTeamStatus:
      if(t) strcpy_s(bf, t->getLegStatusS(pp.index).c_str() );
      break;
    case lTeamTime:
      if(t) strcpy_s(bf, t->getLegRunningTimeS(pp.index).c_str() );
      break;    
    case lTeamTimeStatus:
      if(t) {
        if(t->getLegStatus(pp.index)==StatusOK)
          strcpy_s(bf, t->getLegRunningTimeS(pp.index).c_str() );
        else
          strcpy_s(bf, t->getLegStatusS(pp.index).c_str() );
      }
      break; 
    case lTeamTimeAfter:
      //sprintf_s(bf, "+1:01");
      if (t && t->getLegStatus(pp.index)==StatusOK) {
        int ta=t->getTimeAfter(pp.index);
        if (ta>0)
          sprintf_s(bf, "+%d:%02d", ta/60, ta%60);
      }
      break;
    case lTeamPlace:
      if(t) strcpy_s(bf, t->getLegPrintPlaceS(pp.index).c_str() );
      break;
    case lTeamLegTimeStatus:
      if(t) {
        if(t->getLegStatus(counter.level3)==StatusOK)
          strcpy_s(bf, t->getLegRunningTimeS(counter.level3).c_str() );
        else
          strcpy_s(bf, t->getLegStatusS(counter.level3).c_str() );
      }
      break; 
    case lTeamLegTimeAfter:
      if (t && t->getLegStatus(counter.level3)==StatusOK) {
        int ta=t->getTimeAfter(counter.level3);
        if (ta>0)
          sprintf_s(bf, "+%d:%02d", ta/60, ta%60);
      }
      break;
    case lTeamClub:
      sptr = t && t->Club ? &t->Club->Name : 0;
      break;
    case lTeamRunner:
      if(t && unsigned(pp.index)<t->Runners.size() && t->Runners[pp.index])
        sptr=&t->Runners[pp.index]->Name;
      break;
    case lTeamRunnerCard:
      if(t && unsigned(pp.index)<t->Runners.size() && t->Runners[pp.index]
         && t->Runners[pp.index]->CardNo>0)
        sprintf_s(bf, "%d", t->Runners[pp.index]->CardNo);
      break;
    case lTeamBib:
      if(t) sprintf_s(bf, "%d", t->getStartNo());
      break;
    case lPunchNamedTime:
      if(r && r->Card && r->getCourse()) {
        const pCourse crs=r->getCourse();
        const oControl *ctrl=crs->getControl(counter.level3);
        //oPunch *op=r->Card->GetPunchByType(pctrl->Numbers[0]);
        if (r->getPunchTime(counter.level3)>0 && ctrl->hasName()) {
          sprintf_s(bf, "%s: %s (%s)", ctrl->getName().c_str(),
            r->getNamedSplitS(counter.level3).c_str(),
            r->getPunchTimeS(counter.level3).c_str());
        }
      }
      break;
    case lSubSubCounter:
      sprintf_s(bf, "%d.", counter.level3+1);
      break;
    case lSubCounter:
      sprintf_s(bf, "%d.", counter.level2+1);
      break;
    case lTotalCounter:
      sprintf_s(bf, "%d.", counter.level1+1);
      break;
  }

  if(pp.type!=lString && (sptr==0 || sptr->empty()) && bf[0]==0) 
    out.clear();
  else if(sptr) {
    if(pp.text.empty())
      out=*sptr;
    else {
      sprintf_s(bf, pp.text.c_str(), sptr->c_str());
      out=bf;
    }
  }
  else {
    if(pp.text.empty())
      out=bf;
    else {
      char bf2[512];
      sprintf_s(bf2, pp.text.c_str(), bf);
      out=bf2;
    }
  }
  return !out.empty();
}

bool oEvent::formatPrintPost(const list<oPrintPost> &ppli, gdioutput &gdi, const oListParam &par,
                              const pTeam t, const pRunner r, const pClub c,
                              const pClass pc, oCounter &counter)
{
  list<oPrintPost>::const_iterator ppit;
  string text;
	int y=gdi.getCY(); 
	int x=gdi.getCX();
  bool updated=false;
  
  for (ppit=ppli.begin();ppit!=ppli.end();) {
    const oPrintPost &pp=*ppit;
    int limit=0;
    
    if( ++ppit != ppli.end() && ppit->dy==pp.dy)
      limit=ppit->dx - pp.dx;

    //if(pp.fixedWidth>0)
    limit=max(pp.fixedWidth - pp.dx, limit);

    assert(limit>=0);
    pRunner rr=r;    
    if(!rr && t)
      rr=t->getRunner(pp.index);

    updated=formatListString(pp, text, par, t, rr, c, pc, counter) ? 
                                                      true : updated;  
    if(!text.empty()) {
      if (pp.type == lRunnerName && rr)
        gdi.addStringUT(y+pp.dy, x+pp.dx, pp.format, text, 
                      limit, par.cb).setExtra(rr);
      else if (pp.type == lTeamName && t)
        gdi.addStringUT(y+pp.dy, x+pp.dx, pp.format, text, 
                      limit, par.cb).setExtra(t);
      else
        gdi.addStringUT(y+pp.dy, x+pp.dx, pp.format, text, limit, 0);

    }
  }  

  //reserve.reserveHeight=gdi.getCY()-y+2;

  return updated; 
}

void oEvent::listGeneratePunches(const list<oPrintPost> &ppli, gdioutput &gdi, const oListParam &par,
                                 pTeam t, pRunner r, pClub club, pClass cls) 
{
  if(!r) 
    return;

  pCourse crs=r->getCourse();

  if(!crs)
    return;

  int w=ppli.front().fixedWidth;
 
  int xlimit=gdi.getCX()+600;

  if (w>0) {
    gdi.pushX();
    gdi.fillNone();
  }

  bool updated=false;
  oCounter counter;

  for (int k=0;k<crs->nControls;k++) {
    const pControl ctrl=crs->Controls[k];

    if(w>0 && updated) {
      updated=false;
      if( gdi.getCX()+w> xlimit) {
        gdi.popX();
        gdi.dropLine(1);
      }
      else 
       gdi.setCX(gdi.getCX()+w);
    }
    if (ctrl) 
      updated=formatPrintPost(ppli, gdi, par, t, r, club, cls, counter);

    counter.level3++;
  }

  if(w>0) {
    gdi.popX();
    gdi.fillDown();
    gdi.dropLine(1);
  }
}

void oEvent::generateList(gdioutput &gdi, bool reEvaluate, const oListInfo &li)
{
  if (reEvaluate)
    reEvaluateAll(false);

  oCounter counter;
  //Render header
  formatPrintPost(li.Head, gdi, li.lp, 0,0,0,0, counter);

  oe->calcUseStartSeconds();

  if (li.fixedType) {
    generateFixedList(gdi, li);
    return;
  }

  int Id=0;

  if( li.listType==li.EBaseTypeRunner ) {

    if( li.calcResults ) {
      if (li.lp.useControlIdResult) 
        calculateSplitResults(li.lp.useControlIdResult);
      else 
        calculateResults();
    }
    else sortRunners(li.sortOrder);

    oRunnerList::iterator it;
    //bool lastLeg = li.lp.legNumber < 0;

	  for (it=Runners.begin(); it != Runners.end(); ++it) {	
      if (it->isRemoved())
        continue;

      /* -1 = all runners!?!
      int legNumber = li.lp.legNumber;
      if (lastLeg) {
        if (it->Class)
          legNumber = it->Class->getNumStages()-1;
        else
          legNumber = 0;
      }*/

      if (it->legToRun() != li.lp.legNumber && li.lp.legNumber!=-1)
        continue;

      if(!li.lp.selection.empty() && li.lp.selection.count(it->getClassId())==0)
			  continue;
      
      if(li.filter(EFilterHasResult))
        if(li.lp.useControlIdResult==0 && it->Status==StatusUnknown)
          continue;
        else if(li.lp.useControlIdResult>0 && it->tStatus!=StatusOK)
          continue;

      if(li.filter(EFilterRentCard) && it->getDI().getInt("CardFee")==0)
        continue;

      if(li.filter(EFilterHasCard) && it->getCardNo()==0)
        continue;

      if(li.filter(EFilterHasNoCard) && it->getCardNo()>0)
        continue;

		  if (it->getClassId()!=Id) {
        if (li.lp.pageBreak) {
          if (Id) 
            gdi.addStringUT(gdi.getCY()-1, 0, pageNewPage, "");
          
          gdi.addStringUT(pagePageInfo, it->getClass());
        }

        counter.level2=0;
        counter.level3=0;
			  Id=it->getClassId();
        formatPrintPost(li.subHead, gdi, li.lp, it->tInTeam, &*it, it->Club, it->Class, counter);
		  }

      if (li.lp.filterMaxPer==0 || counter.level2<=li.lp.filterMaxPer) {
        formatPrintPost(li.listPost, gdi, li.lp, it->tInTeam, &*it, it->Club, it->Class, counter);
        
        if(li.listSubType==li.EBaseTypePunches) {
          listGeneratePunches(li.subListPost, gdi, li.lp, it->tInTeam, &*it, it->Club, it->Class); 
        }
      }
      ++counter;
	  }
  }
  else if( li.listType==li.EBaseTypeTeam ) {
    if (li.calcResults)
      calculateTeamResults();
    
    sortTeams(li.sortOrder, li.lp.legNumber);

    oTeamList::iterator it;
	  for (it=Teams.begin(); it != Teams.end(); ++it) {
      if (it->isRemoved())
        continue;

      if (!li.lp.selection.empty() && li.lp.selection.count(it->getClassId())==0)
			  continue;

      if(li.filter(EFilterHasResult) && it->getLegStatus(li.lp.legNumber)==EFilterHasResult)
        continue;

		  if (it->getClassId()!=Id) {
        if(li.lp.pageBreak) {
          if(Id) 
            gdi.addStringUT(gdi.getCY()-1, 0, pageNewPage, "");
          
          gdi.addStringUT(pagePageInfo, it->getClass());
        }
			  Id=it->getClassId();
        //classCounter=0;
        counter.level2=0;
        counter.level3=0;
        formatPrintPost(li.subHead, gdi, li.lp, &*it, 0, it->Club, it->Class, counter);
		  }
      //classCounter++;
      ++counter;
      if (li.lp.filterMaxPer==0 || counter.level2<=li.lp.filterMaxPer) {
        counter.level3=0;
        formatPrintPost(li.listPost, gdi, li.lp, &*it, 0, it->Club, it->Class, counter);
  
        if(li.subListPost.empty())
          continue;

        if(li.listSubType==li.EBaseTypeRunner) {
          for (int k=0;k<int(it->Runners.size());k++) {
            if(it->Runners[k]) {
              formatPrintPost(li.subListPost, gdi, li.lp, &*it, it->Runners[k], 
                              it->Club, it->Class, counter);
              counter.level3++;
            }
          }
        }
        else if(li.listSubType==li.EBaseTypePunches) {
          pRunner r=it->Runners.empty() ? 0:it->Runners[0];
          if(!r) continue;

          listGeneratePunches(li.subListPost, gdi, li.lp, &*it, r, it->Club, it->Class); 
        }
      }
	  }
  }
	gdi.UpdateScrollbars();
}

void oEvent::fillListTypes(gdioutput &gdi, const string &name, int filter)
{
  map<EStdListType, oListInfo> listMap;
  getListTypes(listMap, filter);

  gdi.clearList(name);
  map<EStdListType, oListInfo>::iterator it;
	
  for (it=listMap.begin(); it!=listMap.end(); ++it) {
    gdi.addItem(name, it->second.Name, it->first);
  }
}

void oEvent::getListType(EStdListType type, oListInfo &li) 
{
  map<EStdListType, oListInfo> listMap;
  getListTypes(listMap, false);
  li = listMap[type];
}
  

void oEvent::getListTypes(map<EStdListType, oListInfo> &listMap, int filterResults)
{
  listMap.clear();
  oListInfo li;

  if(!filterResults) {
    li.Name=lang.tl("Startlista, individuell");
    li.listType=li.EBaseTypeRunner;
    li.supportClasses = true;
    li.supportLegs = true;
    li.supportExtra = false;
    listMap[EStdStartList]=li;
  }

  li.Name=lang.tl("Resultat, individuell");
  li.listType=li.EBaseTypeRunner;
  li.supportClasses = true;
  li.supportLegs = true;
  li.supportExtra = true;
  listMap[EStdResultList]=li;

  if(!filterResults) {
    li.Name=lang.tl("Startlista, patrull");
    li.listType=li.EBaseTypeTeam;
    li.supportClasses = true;
    li.supportLegs = false;
    li.supportExtra = false;

    listMap[EStdPatrolStartList]=li;
  }

  li.Name=lang.tl("Resultat, patrull");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = false;
  li.supportExtra = false;

  listMap[EStdPatrolResultList]=li;

  li.Name="RAID-Resultat (STOR)";
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = false;
  li.supportExtra = false;
  listMap[EStdRaidResultListLARGE]=li;

  li.Name=lang.tl("Resultat (STOR)");
  li.supportClasses = true;
  li.supportLegs = true;
  li.supportExtra = true;
  li.listType=li.EBaseTypeRunner;
  listMap[EStdResultListLARGE]=li;

  li.Name=lang.tl("Stafettresultat, strcka (STOR)");
  li.supportClasses = true;
  li.supportLegs = true;
  li.supportExtra = false;
  li.listType=li.EBaseTypeTeam;
  listMap[EStdTeamResultListLegLARGE]=li;
  
  if(!filterResults) {
    li.Name=lang.tl("Hyrbricksrapport");
    li.listType=li.EBaseTypeRunner;
    li.supportClasses = false;
    li.supportLegs = false;
    li.supportExtra = false;
    listMap[EStdRentedCard]=li;
  }

  li.Name=lang.tl("Stafettresultat, delstrckor");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = false;
  li.supportExtra = false;
  listMap[EStdTeamResultListAll]=li;

  li.Name=lang.tl("Stafettresultat, lag");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = false;
  li.supportExtra = false;
  listMap[EStdTeamResultList]=li;

  li.Name=lang.tl("Stafettresultat, strcka");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = true;
  li.supportExtra = false;
  listMap[EStdTeamResultListLeg]=li;

  li.Name=lang.tl("Startlista, stafett (lag)");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = false;
  li.supportExtra = false;
  listMap[EStdTeamStartList]=li;

  li.Name=lang.tl("Startlista, stafett (strcka)");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = true;
  li.supportExtra = false;
  listMap[EStdTeamStartListLeg]=li;

  li.Name=lang.tl("Strcktilldelning, stafett");
  li.listType=li.EBaseTypeTeam;
  li.supportClasses = true;
  li.supportLegs = false;
  li.supportExtra = false;
  listMap[ETeamCourseList]=li;

  li.Name="UM, mstarpong";
  li.listType=li.EBaseTypeRunner;
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  listMap[EStdUM_Master]=li;
  
  li.Name=lang.tl("Tvlingsrapport");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedReport]=li;

  li.Name=lang.tl("Kontroll infr tvlingen");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedPreReport]=li;

  li.Name=lang.tl("Kvar-i-skogen");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedInForest]=li;
  
  li.Name=lang.tl("Fakturor");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedInvoices]=li;
  
  li.Name=lang.tl("Ekonomisk sammanstllning");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedEconomy]=li;
  
  li.Name=lang.tl("Frst-i-ml, klassvis");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedResultFinishPerClass]=li;
  
  li.Name=lang.tl("Frst-i-ml, gemensam");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedResultFinish]=li;
  
  li.Name=lang.tl("Minutstartlista");
  li.supportClasses = false;
  li.supportLegs = false;
  li.supportExtra = false;
  li.listType=li.EBaseTypeNone;
  listMap[EFixedMinuteStartlist]=li;
}


void oEvent::generateListInfo(EStdListType lt, const gdioutput &gdi, int classId, oListInfo &li)
{
  oListParam par;

  if(classId!=0)
    par.selection.insert(classId);

  par.listCode=lt;
 
  generateListInfo(par, gdi.getLineHeight(), li);
}

void oEvent::generateFixedList(gdioutput &gdi, const oListInfo &li)
{
#ifndef MEOSDB
  switch (li.lp.listCode) {
    case EFixedPreReport:
      generatePreReport(gdi);
    break;
    
    case EFixedReport:
      generateCompetitionReport(gdi);
    break;

    case EFixedInForest:
      generateInForestList(gdi, 0, 0);
    break;

    case EFixedEconomy:
      printInvoices(gdi, true);
    break;

    case EFixedInvoices:
      printInvoices(gdi, false);
    break;

    case EFixedResultFinishPerClass:
      generateResultlistFinishTime(gdi, true, li.lp.cb);
    break;
    
    case EFixedResultFinish:
      generateResultlistFinishTime(gdi, false, li.lp.cb);
    break;

    case EFixedMinuteStartlist:
      generateMinuteStartlist(gdi);
    break;

  }
#endif
}

void oEvent::generateListInfo(oListParam &par, int lineHeight, oListInfo &li)
{
  par.cb = 0;
  const int lh=lineHeight;
  const int vspace=lh/2;
  EStdListType lt=par.listCode;
  li=oListInfo();
  li.lp = par;
  int bib, ln;
  switch (lt) {
    case EStdStartList: 
    {
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Startlista - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));
      
      int bib=0;
      if (hasBib()) {
        li.addListPost(oPrintPost(lRunnerBib, "", normalText, 0, 0));
        bib=40;
      }
      li.addListPost(oPrintPost(lRunnerStart, "", normalText, 0+bib, 0));
      li.addListPost(oPrintPost(lRunnerName, "", normalText, 70+bib, 0));
      li.addListPost(oPrintPost(lRunnerClub, "", normalText, 300+bib, 0));
      if (hasRank()) {
        li.addListPost(oPrintPost(lRunnerBib, "", normalText, 470+bib, 0));
        bib+=50;
      }
      li.addListPost(oPrintPost(lRunnerCard, "", normalText, 470+bib, 0));

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      li.addSubHead(oPrintPost(lClassLength, lang.tl("%s meter"), normalText, 300+bib, 10));
      li.addSubHead(oPrintPost(lClassStartName, "", normalText, 470+bib, 10));

      li.listType=li.EBaseTypeRunner;
      li.sortOrder=ClassStartTime;
      break;
    }   

    case EStdRentedCard: 
    {
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Hyrbricksrapport - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));
     
      li.addListPost(oPrintPost(lTotalCounter, "%s", normalText, 0, 0));     
      li.addListPost(oPrintPost(lRunnerCard, "", normalText, 30, 0));
      li.addListPost(oPrintPost(lRunnerName, "", normalText, 130, 0));      
      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      
      li.setFilter(EFilterHasCard);
      li.setFilter(EFilterRentCard);
      li.listType=li.EBaseTypeRunner;
      li.sortOrder=ClassStartTime;
      break;
    }   

    case EStdResultList:
      char title[256];
      if (li.lp.useControlIdResult==0)
        sprintf_s(title, "%s", lang.tl("Resultat - %s").c_str());
      else {
        pControl ctrl=getControl(li.lp.useControlIdResult, false);
        if(ctrl && !ctrl->Name.empty())
          sprintf_s(title, "%s, %s", lang.tl("Resultat - %s").c_str(), ctrl->Name.c_str());
        else if(ctrl)
          sprintf_s(title, lang.tl("%s, vid kontroll %d").c_str(), 
          lang.tl("Resultat - %s").c_str(), ctrl->Numbers[0]);
        else
          sprintf_s(title, lang.tl("Resultat vid oknd kontroll").c_str());
      }
      li.addHead(oPrintPost(lCmpName, MakeDash(title), boldLarge, 0,0));

      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));

      li.addListPost(oPrintPost(lRunnerPlace, "", normalText, 2, 0));
      li.addListPost(oPrintPost(lRunnerName, "", normalText, 70, 0));
      li.addListPost(oPrintPost(lRunnerClub, "", normalText, 300, 0));

      if (li.lp.useControlIdResult==0) {
        li.addSubHead(oPrintPost(lClassResultFraction, "", normalText, 300, 10));

        li.addListPost(oPrintPost(lRunnerTimeStatus, "", normalText, 480, 0));
        li.addListPost(oPrintPost(lRunnerTimeAfter, "", normalText, 530, 0));
        if(li.lp.showInterTimes) {
          li.addSubListPost(oPrintPost(lPunchNamedTime, "", normalText, 10, 0, 1));
          li.subListPost.back().fixedWidth=160;
        }
      }
      else {
        li.addListPost(oPrintPost(lRunnerTempTimeStatus, "", normalText, 480, 0));
        li.addListPost(oPrintPost(lRunnerTempTimeAfter, "", normalText, 530, 0));
      }
      
      //li.lp.legNumber = 0;
      li.calcResults = true;
      li.listType=li.EBaseTypeRunner;
      li.sortOrder=ClassResult;
      li.setFilter(EFilterHasResult);
      break;

    case EStdTeamResultList:
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Resultatsammanstllning - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 14));
      li.addSubHead(oPrintPost(lClassResultFraction, "", normalText, 280, 14));

      //Use last leg for every team (index=-1)
      li.addListPost(oPrintPost(lTeamPlace, "", normalText, 0, 5, -1));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 25, 5, -1));
      li.addListPost(oPrintPost(lTeamTimeStatus, "", normalText, 280, 5, -1));
      li.addListPost(oPrintPost(lTeamTimeAfter, "", normalText, 340, 5, -1));
      
      li.lp.legNumber=-1;
      li.calcResults=true;
      li.listType=li.EBaseTypeTeam;
      li.listSubType=li.EBaseTypeRunner;
      li.sortOrder=ClassResult;
      li.setFilter(EFilterHasResult); 
      break;

    case EStdTeamResultListAll:
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Resultat - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 14));
      li.addSubHead(oPrintPost(lClassResultFraction, "", normalText, 280, 14));

      //Use last leg for every team (index=-1)
      li.addListPost(oPrintPost(lTeamPlace, "", normalText, 0, 5, -1));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 25, 5, -1));
      li.addListPost(oPrintPost(lTeamTimeStatus, "", normalText, 280, 5, -1));
      li.addListPost(oPrintPost(lTeamTimeAfter, "", normalText, 340, 5, -1));
      
      li.addSubListPost(oPrintPost(lSubSubCounter, "", normalText, 25, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerName, "", normalText, 45, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerTimeStatus, "", normalText, 280, 0, 0));
      li.addSubListPost(oPrintPost(lTeamLegTimeStatus, "", normalText, 400, 0, 0));
      li.addSubListPost(oPrintPost(lTeamLegTimeAfter, "", normalText, 460, 0, 0));

      li.lp.legNumber=-1;
      li.calcResults=true;
      li.listType=li.EBaseTypeTeam;
      li.listSubType=li.EBaseTypeRunner;
      li.sortOrder=ClassResult;
      li.setFilter(EFilterHasResult); 
      break;

    case EStdTeamResultListLeg:
      if (li.lp.legNumber>=0)
        sprintf_s(title, (lang.tl("Resultat efter strcka %d")+" - %%s").c_str(), li.lp.legNumber+1);
      else
        sprintf_s(title, (lang.tl("Resultat efter strckan")+" - %%s").c_str());
      
      li.addHead(oPrintPost(lCmpName, MakeDash(title), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 14));
      li.addSubHead(oPrintPost(lClassResultFraction, "", normalText, 280, 14));
      
      ln=li.lp.legNumber;
      li.addListPost(oPrintPost(lTeamPlace, "", normalText, 0, 5, ln));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 25, 5, ln));
      li.addListPost(oPrintPost(lRunnerName, "", normalText, 240, 5, ln));
      li.addListPost(oPrintPost(lRunnerTimeStatus, "", normalText, 460, 5, ln));
    
      li.addListPost(oPrintPost(lTeamTimeStatus, "", normalText, 510, 5, ln));
      li.addListPost(oPrintPost(lTeamTimeAfter, "", normalText, 560, 5, ln));
      
      li.calcResults=true;
      li.listType=li.EBaseTypeTeam;
      li.sortOrder=ClassResult;
      li.setFilter(EFilterHasResult); 
      break;

    case EStdTeamResultListLegLARGE:
      if (li.lp.legNumber>=0)
        sprintf_s(title, ("%%s - "+lang.tl("strcka %d")).c_str(), li.lp.legNumber+1);
      else
        sprintf_s(title, ("%%s - "+lang.tl("slutstrckan")).c_str());
      
      li.addSubHead(oPrintPost(lClassName, MakeDash(title), boldLarge, 0, 14));
      li.addSubHead(oPrintPost(lClassResultFraction, "", fontLarge, 500, 14));
      
      ln=li.lp.legNumber;
      li.addListPost(oPrintPost(lTeamPlace, "", fontLarge, 0, 5, ln));
      li.addListPost(oPrintPost(lTeamName, "", fontLarge, 45, 5, ln)); 
      {
        const int ex1=20, ex2=40;
        li.addListPost(oPrintPost(lRunnerName, "", fontLarge, 260+ex1, 5, ln));
        li.addListPost(oPrintPost(lRunnerTimeStatus, "", fontLarge, 500+ex2, 5, ln));
      
        li.addListPost(oPrintPost(lTeamTimeStatus, "", fontLarge, 580+ex2, 5, ln));
        li.addListPost(oPrintPost(lTeamTimeAfter, "", fontLarge, 670+ex2, 5, ln));
      }
      li.calcResults=true;
      li.listType=li.EBaseTypeTeam;
      li.sortOrder=ClassResult;
      li.setFilter(EFilterHasResult); 
      break;

    case EStdTeamStartList:
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Startlista - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));

      li.listType=li.EBaseTypeTeam;
      bib=0;
      if (hasBib()) {
        li.addListPost(oPrintPost(lTeamBib, "", normalText, 0, 0));
        bib=40;
      }
      li.addListPost(oPrintPost(lTeamStart, "", normalText, 0+bib, 0));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 70+bib, 0));
      li.addListPost(oPrintPost(lTeamClub, "", normalText, 300+bib, 0));
      
      li.listSubType=li.EBaseTypeRunner;
      li.addSubListPost(oPrintPost(lSubSubCounter, "", normalText, 0+bib, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerName, "", normalText, 70+bib, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerCard, "", normalText, 300+bib, 0, 0));
    
      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      li.addSubHead(oPrintPost(lClassStartName, "", normalText, 470+bib, 10));

      li.sortOrder=ClassStartTime;
      break;

    case ETeamCourseList:
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Bantilldelningslista - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));
      li.listType=li.EBaseTypeTeam;

      bib=0;
      li.addListPost(oPrintPost(lTeamBib, "", normalText, 0+bib, 0));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 70+bib, 0));
      li.addListPost(oPrintPost(lTeamClub, "", normalText, 250+bib, 0));
      li.addListPost(oPrintPost(lString, "Bana", normalText, 320+bib, 0));
      
      li.listSubType=li.EBaseTypeRunner;
      li.addSubListPost(oPrintPost(lSubSubCounter, "", normalText, 0+bib, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerName, "", normalText, 70+bib, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerCard, "", normalText, 300+bib, 0, 0));
      li.addSubListPost(oPrintPost(lRunnerCourse, "", normalText, 370+bib, 0, 0));
    
      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      
      li.sortOrder=ClassStartTime;
      break;

    case EStdTeamStartListLeg:
      if (li.lp.legNumber==-1)
        throw std::exception("Ogiltigt val av strcka");

      sprintf_s(title, lang.tl("Startlista %%s - strcka %d").c_str(), li.lp.legNumber+1);

      li.addHead(oPrintPost(lCmpName, MakeDash(title), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));

      ln=li.lp.legNumber;
      li.listType=li.EBaseTypeTeam;
      bib=0;
      if (hasBib()) {
        li.addListPost(oPrintPost(lTeamBib, "", normalText, 0, 0));
        bib=40;
      }
      li.addListPost(oPrintPost(lTeamStart, "", normalText, 0+bib, 0, ln));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 70+bib, 0, ln));
      li.addListPost(oPrintPost(lTeamRunner, "", normalText, 300+bib, 0, ln));
      li.addListPost(oPrintPost(lTeamRunnerCard, "", normalText, 520+bib, 0, ln));
      
      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      li.addSubHead(oPrintPost(lClassStartName, "", normalText, 300+bib, 10));

      li.sortOrder=ClassStartTime;
      break;

    case EStdPatrolStartList:
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Startlista - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));
      
      bib=0;
      if (true || hasBib()) {
        li.addListPost(oPrintPost(lTeamBib, "", normalText, 0, 0));
        bib=40;
      }
      li.addListPost(oPrintPost(lTeamStart, "", normalText, 0+bib, 0));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 70+bib, 0));
      li.addListPost(oPrintPost(lTeamClub, "", normalText, 300+bib, 0));
      
      li.addListPost(oPrintPost(lTeamRunner, "", normalText, 70+bib, lh, 0));
      li.addListPost(oPrintPost(lTeamRunnerCard, "", normalText, bib+220, lh, 0));
      
      li.addListPost(oPrintPost(lTeamRunner, "", normalText, 300+bib, lh, 1));
      li.addListPost(oPrintPost(lTeamRunnerCard, "", normalText, bib+520-70, lh, 1));

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      li.addSubHead(oPrintPost(lClassLength, "%s meter", normalText, 300+bib, 10));
      li.addSubHead(oPrintPost(lClassStartName, "", normalText, 470+bib, 10));

      li.listType=li.EBaseTypeTeam;
      li.sortOrder=ClassStartTime;
      break;

    case EStdPatrolResultList:
      li.addHead(oPrintPost(lCmpName, MakeDash(lang.tl("Resultatlista - %s")), boldLarge, 0,0));
      li.addHead(oPrintPost(lCmpDate, "", normalText, 0, 25));
            
      li.addListPost(oPrintPost(lTeamPlace, "", normalText, 0, vspace, 1));
      li.addListPost(oPrintPost(lTeamName, "", normalText, 70, vspace));
      li.addListPost(oPrintPost(lTeamClub, "", normalText, 250, vspace));
      li.addListPost(oPrintPost(lTeamTimeStatus, "", normalText, 400, vspace, 1));
      li.addListPost(oPrintPost(lTeamRunner, "", normalText, 70, lh+vspace, 0));
      li.addListPost(oPrintPost(lTeamRunner, "", normalText, 250, lh+vspace, 1));
      li.setFilter(EFilterHasResult); 

      if(li.lp.showInterTimes) {
        li.addSubListPost(oPrintPost(lPunchNamedTime, "", normalText, 10, 0, 1));
        li.subListPost.back().fixedWidth=160;
      }

      li.addSubHead(oPrintPost(lClassName, "", boldText, 0, 10));
      li.listType=li.EBaseTypeTeam;
      li.listSubType=li.EBaseTypePunches;
      li.sortOrder=ClassResult;
      li.calcResults=true;
      break;

    case EStdRaidResultListLARGE:
      li.addListPost(oPrintPost(lTeamPlace, "", fontLarge, 0, vspace));
      li.addListPost(oPrintPost(lTeamName, "", fontLarge, 40, vspace));
      li.addListPost(oPrintPost(lTeamTimeStatus, "", fontLarge, 490, vspace));
      
      li.addListPost(oPrintPost(lTeamRunner, "", fontLarge, 40, 25+vspace, 0));
      li.addListPost(oPrintPost(lTeamRunner, "", fontLarge, 300, 25+vspace, 1));
      
      li.addSubListPost(oPrintPost(lPunchNamedTime, "", fontMedium, 0, 2, 1));
      li.subListPost.back().fixedWidth=200;
      li.setFilter(EFilterHasResult); 
 
      li.addSubHead(oPrintPost(lClassName, "", boldLarge, 0, 10));

      li.listType=li.EBaseTypeTeam;
      li.listSubType=li.EBaseTypePunches;
      li.sortOrder=ClassResult;
      li.calcResults=true;
      break;

   case EStdResultListLARGE:
      li.addListPost(oPrintPost(lRunnerPlace, "", fontLarge, 0, vspace));
      li.addListPost(oPrintPost(lRunnerName, "", fontLarge, 40, vspace));
      li.addListPost(oPrintPost(lRunnerClub, "", fontLarge, 250, vspace));
      
      li.addListPost(oPrintPost(lRunnerTimeStatus, "", fontLarge, 490, vspace));
      li.setFilter(EFilterHasResult); 
 
      li.addSubHead(oPrintPost(lClassName, "", boldLarge, 0, 10));
      li.addSubHead(oPrintPost(lClassResultFraction, "", fontLarge, 250, 10));

      li.lp.legNumber = 0;
      li.listType=li.EBaseTypeRunner;
      li.sortOrder=ClassResult;
      li.calcResults=true;
      break;

   case EStdUM_Master:
      li.addListPost(oPrintPost(lRunnerPlace, "", fontMedium, 0, 0));
      li.addListPost(oPrintPost(lRunnerName, "", fontMedium, 40, 0));
      li.addListPost(oPrintPost(lRunnerClub, "", fontMedium, 250, 0));
      li.addListPost(oPrintPost(lClassName, "", fontMedium, 490, 0));
      li.addListPost(oPrintPost(lRunnerUMMasterPoint, "", fontMedium, 580, 0));
            
      li.setFilter(EFilterHasResult); 
    
      li.lp.legNumber = 0;
      li.listType=li.EBaseTypeRunner;
      li.sortOrder=ClassResult;
      li.calcResults=true;
      break;

    case EFixedPreReport:
    case EFixedReport:
    case EFixedInForest:
    case EFixedEconomy:
    case EFixedInvoices:
    case EFixedResultFinishPerClass:
    case EFixedResultFinish:
    case EFixedMinuteStartlist:
      li.fixedType = true;
    break;

    default:
      throw std::exception("Not implemented");
  }  
}
