/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

// oRunner.cpp: implementation of the oRunner class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "oRunner.h"

#include "oEvent.h"
#include "gdioutput.h"
#include "table.h"
#include "meos_util.h"
#include <cassert>
#include "localizer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
oAbstractRunner::oAbstractRunner(oEvent *poe):oBase(poe)
{
  Class=0;
  Club=0;
  StartTime=0;
	FinishTime=0;
	Status=StatusUnknown;
}

string oAbstractRunner::getInfo() const
{
  return getName();
}

void oAbstractRunner::setFinishTimeS(const string &t)
{
	setFinishTime(oe->getRelativeTime(t));
}

void oAbstractRunner::setStartTimeS(const string &t)
{
	setStartTime(oe->getRelativeTime(t));
}

oRunner::oRunner(oEvent *poe):oAbstractRunner(poe)
{
	Id=oe->getFreeRunnerId();
	Course=0;
	StartNo=0;
	CardNo=0;

	tInTeam=0;
	tLeg=0;
	tNeedNoCard=false;
  tUseStartPunch=true;
	getDI().initData();

  tDuplicateLeg=0;
  tParentRunner=0;

	Card=0;
	cPriority=0;
}

oRunner::oRunner(oEvent *poe, int id):oAbstractRunner(poe)
{
	Id=id;
  oe->qFreeRunnerId = max(id, oe->qFreeRunnerId);
	Course=0;
	StartNo=0;
	CardNo=0;

	tInTeam=0;
	tLeg=0;
	tNeedNoCard=false;
  tUseStartPunch=true;
	getDI().initData();

  tDuplicateLeg=0;
  tParentRunner=0;

	Card=0;
	cPriority=0;
}


oRunner::~oRunner()
{
	if(tInTeam){
    for(unsigned i=0;i<tInTeam->Runners.size(); i++)
			if(tInTeam->Runners[i] && tInTeam->Runners[i]==this)
				tInTeam->Runners[i]=0;

		tInTeam=0;
	}

  for (size_t k=0;k<multiRunner.size(); k++) {
    if (multiRunner[k])
      multiRunner[k]->tParentRunner=0;  
  }

  if (tParentRunner) {    
    for (size_t k=0;k<tParentRunner->multiRunner.size(); k++) 
      if (tParentRunner->multiRunner[k] == this)
        tParentRunner->multiRunner[k]=0;  
  
  }
}

bool oRunner::Write(xmlparser &xml) 
{
	if(Removed) return true;

	xml.StartTag("Runner");
	xml.Write("Id", Id);
	xml.Write("Updated", Modified.GetStamp());
	xml.Write("Name", Name);
	xml.Write("Start", StartTime);
	xml.Write("Finish", FinishTime);
	xml.Write("Status", Status);
	xml.Write("CardNo", CardNo);
	xml.Write("StartNo", StartNo);

	if(Club) xml.Write("Club", Club->Id);
	if(Class) xml.Write("Class", Class->Id);
	if(Course) xml.Write("Course", Course->Id);

  if (multiRunner.size()>0)
    xml.Write("MultiR", codeMultiR());

  if(Card) {
    assert(Card->tOwner==this);
    Card->Write(xml);
  }
	getDI().write(xml);

	xml.EndTag();

  for (size_t k=0;k<multiRunner.size();k++)
    if(multiRunner[k])
      multiRunner[k]->Write(xml);

	return true;
}

bool oRunner::WriteSmall(xmlparser &xml)
{
	xml.StartTag("Runner");
	xml.Write("Name", Name);
	xml.Write("CardNo", CardNo);

	if(Club) xml.Write("ClubName", Club->Name);
	else xml.Write("ClubName", ClubName);

	//if(Card) Card->Write(xml);

	xml.EndTag();
	return true;
}

void oRunner::Set(const xmlobject *xo)
{
	const xmlList *xl=xo->GetObjects();

	if(xl){
		xmlList::const_iterator it;

		for(it=xl->begin(); it != xl->end(); ++it){
			if(it->Is("Id")){
				Id=atoi(it->Get().c_str());			
			}
			else if(it->Is("Name")){
				Name=it->Get();
			}
			else if(it->Is("Start")){
				StartTime=atoi(it->Get().c_str());			
			}
			else if(it->Is("Finish")){
				FinishTime=atoi(it->Get().c_str());			
			}
			else if(it->Is("Status")){
				Status=RunnerStatus(atoi(it->Get().c_str()));	
			}
			else if(it->Is("CardNo")){
				CardNo=atoi(it->Get().c_str());	
			}
			else if(it->Is("StartNo") || it->Is("OrderId"))
				StartNo=atoi(it->Get().c_str());
			else if(it->Is("Club"))
				Club=oe->getClub(atoi(it->Get().c_str()));
			else if(it->Is("ClubName"))
				ClubName=it->Get();
			else if(it->Is("Class"))
				Class=oe->getClass(atoi(it->Get().c_str()));
			else if(it->Is("Course"))
				Course=oe->getCourse(atoi(it->Get().c_str()));
			else if(it->Is("Card")){
				Card=oe->allocateCard(this);
				Card->Set(&*it);
        assert(Card->getId()!=0);
			}
			else if(it->Is("oData"))
				getDI().set(&*it);
			else if(it->Is("Updated"))
				Modified.SetStamp(it->Get());
      else if(it->Is("MultiR")) 
        decodeMultiR(it->Get());
    }
	}
}

string oRunner::codeMultiR() const
{
  char bf[32];
  string r;

  for (size_t k=0;k<multiRunner.size() && multiRunner[k];k++) {
    if(!r.empty())
      r+=":";
    sprintf_s(bf, "%d", multiRunner[k]->getId());
    r+=bf;
  }
  return r;
}

void oRunner::decodeMultiR(const string &r)
{
  vector<string> sv;
  split(r, ":", sv);  
  multiRunnerId.clear();

  for (size_t k=0;k<sv.size();k++) {
    int d=atoi(sv[k].c_str());
    if (d>0) 
      multiRunnerId.push_back(d);
  }
}

void oAbstractRunner::setClassId(int id)
{
	pClass pc=Class;
  Class=id ? oe->getClass(id):0;
	
  if (Class!=pc) {
    apply(false);
    if(Class)
      oe->reCalculateLeaderTimes(Class->getId());

    if(pc)
      oe->reCalculateLeaderTimes(pc->getId());

		updateChanged();
  }
}

// Update all classes (for multirunner)
void oRunner::setClassId(int id)
{
  if (tParentRunner)
    tParentRunner->setClassId(id);
  else { 
	  pClass pc = Class;
    pClass nPc = id>0 ? oe->getClass(id):0;

    if (pc && pc->isSingleRunnerMultiStage() && nPc!=pc && tInTeam) {
      oe->autoRemoveTeam(this);
    }

    Class = nPc;

    if (Class != 0 && tInTeam==0 && Class->isSingleRunnerMultiStage()) {
      pTeam t = oe->addTeam(getName(), getClubId(), getClassId());
      t->setRunner(0, this, true);
    }

    apply(false); //We may get old class back from team.

    for (size_t k=0;k<multiRunner.size();k++) {
      if (multiRunner[k] && Class!=multiRunner[k]->Class) {
		    multiRunner[k]->Class=Class;
		    multiRunner[k]->updateChanged();
      }
    }

    if(Class!=pc) {
      if(Class)
        oe->reCalculateLeaderTimes(Class->getId());

      if(pc)
        oe->reCalculateLeaderTimes(pc->getId());

		  updateChanged();
    }
	}  
}

void oRunner::setCourseId(int id)
{
	pCourse pc=Course;

	if(id>0)
		Course=oe->getCourse(id);
	else 
		Course=0;

	if(Course!=pc)
		updateChanged();
}

void oAbstractRunner::setStartTime(int t)
{
	int OST=StartTime;

	if(t>0)
		StartTime=t;
	else StartTime=0;

	if(OST!=StartTime)
		updateChanged();

  if(OST<StartTime && Class)
    oe->reCalculateLeaderTimes(Class->getId());
}

void oAbstractRunner::setFinishTime(int t)
{
	int OFT=FinishTime;

	if(t>StartTime)
		FinishTime=t;
	else //Beeb
		FinishTime=0;

  if(OFT!=FinishTime) 
		updateChanged();
}

void oRunner::setFinishTime(int t)
{
  bool update=false;
  if(Class && (getTimeAfter(tDuplicateLeg)==0 || getTimeAfter()==0))
    update=true;
  
  oAbstractRunner::setFinishTime(t);

  if(update && t!=FinishTime)
    oe->reCalculateLeaderTimes(Class->getId());
}

string oAbstractRunner::getStartTimeS() const
{
	if(StartTime>0)
		return oe->getAbsTime(StartTime);
	else return "-";
}

string oAbstractRunner::getStartTimeCompact() const
{
	if(StartTime>0)
    if(oe->useStartSeconds())
		  return oe->getAbsTime(StartTime);
    else
      return oe->getAbsTimeHM(StartTime);

	else return "-";
}

string oAbstractRunner::getFinishTimeS() const
{
	if(FinishTime>0)
		return oe->getAbsTime(FinishTime);
	else return "-";
}

string oAbstractRunner::getRunningTimeS() const
{
  return formatTime(getRunningTime());
}

const string &oAbstractRunner::getStatusS() const
{
  return oEvent::formatStatus(Status);
}

/*
 - Inactive		: Has not yet started
	 - DidNotStart	: Did Not Start (in this race)
	 - Active		: Currently on course
	 - Finished		: Finished but not validated
	 - OK			: Finished and validated
	 - MisPunch		: Missing Punch
	 - DidNotFinish	: Did Not Finish
	 - Disqualified	: Disqualified
	 - NotCompeting	: Not Competing (running outside the competition)
	 - SportWithdr	: Sporting Withdrawal (e.g. helping injured)
	 - OverTime 	: Overtime, i.e. did not finish within max time
	 - Moved		: Moved to another class
	 - MovedUp		: Moved to a "better" class, in case of entry
	 				  restrictions
	 - Cancelled	
*/


string oAbstractRunner::getIOFStatusS() const
{
	switch(Status)
	{
	case StatusOK:
		return "OK";
	case StatusDNS:
		return "DidNotStart";
	case StatusMP:
		return "MisPunch";
	case StatusDNF:
		return "DidNotFinish";
	case StatusDQ:
		return "Disqualified";
	case StatusMAX:
		return "OverTime";
	}

	return "Inactive";
}

void oRunner::addPunches(pCard card, vector<int> &MissingPunches)
{
	Card=card;
	
	updateChanged();

	if(card) {
		if(CardNo==0)
			CardNo=card->CardNo;

		//if(card->CardNo!=CardNo)
		//	oe->Notify("Warning: SI Card number doesn't match: ", CardNo);

    assert(card->tOwner==0 || card->tOwner==this);
	}

  if(Card)
    Card->tOwner=this;

	evaluateCard(MissingPunches, 0, true);
}

pCourse oRunner::getCourse() const
{
	if (Course) return Course;

	if (Class) {		
		if (Class->hasMultiCourse()) {
			const_cast<oRunner*>(this)->apply(false);

      if(tInTeam) {
        unsigned leg=legToRun();
			  if (leg<Class->MultiCourse.size()) {
				  vector<pCourse> &courses=Class->MultiCourse[leg];
				  if(courses.size()>0) {
					  int index=StartNo;
            if(index>0)
              index = (index-1) % courses.size();

					  return courses[index];
				  }
			  }
      }
      else {
			  if (unsigned(tDuplicateLeg)<Class->MultiCourse.size()) {
				  vector<pCourse> &courses=Class->MultiCourse[tDuplicateLeg];
				  if(courses.size()>0) {
					  int index=StartNo % courses.size();
					  return courses[index];
				  }
			  }
      }
		}
		return Class->Course;
	}

	return 0;
}

string oRunner::getCourseName() const
{
	pCourse oc=getCourse();
	if(oc) return oc->getName();
	return "[Ingen bana]";
}

#define NOTATIME 0xF0000000
bool oRunner::evaluateCard(vector<int> & MissingPunches, int addpunch, bool sync)
{
  MissingPunches.clear();
  bool inTeam=apply(sync);

  if(!Card) {
    storeTimes();
		return false;
  }
	//Try to match class?!
	if(!Class)
		return false;

	pCourse course=getCourse();

  if(!course) {
    storeTimes();
		return false;
  }

	if(addpunch && Card->Punches.empty()) {		
    Card->addPunch(addpunch, -1, course->Controls[0] ? course->Controls[0]->getId():0);
	}

	if (Card->Punches.empty()) {
    for(int k=0;k<course->nControls;k++) {
      if (course->Controls[k]) {
        course->Controls[k]->startCheckControl();
        course->Controls[k]->addUncheckedPunches(MissingPunches);
      }
			//MissingPunches.push_back(course->Controls[k]->Numbers[0]);
    }
    storeTimes();
		return false;
	}

	oPunchList::iterator p_it;
	p_it=Card->Punches.begin();
	
  //Search for start and update start time.
  while ( p_it!=Card->Punches.end()) {
	  if (p_it->Type==oPunch::PunchStart) {
		  if(tUseStartPunch && p_it->getAdjustedTime()!=StartTime) {
			  StartTime=p_it->getAdjustedTime();
			  updateChanged();
		  }
	  }
	  ++p_it;
  }
  p_it=Card->Punches.begin();
	
  splitTimes.clear();
  splitTimes.resize(course->nControls, NOTATIME);
  int k=0;
  for (k=0;k<course->nControls;k++) {
    //Skip start finish check
    while(p_it!=Card->Punches.end() && (p_it->isCheck() 
      || p_it->isFinish() || p_it->isStart()))
      ++p_it;

    if(p_it==Card->Punches.end())
      break;

    oPunchList::iterator tp_it=p_it;
		pControl ctrl=course->Controls[k];
    
		if (ctrl) {
      int timeAdjust=ctrl->getTimeAdjust();
      ctrl->startCheckControl();

      while(!ctrl->controlCompleted() && tp_it!=Card->Punches.end()) {
				if (ctrl->hasNumberUnchecked(tp_it->Type)) {
					tp_it->isUsed=true; //Show that this is used when splittimes are calculated.

          // Adjust if the time of this control was incorrect set.
          tp_it->setTimeAdjust(timeAdjust);
          tp_it->tMatchControlId=ctrl->getId();
          if (ctrl->controlCompleted())
            splitTimes[k]=tp_it->getAdjustedTime();
					++tp_it;
					p_it=tp_it;
				}
				else {					
					if(ctrl->hasNumberUnchecked(addpunch)){
						//Add this punch. 
						oPunch op(oe);
						op.Type=addpunch;
						op.Time=-1;
						op.isUsed=true;

            op.tMatchControlId=ctrl->getId();
						Card->Punches.insert(tp_it, op);
						Card->updateChanged();
						if (ctrl->controlCompleted())
              splitTimes[k]=-1;						
					}
					else {
						tp_it->isUsed=false;
						++tp_it;
					}
				}
			}

      if (tp_it==Card->Punches.end() && !ctrl->controlCompleted() 
                    && ctrl->hasNumberUnchecked(addpunch) ) {
        Card->addPunch(addpunch, -1, ctrl->getId());
				if (ctrl->controlCompleted())
          splitTimes[k]=-1;
				Card->Punches.back().isUsed=true;		
        Card->Punches.back().tMatchControlId=ctrl->getId();
			}

      if(ctrl->controlCompleted() && splitTimes[k]==NOTATIME)
        splitTimes[k]=-1;
		}
		else //if(ctrl && ctrl->Status==oControl::StatusBad){
			splitTimes[k]=-1;

    //Add missing punches
    if(ctrl && !ctrl->controlCompleted())
      ctrl->addUncheckedPunches(MissingPunches);
	}



  //Add missing punches for remaining controls
  while (k<course->nControls) {
    if (course->Controls[k]) {
      pControl ctrl = course->Controls[k];
      ctrl->startCheckControl();

      if (ctrl->hasNumberUnchecked(addpunch)) {
        Card->addPunch(addpunch, -1, ctrl->getId());
				Card->updateChanged();
				if (ctrl->controlCompleted())
            splitTimes[k]=-1;						
			}
      ctrl->addUncheckedPunches(MissingPunches);
    }
	  k++;		
  }

	//Set the rest (if exist -- probably not) to "not used"
	while(p_it!=Card->Punches.end()){
		p_it->isUsed=false;
		++p_it;
	}

  int OK=MissingPunches.empty();
  
	int OldStatus=Status;
	int OldFT=FinishTime;

	if(OK && (Status==0 || Status==StatusDNS || Status==StatusMP || Status==StatusOK || Status==StatusDNF))
		Status=StatusOK;
	else	Status=RunnerStatus(max(int(StatusMP), int(Status)));


  if (Card->Punches.back().Type==oPunch::PunchFinish) {
		FinishTime=Card->Punches.back().getAdjustedTime();
    Card->Punches.back().tMatchControlId=oPunch::PunchFinish;
  }
	else if (FinishTime<=0) {
		Status=RunnerStatus(max(int(StatusDNF), int(Status)));
		FinishTime=0;
	}

	if(OldStatus!=Status || OldFT!=FinishTime)
		updateChanged();

  if(inTeam)
    apply(sync); //Post apply. Update start times.

  storeTimes();
	return true;
}

void oRunner::storeTimes()
{
  //Store best time in class
  if (tInTeam) {
    if (Class && unsigned(tLeg)<Class->tLeaderTime.size()) {
      if (Status==StatusOK) {
        int &bt=Class->tLeaderTime[tLeg].first;
        int rt=getRunningTime(); 
        if (bt==0 || rt<bt)
          bt=rt;
      }
      if (tInTeam->getLegStatus(tLeg)==StatusOK) {
        int &bt=Class->tLeaderTime[tLeg].second;      
        int rt=tInTeam->getLegRunningTime(tLeg);
        if(rt>0 && (bt==0 || rt<bt))
          bt=rt;
      }
    }
  }
  else {
    if (Class && unsigned(tDuplicateLeg)<Class->tLeaderTime.size()) {
      if (Status==StatusOK) {
        int &bt=Class->tLeaderTime[tDuplicateLeg].first;
        int rt=getRunningTime(); 
        if (bt==0 || rt<bt)
          bt=rt;
      }
      int &bt=Class->tLeaderTime[tDuplicateLeg].second;
      int rt=getRaceRunningTime(tDuplicateLeg);
      if(rt>0 && (bt==0 || rt<bt))
        bt=rt;
    }
  }
}

int oRunner::getRaceRunningTime(int leg) const
{
  if(tParentRunner)
    return tParentRunner->getRaceRunningTime(leg);

	if (leg==-1)
    leg=multiRunner.size()-1;
		
  if(leg==0) {
    if(statusOK())
      return getRunningTime();
    else return 0;
  }
  leg--;

  if (leg>=0 && multiRunner[leg]) {
    if (Class) {
      pClass pc=Class;
      LegTypes lt=pc->getLegType(leg);
      pRunner r=multiRunner[leg];

      switch(lt) {
        case LTNormal:
          if(r->statusOK()) {            
            int dt=leg>0 ? r->getRaceRunningTime(leg)+r->getRunningTime():0;     
            return max(r->getFinishTime()-StartTime, dt); // ### Luckor, jaktstart???
          }
          else return 0;
        break;

        case LTSum:
          if(r->statusOK())
            return r->getRunningTime()+getRaceRunningTime(leg);
          else return 0;
        
        default:
          return 0;
      }
    }
    else return getRunningTime();
	}
	return 0;
}



bool oRunner::sortSplit(const oRunner &a, const oRunner &b)
{
  int acid=a.getClassId();
  int bcid=b.getClassId();
  if(acid!=bcid)
	  return acid<bcid;
  else if(a.tStatus != b.tStatus)
	  return a.tStatus<b.tStatus;
  else {
	  if(a.tStatus==StatusOK) {
		  if(a.tRT!=b.tRT)
			  return a.tRT<b.tRT;
	  }
	  return a.Name<b.Name;
  }		
}

bool oRunner::operator <(const oRunner &c)
{
	if(!Class || !c.Class) return false;

	if(oe->CurrentSortOrder==ClassStartTime) {
		if(Class->Id < c.Class->Id)
			return true;
		else if(Class->Id > c.Class->Id)
			return false;
    else if (StartTime != c.StartTime) { 
      if (StartTime <= 0 && c.StartTime > 0)
        return false;
      else if (c.StartTime <= 0 && StartTime > 0)
        return true;
      else return StartTime < c.StartTime;
    }
		else if(Name<c.Name)
			return true;
		else return false;
	}
	else if(oe->CurrentSortOrder==ClassResult) {
		if(Class->Id != c.Class->Id)
			return Class->Id < c.Class->Id;
    else if (tDuplicateLeg != c.tDuplicateLeg)
      return tDuplicateLeg < c.tDuplicateLeg;
		else if(Status != c.Status)
			return Status < c.Status;
		else {
			if(Status==StatusOK) {
        if (Class->getNoTiming())
          return Name<c.Name;

				int t=getRunningTime();
				int ct=c.getRunningTime();
				if(t!=ct)
					return t<ct;
			}			
			return Name<c.Name;
		}		
	}
	else if(oe->CurrentSortOrder==SortByName)
		return Name<c.Name;
	else if(oe->CurrentSortOrder==SortByFinishTime){
		if(Status < c.Status)
			return true;
		else if(Status > c.Status)
			return false;
		else{
			if(Status==StatusOK && FinishTime!=c.FinishTime)
				return FinishTime<c.FinishTime;
			else if(Name<c.Name)
				return true;
			else return false;
		}
	}
	else if(oe->CurrentSortOrder==SortByClassFinishTime){
		if(Class->Id < c.Class->Id)
			return true;
		else if(Class->Id > c.Class->Id)
			return false;
		if(Status < c.Status)
			return true;
		else if(Status > c.Status)
			return false;
		else{
			if(Status==StatusOK && FinishTime!=c.FinishTime)
				return FinishTime<c.FinishTime;
			else if(Name<c.Name)
				return true;
			else return false;
		}
	}
	else if(oe->CurrentSortOrder==SortByStartTime){
		if(StartTime < c.StartTime)
			return true;
		else  if(StartTime > c.StartTime)
			return false;
		else if(Name<c.Name)
			return true;
		else return false;
	}
  else throw std::exception("Bad sort order");
}

void oAbstractRunner::setClub(const string &Name)
{
	pClub pc=Club;
  Club = Name.empty() ? 0 : oe->getClubCreate(0, Name);
	if(pc!=Club) updateChanged();
}

pClub oAbstractRunner::setClubId(int clubId)
{
	pClub pc=Club;
  Club = oe->getClub(clubId);
	if(pc!=Club) 
    updateChanged();
  return Club;
}

void oRunner::setClub(const string &Name)
{
  if (tParentRunner)
    tParentRunner->setClub(Name);
  else { 
    oAbstractRunner::setClub(Name);

    for (size_t k=0;k<multiRunner.size();k++) 
      if (multiRunner[k] && multiRunner[k]->Club!=Club) {
		    multiRunner[k]->Club = Club;
		    multiRunner[k]->updateChanged();
      }  
	}
}

pClub oRunner::setClubId(int clubId)
{
  if (tParentRunner)
    tParentRunner->setClubId(clubId);
  else { 
    oAbstractRunner::setClubId(clubId);

    for (size_t k=0;k<multiRunner.size();k++) 
      if (multiRunner[k] && multiRunner[k]->Club!=Club) {
		    multiRunner[k]->Club = Club;
		    multiRunner[k]->updateChanged();
      }  
	}
  return Club;
}


void oAbstractRunner::setStartNo(int no)
{
	if(no!=StartNo) {
		StartNo=no;
		updateChanged();
	}
}

void oRunner::setStartNo(int no)
{
  if (tParentRunner)
    tParentRunner->setStartNo(no);
  else { 
    oAbstractRunner::setStartNo(no);

    for (size_t k=0;k<multiRunner.size();k++) 
      if(multiRunner[k])
        multiRunner[k]->oAbstractRunner::setStartNo(no);
	}
}

int oRunner::getPlace() const
{
	return tPlace;
}

string oAbstractRunner::getPlaceS() const
{
	char bf[16];
  int p=getPlace();
	if(p>0 && p<10000){
		_itoa_s(p, bf, 16, 10);
		return bf;
	}
	else return _EmptyString;
}

string oAbstractRunner::getPrintPlaceS() const
{
	char bf[16];
  int p=getPlace();
	if(p>0 && p<10000){
		_itoa_s(p, bf, 16, 10);
		return string(bf)+".";
	}
  else return _EmptyString;
}

string oRunner::getGivenName() const
{
	return trim(Name.substr(0, Name.find_first_of(' ')));

}
string oRunner::getFamilyName() const
{
  int k=Name.find_first_of(' ');

  if(k==string::npos)
    return "";
  else return trim(Name.substr(k, string::npos));
}

void oRunner::setCardNo(int cno, bool matchCard)
{
	if(cno!=CardNo){
		CardNo=cno;

    if(matchCard && !Card) {
      pCard c=oe->getCardByNumber(cno);

      if(c && !c->tOwner) {
        vector<int> mp;
        addPunches(c, mp);
      }
    }

		updateChanged();
	}
}

int oRunner::setCard(int cardId)
{
  pCard c=cardId ? oe->getCard(cardId) : 0;
  int oldId=0;

  if (Card!=c) {
    if (Card) { 
      oldId=Card->getId();
      Card->tOwner=0;
    }
    if (c) {
      if(c->tOwner) {
        pRunner otherR=c->tOwner;        
        assert(otherR!=this);
        otherR->Card=0;
        otherR->updateChanged();
        otherR->setStatus(StatusUnknown);
        otherR->syncronize(true);
      }
      c->tOwner=this;
      CardNo=c->CardNo;
    }
    Card=c;
    vector<int> mp;
    evaluateCard(mp);
		updateChanged();
    syncronize(true);
	}
  return oldId;
}

void oAbstractRunner::setName(const string &n)
{
	if(n!=Name){
		Name=n;
		updateChanged();
	}
}

void oRunner::setName(const string &n)
{
  if (tParentRunner)
    tParentRunner->setName(n);
  else { 
    if (n!=Name) {
		  Name=n;
		  updateChanged();
    }

    for (size_t k=0;k<multiRunner.size();k++) {
      if (multiRunner[k] && n!=multiRunner[k]->Name) {
		    multiRunner[k]->Name=n;
		    multiRunner[k]->updateChanged();
      }
    }
	}
}


void oAbstractRunner::setStatus(RunnerStatus st)
{
	if(Status!=st){
		Status=st;
		updateChanged();

    if(Class && Status!=StatusOK)
      oe->reCalculateLeaderTimes(Class->getId());
	}
}

int oAbstractRunner::getPrelRunningTime() const
{
  if(FinishTime>0 && Status!=StatusDNS && Status!=StatusDNF) 
		return getRunningTime();
  else if(Status==StatusUnknown)
	  return oe->ComputerTime-StartTime;
	else return 0;
}

string oAbstractRunner::getPrelRunningTimeS() const
{
	int rt=getPrelRunningTime();
	return formatTime(rt);
}

oDataInterface oRunner::getDI(void) 
{
	return oe->oRunnerData->getInterface(oData, sizeof(oData), this);
}

oDataConstInterface oRunner::getDCI(void) const
{
	return oe->oRunnerData->getConstInterface(oData, sizeof(oData), this);
}

pRunner oEvent::getRunner(int Id, int stage)
{
  map<int, pRunner>::const_iterator it = runnerById.find(Id);

  if (it != runnerById.end()) {
    assert(it->second->Id == Id);
    if (stage==0)
      return it->second;
    else if (unsigned(stage)<=it->second->multiRunner.size())
      return it->second->multiRunner[stage-1];
  }
/*
	oRunnerList::iterator it;	

	for (it=Runners.begin(); it != Runners.end(); ++it) {
    if(it->Id==Id) {
      if (stage==0)
        return &*it;
      else if(stage<=it->nMultiR)
        return it->multiRunner[stage-1];
    }
	}*/
	return 0;
}

pRunner oRunner::nextNeedReadout() const
{
  if(!Card || Card->CardNo!=CardNo)
    return pRunner(this);
  
  for (size_t k=0;k<multiRunner.size();k++) {
    if (multiRunner[k] && (!multiRunner[k]->Card || 
           multiRunner[k]->Card->CardNo!=CardNo))
      return multiRunner[k];
  }
  return 0;
}

pRunner oEvent::getRunnerByCard(int CardNo, bool OnlyWithNoCard) const
{
	oRunnerList::const_iterator it;	

	//First try runners with no card read or a different card read.
	for (it=Runners.begin(); it != Runners.end(); ++it) {
    if (it->skip())
      continue;

		if(it->CardNo==CardNo && it->nextNeedReadout())	
      return it->nextNeedReadout();
	}

	if (!OnlyWithNoCard) 	{
		//Then try all runners.
		for (it=Runners.begin(); it != Runners.end(); ++it){
      if(!it->isRemoved() && it->CardNo==CardNo) {	
        pRunner r = it->nextNeedReadout();
        return r ? r : pRunner(&*it);
      }
		}
	}

	return 0;
}

pRunner oEvent::getRunnerByStartNo(int startNo) const
{
  // Several runners can have the same bib, try to get the right one.
  // when looking up for manual speaker times.

	oRunnerList::const_iterator it;	

  // First try via teams (get first non-finished runner in team)
  oTeamList::const_iterator tit;	

  for (tit=Teams.begin(); tit!=Teams.end(); ++tit) {
    if (tit->skip())
      continue;

    if (tit->getStartNo()==startNo) {
      const oTeam &t=*tit;

      for (int leg=0; leg<t.getNumRunners(); leg++) {
        if(t.Runners[leg] && t.Runners[leg]->getStatus()==StatusUnknown)
          return t.Runners[leg];
      }
    }
  }

	//Then try normal runners
	for (it=Runners.begin(); it != Runners.end(); ++it) {
    if (it->skip())
      continue;

    if (it->getStartNo()==startNo) {
      if(it->getNumMulti()==0 || it->Status==StatusUnknown)
        return pRunner(&(*it));
      else {
        for(int race=0;race<it->getNumMulti();race++) {
          pRunner r=it->getMultiRunner(race);
          if (r && r->Status==StatusUnknown)
            return r;
        }
        return pRunner(&(*it));
      }
    }
	}


	return 0;
}

pRunner oEvent::getRunnerByName(const string &pname, const string &pclub) const
{
	oRunnerList::const_iterator it;	

	for (it=Runners.begin(); it != Runners.end(); ++it) {
		if (it->Name==pname) {
			if(pclub=="" || pclub==it->getClub())
				return pRunner(&*it);
		}
	}
	
	return 0;
}

bool oEvent::fillRunners(gdioutput &gdi, string name, 
                         bool longName, bool showAll) 
{	
	syncronizeList(oLRunnerId);
	oRunnerList::iterator it;	

	CurrentSortOrder=SortByName;
	Runners.sort();
	gdi.clearList(name);
  char bf[512];

  if(longName) {
    for (it=Runners.begin(); it != Runners.end(); ++it) {
      if(!it->skip() || (showAll && !it->isRemoved())) {
        sprintf_s(bf, "%s\t%s\t%s", it->getNameAndRace().c_str(),
                                          it->getClass().c_str(),
                                          it->getClub().c_str());

        gdi.addItem(name, bf, it->Id);
      }
	  }
  }
  else {
	  for (it=Runners.begin(); it != Runners.end(); ++it) {
      if(!it->skip() || (showAll && !it->isRemoved())) {
        if( it->getClubId()!=cVacantId )
			    gdi.addItem(name, it->Name, it->Id);
        else {
          sprintf_s(bf, "%s (%s)", it->Name.c_str(), it->getClass().c_str());
          gdi.addItem(name, bf, it->Id);
        }
      }
	  }
  }
	return true;
}

void oRunner::resetPersonalData()
{
	getDI().initData();
}

string oRunner::getNameAndRace() const
{
  if (tDuplicateLeg>0 || multiRunner.size()>0) {
    char bf[16];
    sprintf_s(bf, " (%d)", getRaceNo()+1);
    return Name+bf;
  } 
  else return Name;
}

pRunner oRunner::getMultiRunner(int race) const
{
  if (race==0) {
    if(!tParentRunner)
      return pRunner(this);
    else return tParentRunner;
  }
  
  if (unsigned(race-1)>=multiRunner.size()) {
    assert(tParentRunner);
    return 0;
  }

  if (!tParentRunner)
    return multiRunner[race-1];
  else
    return tParentRunner->multiRunner[race-1];
}

void oRunner::createMultiRunner(bool createMaster, bool sync)
{
  if(tDuplicateLeg)
    return; //Never allow chains.

  if (multiRunnerId.size()>0) {
    multiRunner.resize(multiRunnerId.size());
    for (size_t k=0;k<multiRunner.size();k++) {
      multiRunner[k]=oe->getRunner(multiRunnerId[k], 0);
      if (multiRunner[k]) {
        multiRunner[k]->multiRunner.clear(); //Do not allow chains
        multiRunner[k]->multiRunnerId.clear();
        multiRunner[k]->tDuplicateLeg = k+1;
        multiRunner[k]->tParentRunner = this;
        multiRunner[k]->CardNo=0;
      }
    }
    multiRunnerId.clear();
  }

  if (!Class || !createMaster)
    return;

  int ndup=0;

  if (!tInTeam) 
    ndup=Class->getNumMultiRunners(0);
  else
    ndup=Class->getNumMultiRunners(tLeg);
  
  bool update = false;
  multiRunner.resize(ndup-1);
  for(int k=1;k<ndup; k++) {
    if (!multiRunner[k-1]) {
      update = true;
      multiRunner[k-1]=oe->addRunner(Name, Club ? Club->getId(): 0, 
                                           Class ? Class->getId():0, false);
      multiRunner[k-1]->tDuplicateLeg=k;
      multiRunner[k-1]->tParentRunner=this;

      if (sync)
        multiRunner[k-1]->syncronize();
    }
  }
  if (update) 
    updateChanged();

  if (sync)
    syncronize(true);
}

pRunner oRunner::getPredecessor() const
{
  if(!tParentRunner || unsigned(tDuplicateLeg-1)>=16)
    return 0;
  
  if(tDuplicateLeg==1)
    return tParentRunner;
  else
    return tParentRunner->multiRunner[tDuplicateLeg-2];
}

bool oRunner::apply(bool sync) 
{
  createMultiRunner(false, sync);
	tLeg=-1;
  tUseStartPunch=true;
	if(tInTeam)
		tInTeam->apply(sync);
  else {
    if (Class && Class->hasMultiCourse()) {
      pClass pc=Class;
      StartTypes st=pc->getStartType(tDuplicateLeg);
      //LegTypes lt=pc->getLegType(tDuplicateLeg);

      if (st==STTime) {
        setStartTime(pc->getStartData(tDuplicateLeg));
      }
      else if (st==STChange) {
        pRunner r=getPredecessor();
        int lastStart=0;
        if(r && r->FinishTime>0) 
          lastStart = r->FinishTime;

        int restart=pc->getRestartTime(tDuplicateLeg);
        int rope=pc->getRopeTime(tDuplicateLeg);

        if (restart && rope && (lastStart>rope || lastStart==0))
          lastStart=restart; //Runner in restart

        setStartTime(lastStart);
        tUseStartPunch=false; 
      }
      else if (st==STHunting) {
        pRunner r=getPredecessor();
        int lastStart=0;

        if (r && r->FinishTime>0 && r->statusOK()) {
          int rt=r->getRaceRunningTime(tDuplicateLeg-1);
          int timeAfter=rt-pc->getTotalLegLeaderTime(r->tDuplicateLeg);
          if(rt>0 && timeAfter>=0)
            lastStart=pc->getStartData(tDuplicateLeg)+timeAfter;
        }
        int restart=pc->getRestartTime(tDuplicateLeg);
        int rope=pc->getRopeTime(tDuplicateLeg);

        if (restart && rope && (lastStart>rope || lastStart==0))
          lastStart=restart; //Runner in restart

        setStartTime(lastStart);
        tUseStartPunch=false;  
      }

    }
  }

	if (tLeg==-1) {
		tLeg=0;
		tInTeam=0;
    return false;
	}
  else return true;
}

/*
Table *oEvent::getRunnersTB()//Table mode
{	
	oClassList::iterator it;	
	SyncronizeList(oLClassId);

	Table *table=new Table(20, "Deltagare");

	table->addColumn("Id", 40, true);
	table->addColumn("Klass", 120, false);
	table->addColumn("Bana", 90, false);
	table->addColumn("Deltagare", 120, true);
	table->addColumn("TS", 120, true);
	//table
	
	for (it=Classes.begin(); it != Classes.end(); ++it){		
		if(!it->Removed){
			table->addRow();
		
			char num[16];
			_itoa_s(it->getId(), num, 10);
			table->set(0, *it, TID_ID, num, false);

			table->set(1, *it, TID_CLASSNAME, it->GetName());
			
			pCourse pc=it->getCourse();
			if(pc) table->set(2, *it, TID_COURSE, pc->GetName());
			else table->set(2, *it, TID_COURSE, "-", true, true);
			
			//char num[10];
			_itoa_s(it->GetNumRunners(), num, 10);
			table->set(3, *it, TID_NUM, num, false);

			table->set(4, *it, TID_MODIFIED, it->Modified.GetStamp(), false);
		}
	}
	return table;
}
*/

Table *oEvent::getRunnersTB()//Table mode
{	
	Table *table=new Table(this, 20, "Deltagare", "runners");

	table->addColumn("Namn", 200, false);
	table->addColumn("Klass", 120, false);
  table->addColumn("Bana", 120, false);
	
	table->addColumn("Klubb", 120, false);
  table->addColumn("SI", 70, true);

  table->addColumn("Start", 70, false);
  table->addColumn("Ml", 70, false);
  table->addColumn("Status", 70, false);
  table->addColumn("Tid", 70, false);
  table->addColumn("Plac.", 70, false);

  oe->oRunnerData->buildTableCol(table);

  table->update();
  
  return table;
}

void oEvent::generateRunnerTableData(Table *table)
{ 
  syncronizeList(oLRunnerId);
	oRunnerList::iterator it;	

  for (it=Runners.begin(); it != Runners.end(); ++it){		
		if(!it->skip()){
      table->addRow(it->getId());
					
      table->set(0, *it, TID_RUNNER, it->getName(), true);
			table->set(1, *it, TID_CLASSNAME, it->getClass(), true, cellSelection);			
      table->set(2, *it, TID_COURSE, it->getCourseName(), true, cellSelection);			
      table->set(3, *it, TID_CLUB, it->getClub(), true, cellSelection);
			
      char bf[16];
      _itoa_s(it->getCardNo(), bf, 10);

      table->set(4, *it, TID_CARD, bf , true);
			
      table->set(5, *it, TID_START, it->getStartTimeS(), true);
			table->set(6, *it, TID_FINISH, it->getFinishTimeS(), true);
      table->set(7, *it, TID_STATUS, it->getStatusS(), true, cellSelection);
      table->set(8, *it, TID_RUNNINGTIME, it->getRunningTimeS(), false);
      
      table->set(9, *it, TID_PLACE, it->getPlaceS(), false);
			
      oe->oRunnerData->fillTableCol(it->oData, *it, table);
		}
	}
}

bool oRunner::inputData(int id, const string &input, 
                        int inputId, string &output, bool noUpdate)
{
  int t,s;
  vector<int> mp;

  if(id>1000)
    return oe->oRunnerData->inputData(oData, id, input, output, noUpdate);

  switch(id) {
    case TID_RUNNER:
      setName(input);
      syncronizeAll();
      output=getName();
      return true;
    break;

    case TID_START:
      setStartTimeS(input);
      t=getStartTime();      
      evaluateCard(mp);
      s=getStartTime();
      if (s!=t) 
        throw std::exception("Starttiden r definerad genom klassen eller lparens startstmpling.");
      syncronize(true);
      output=getStartTimeS();
      return true;
    break;

    case TID_FINISH:
      setFinishTimeS(input);
      t=getFinishTime();      
      evaluateCard(mp);
      s=getFinishTime();
      if (s!=t) 
        throw std::exception("Fr att ndra mltiden mste lparens mlstmplingstid ndras.");
      syncronize(true);
      output=getStartTimeS();
      return true;
    break;

    case TID_COURSE:
      setCourseId(inputId);
      syncronize(true);
      output = getCourseName();
      break;

    case TID_CLUB: 
      {
        pClub pc = oe->getClub(inputId);
        setClub(pc ? pc->getName() : "");
        syncronize(true);
        output = getClub();
      }
      break;

    case TID_CLASSNAME:
      setClassId(inputId);
      syncronize(true);
      output = getClass();
      break;

    case TID_STATUS:
      setStatus(RunnerStatus(inputId));
      int s = getStatus();
      evaluateCard(mp);
      if (s!=getStatus())
        throw std::exception("Status matchar inte data i lparbrickan.");
      syncronize(true);
      output = getStatusS();
      break;
  }

  return false;
}

void oRunner::fillInput(int id, gdioutput &gdi, const string &controlId)
{ 
  if (id==TID_COURSE) {
    oe->fillCourses(gdi, controlId, true);
    gdi.addItem(controlId, lang.tl("Klassens bana"), 0);
    gdi.selectItemByData(controlId.c_str(), Course ? Course->getId() : 0);
  }
  else if (id==TID_CLASSNAME) {
    oe->fillClasses(gdi, controlId, oEvent::extraNone, oEvent::filterNone);
    gdi.addItem(controlId, lang.tl("Ingen klass"), 0);
    gdi.selectItemByData(controlId.c_str(), getClassId());
  }  
  else if (id==TID_CLUB) {
    oe->fillClubs(gdi, controlId);
    gdi.addItem(controlId, lang.tl("Klubbls"), 0);
    gdi.selectItemByData(controlId.c_str(), getClubId());
  }
  else if (id==TID_STATUS) {
    oe->fillStatus(gdi, controlId);
    gdi.selectItemByData(controlId.c_str(), getStatus());
  }
}

int oRunner::getSplitTime(int controlNumber) const
{
  if (controlNumber>0 && controlNumber==splitTimes.size() && FinishTime>0)
    return max(FinishTime - splitTimes.back(), -1);
  if ( unsigned(controlNumber)<splitTimes.size() ) {
    if(controlNumber==0)
      return (StartTime>0 && splitTimes[0]>0) ? max(splitTimes[0]-StartTime, -1) : -1;
    else if (splitTimes[controlNumber]>0 && splitTimes[controlNumber-1]>0)
      return max(splitTimes[controlNumber] - splitTimes[controlNumber-1], -1);
    else return -1;
  }
  return -1;
}

int oRunner::getNamedSplit(int controlNumber) const
{
  pCourse crs=getCourse();
  if(!crs || unsigned(controlNumber)>=unsigned(crs->nControls)
        || unsigned(controlNumber)>=splitTimes.size())
    return -1;

  pControl ctrl=crs->Controls[controlNumber];
  if(!ctrl || !ctrl->hasName()) 
    return -1;

  int k=controlNumber-1;

  //Measure from previous named control
  while(k>=0) {
    pControl c=crs->Controls[k];
  
    if(c && c->hasName()) {
      if (splitTimes[controlNumber]>0 && splitTimes[k]>0)
        return max(splitTimes[controlNumber] - splitTimes[k], -1);
      else return -1;
    }
    k--;
  }

  //Measure from start time
  if(splitTimes[controlNumber]>0) 
    return max(splitTimes[controlNumber]-StartTime, -1);

  return -1;
}

string oRunner::getSplitTimeS(int controlNumber) const
{
  return formatTime(getSplitTime(controlNumber));
}

string oRunner::getNamedSplitS(int controlNumber) const
{
  return formatTime(getNamedSplit(controlNumber));
}

int oRunner::getPunchTime(int controlNumber) const
{
  if ( unsigned(controlNumber)<splitTimes.size() ) {
    if(splitTimes[controlNumber]>0)
      return splitTimes[controlNumber]-StartTime;
    else return -1;
  }
  else if ( unsigned(controlNumber)==splitTimes.size() )
    return FinishTime-StartTime;

  return -1;
}

string oRunner::getPunchTimeS(int controlNumber) const
{
  return getTimeMS(getPunchTime(controlNumber));
}

bool oRunner::isVacant() const
{
  return getClubId()==oe->getVacantClub();
}

bool oRunner::needNoCard() const
{
  const_cast<oRunner*>(this)->apply(false);
  return tNeedNoCard;
}

void oRunner::getSplitTime(int controlId, RunnerStatus &stat, int &rt) const
{
  stat=StatusUnknown;
  int cardno = tParentRunner ? tParentRunner->CardNo : CardNo;
  if (controlId==oPunch::PunchFinish && 
      FinishTime>0 && Status!=StatusUnknown) {
    stat=Status;
    rt=FinishTime;
  }
  else if (Card) {
		oPunch *p=Card->getPunchById(controlId);
		if (p && p->Time>0) {
      rt=p->getAdjustedTime();
      stat = StatusOK;
		}
    else 
      stat = controlId==oPunch::PunchFinish ? StatusDNF: StatusMP;
	}
  else if (cardno)	{
		oFreePunch *fp=oe->getPunch(getRaceNo(), controlId, cardno);

		if (fp) {
      rt=fp->getAdjustedTime();
      stat=StatusOK;
		}
    if(controlId==oPunch::PunchFinish && Status!=StatusUnknown)
      stat = Status;
	}
  rt-=StartTime;

  if(rt<0)
    rt=0;
}

void oRunner::fillSpeakerObject(int leg, int controlId, oSpeakerObject &spk) const
{
  spk.runningTime=0;
  spk.status=StatusUnknown;
  spk.owner=const_cast<oRunner *>(this);

  spk.preliminaryRunningTimeLeg = 0;
  spk.runningTimeLeg = 0;

  getSplitTime(controlId, spk.status, spk.runningTime);

  int bib=getBib();
  char name[256];
  if(bib>0) {
    sprintf_s(name, "%d %s", bib, getName().c_str());
    spk.name=name;
  }
  else
    spk.name=getName();

  spk.club=getClub();
  spk.finishStatus=getStatus();

  spk.startTimeS=getStartTimeCompact();
  spk.missingStartTime = StartTime<=0;
  
  spk.isRendered=false;

	map<int,int>::const_iterator mapit=Priority.find(controlId);
	if(mapit!=Priority.end())
		spk.priority=mapit->second;
  else 
    spk.priority=0;

  spk.preliminaryRunningTime=getPrelRunningTime();
  
  if(spk.status==StatusOK) {
    spk.runningTimeLeg=spk.runningTime;
    spk.preliminaryRunningTime=spk.runningTime;
    spk.preliminaryRunningTimeLeg=spk.runningTime;
  }
  else {
    spk.runningTimeLeg = spk.preliminaryRunningTime;
    spk.preliminaryRunningTimeLeg = spk.preliminaryRunningTime;
  }
}

pRunner oEvent::findRunner(const string &s, int lastId) const
{
  int len=s.length();
  oRunnerList::const_iterator itstart=Runners.begin();

  if(lastId) 
    for (; itstart != Runners.end(); ++itstart) 
      if(itstart->Id==lastId) {
        ++itstart;
        break;  
      }

  if(!len)
    return 0;

  int sn=atoi(s.c_str());
  oRunnerList::const_iterator it;

  if (sn>0) {
     for (it=itstart; it != Runners.end(); ++it) {  
       if (it->skip())
         continue;
       if(it->StartNo==sn || it->CardNo==sn)
        return pRunner(&*it);
    }
    for (it=Runners.begin(); it != itstart; ++it) { 
      if (it->skip())
         continue;
      if(it->StartNo==sn || it->CardNo==sn)
        return pRunner(&*it);
    }
  }

  char s_lc[1024];
  strcpy_s(s_lc, s.c_str());
  CharLowerBuff(s_lc, len);

  for (it=itstart; it != Runners.end(); ++it) {  
    if (it->skip())
       continue;
    if(filterMatchString(it->Name, s_lc))
      return pRunner(&*it);
  }
  for (it=Runners.begin(); it != itstart; ++it) {  
    if (it->skip())
      continue;
    if(filterMatchString(it->Name, s_lc))
      return pRunner(&*it);
  }
  return 0;
}

int oRunner::getTimeAfter(int leg) const
{
  if(leg==-1)
    leg=tDuplicateLeg;

  if(!Class || Class->tLeaderTime.size()<=unsigned(leg))
    return -1;

  int t=getRaceRunningTime(leg);

  if (t<=0)
    return -1;

  return t-Class->getTotalLegLeaderTime(leg);
}

int oRunner::getTimeAfter() const
{
  int leg=0;
  if(tInTeam)
    leg=tLeg;
  else 
    leg=tDuplicateLeg;

  if(!Class || Class->tLeaderTime.size()<=unsigned(leg))
    return -1;

  int t=getRunningTime();

  if (t<=0)
    return -1;

  return t-Class->getBestLegTime(leg);
}

bool oRunner::syncronizeAll()
{
  if (tParentRunner)
    tParentRunner->syncronizeAll();
  else { 
    syncronize();
    for (size_t k=0;k<multiRunner.size();k++) {
      if (multiRunner[k]) 
        multiRunner[k]->syncronize();
    }
	}
  return true;
}

int oRunner::getBib() const
{
  return getDCI().getInt("Bib");
}

void oRunner::setBib(int bib)
{
  if (tParentRunner)
    tParentRunner->setBib(bib);
  else {
    setStartNo(bib);
    getDI().setInt("Bib", bib);
    
    for (size_t k=0;k<multiRunner.size();k++) 
      if(multiRunner[k])
        multiRunner[k]->getDI().setInt("Bib", bib);
	}
}


void oEvent::analyseDNS(vector<pRunner> &unknown_dns, vector<pRunner> &known_dns, 
                        vector<pRunner> &known, vector<pRunner> &unknown)
{
  autoSyncronizeLists(true);
  
  vector<pRunner> stUnknown;
  vector<pRunner> stDNS;

  for (oRunnerList::iterator it = Runners.begin(); it!=Runners.end();++it) {
    if (!it->isRemoved()) {
      if (it->getStatus() == StatusUnknown)
        stUnknown.push_back(&*it);
      else if(it->getStatus() == StatusDNS)
        stDNS.push_back(&*it);
    }
  }

  set<int> knownPunches;
  for (oFreePunchList::iterator it = punches.begin(); it!=punches.end(); ++it) {
    knownPunches.insert(it->getCardNo());
  }

  set<int> knownCards;
  for (oCardList::iterator it = Cards.begin(); it!=Cards.end(); ++it) {
    if (it->tOwner == 0)
      knownCards.insert(it->CardNo);
  }

  unknown.clear();
  known.clear();

  for (size_t k=0;k<stUnknown.size();k++) {
    int card = stUnknown[k]->CardNo;
    if (card == 0)
      unknown.push_back(stUnknown[k]);
    else if (knownCards.count(card)==1 || knownPunches.count(card)==1)
      known.push_back(stUnknown[k]);
    else
      unknown.push_back(stUnknown[k]); //These can be given "dns"
  }

  unknown_dns.clear();
  known_dns.clear();

  for (size_t k=0;k<stDNS.size(); k++) {
    int card = stDNS[k]->CardNo;
    if (card == 0)
      unknown_dns.push_back(stDNS[k]);
    else if (knownCards.count(card)==1 || knownPunches.count(card)==1)
      known_dns.push_back(stDNS[k]);
    else
      unknown_dns.push_back(stDNS[k]); 
  }
}

static int findNextControl(const list<pControl> &ctrl, int startIndex, int id)
{
  list<pControl>::const_iterator it=ctrl.begin();
  int index=0;
  while(startIndex>0 && it!=ctrl.end())
    ++it, --startIndex, ++index;

  while(it!=ctrl.end() && (*it) && (*it)->getId()!=id)
    ++it, ++index;

  if (it==ctrl.end())
    return -1;
  else
    return index;
}

void oRunner::printSplits(gdioutput &gdi) const {
  gdi.setCX(10);
  gdi.fillDown();
  gdi.addString("", boldText, oe->getName());
  gdi.addString("", fontSmall, oe->getDate());
  gdi.dropLine(0.5);
  
  gdi.addString("", boldSmall, getName() + ", " + getClass());
  gdi.addString("", fontSmall, getClub());
  gdi.dropLine(0.5);
  gdi.addString("", fontSmall, "Start: " + getStartTimeS() + ", Ml: " + getFinishTimeS());
  gdi.addString("", fontSmall, "Status: " + getStatusS() + ", Tid: " + getRunningTimeS());
  int cy = gdi.getCY();
  int cx = gdi.getCX();
  int pnr = 1;
  const int c1=20;
  const int c2=79;
  const int c3=125;
  char bf[256];
  int lastIndex = -1;

  list<pControl> ctrl;
  pCourse pc = getCourse();
  if (pc)
    pc->getControls(ctrl);
  
  if (Card) {
    oPunchList &p=Card->Punches;
    for (oPunchList::iterator it=p.begin();it!=p.end();++it) {
      int cid = it->tMatchControlId;

      if (it->isFinish()) {
        sprintf_s(bf, "%d.", pnr++);
        gdi.addString("", cy, cx, fontSmall, bf);
        gdi.addString("", cy, cx+c1, fontSmall, "Ml");
        int sp = getSplitTime(splitTimes.size());
        if (sp>0)
          gdi.addString("", cy, cx+c2, fontSmall, formatTime(sp));
        gdi.addString("", cy, cx+c3, fontSmall, oe->getAbsTime(it->Time));
        cy+=gdi.getLineHeight();
      }
      else if (it->Type>10) { //Filter away check and start
        sprintf_s(bf, "%d.", pnr++);
        gdi.addString("", cy, cx, fontSmall, bf);
        int index = -1;
        if (cid>0)
          index = findNextControl(ctrl, lastIndex+1, cid);
        if (index>=0) {
          lastIndex = index;
          sprintf_s(bf, "%d (%d)", index+1, it->Type);
          gdi.addString("", cy, cx+c1, fontSmall, bf);

          int sp = getSplitTime(index);
          if (sp>0)
            gdi.addString("", cy, cx+c2, fontSmall, formatTime(sp));
        }
        else {
          sprintf_s(bf, "- (%d)", it->Type);
          gdi.addString("", cy, cx+c1, fontSmall, bf);
        }
        gdi.addString("", cy, cx+c3, fontSmall, oe->getAbsTime(it->Time));
        cy+=int(gdi.getLineHeight()*0.9);
      }
    }
  }
  gdi.dropLine();
  gdi.addString("", fontSmall, "Av MeOS: www.melin.nu/meos/sv");
  
}
