/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

// xmlparser.cpp: implementation of the xmlparser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "xmlparser.h"
#include "meos_util.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

string xmlobject::emptystring;

xmlparser::xmlparser()
{
	xmlobj=0;
	tagStackPointer=0;	

	OnlyData=false;
}

xmlparser::~xmlparser()
{
	//if(fin.good())
	fin.close();
	fout.close();

	if(xmlobj)
		delete xmlobj;	
}


xmlobject::xmlobject()
{
	isclosed=false;
//	objects=0;
	parent=0;
	
}


xmlobject::~xmlobject()
{
	//MessageBox(NULL, name.c_str(), "Destroying: ", MB_OK);

//	if(objects) delete objects;
}

bool xmlparser::Write(const char *tag, const string &Value)
{
	if(Value!="")	{		
		fout << "<" << tag << ">";
		fout << encodeXML(Value);
		fout << "</" << tag << ">" << endl;
	}
	return fout.good();
}

bool xmlparser::Write(const char *tag, const char *Property, const string &Value)
{
	if(Value!="")	{		
		fout << "<" << tag << " " << Property << "=\"";
		fout << encodeXML(Value) << "\"/>" << endl;		
	}
	return fout.good();
}

bool xmlparser::Write(const char *tag, const char *Property, const string &PropValue, const string &Value)
{
	if(Value!="")	{		
		fout << "<" << tag << " " << Property << "=\"";
		fout << encodeXML(PropValue) << "\">" << encodeXML(Value);		
    fout << "</" << tag << ">" << endl;
	}
	return fout.good();
}

bool xmlparser::Write(const char *tag, int Value)
{
	if(Value!=0) {
		fout << "<" << tag << ">";
		fout << Value;
		fout << "</" << tag << ">" << endl;
	}
	return fout.good();
}


bool xmlparser::StartTag(const char *tag, const char *property, const string &Value)
{
	if(tagStackPointer<32) {
		fout << "<" << tag << " " << property << "=\"" << encodeXML(Value) << "\">" << endl;
		tagStack[tagStackPointer++]=tag;
		return fout.good();
	}
	else return false;
}

bool xmlparser::StartTag(const char *tag)
{
	if(tagStackPointer<32) {
		fout << "<" << tag << ">" << endl;
		tagStack[tagStackPointer++]=tag;
		return fout.good();
	}
	else return false;
}

bool xmlparser::EndTag()
{
	if(tagStackPointer>0)	{
		fout << "</" << tagStack[--tagStackPointer] << ">" << endl;
		return fout.good();
	}
  else throw std::exception("BAD XML CODE");

	return false;
}


bool xmlparser::OpenOutput(const char *file, const string &type)
{
	fout.open(file);

	tagStackPointer=0;

	if(fout.bad())
		return false;

	fout << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n\n\n";

  if (!StartTag(type.c_str())) {
    CloseOut();
    return false;
  }

	return true;
}

bool xmlparser::CloseOut()
{
	while(tagStackPointer>0)
		EndTag();

	fout.close();
	return true;
}

void xmlobject::Set(const string &pname, const string &pvalue)
{
	name=pname;
  value=decodeXML(pvalue);
}

void xmlobject::Set(char *pname, char *pvalue)
{
	name=pname;
	value=decodeXML(pvalue);
}

bool xmlobject::AddObject(xmlobject *xo)
{
//	if(!objects)
//		objects=new xmlList;

	objects.push_back(*xo);

	return true;
}

const xmlobject *xmlobject::GetObject(const char *pname) const
{
	if(!objects.empty()) {
		xmlList::const_iterator it;	

		for(it =objects.begin(); it != objects.end(); ++it){
			if(it->name==pname)
				return &*it;
		}
	}

	if(name==pname)
		return this;

	return 0;
}

bool xmlparser::Read(const char *file)
{
	fin.open(file);

	if(!fin.good())
		return false;

	if(xmlobj) delete xmlobj;

	xmlobj=new xmlobject;

	char bf[1024];
	bf[0]=0;

	do
	{
		fin.getline(bf, 1024);	
		LineNumber++;
	}
	while(fin.good() && bf[0]==0);

	char *ptr=ltrim(bf);
	
	if(memcmp(ptr, "<?xml", 5)!=0)
		return false;

	LineNumber=0;
	return ReadXMLData(xmlobj);
}


bool xmlparser::Open(const char *file)
{
	ErrorMessage="";
	
	
	//fin.close();
	//fin.clear();

	fin.open(file);

	if(!fin.good()){
		ErrorMessage=string("Cannot open file for reading. ") + file;
		return false;
	}

	if(xmlobj) delete xmlobj;
	xmlobj=new xmlobject;

	char bf[1024];
	bf[0]=0;

	do{
		fin.getline(bf, 1024);	
		LineNumber++;
	}
	while(fin.good() && bf[0]==0);

	char *ptr=ltrim(bf);
	
	if(memcmp(ptr, "<?xml", 5)!=0){
		ErrorMessage=string("File is not recognized as XML data. ") + file;	
		return false;
	}

	LineNumber=0;
	return true;
}


bool xmlparser::ParseTag(xmlobject *xo, string &tag)
{
	int pos=tag.find_first_of(' ');

	if(pos==string::npos)
		return true;

	const char *str=tag.c_str();
	int n=pos;

	while(str[n]){
		
		while(str[n] && (str[n]==' ' || str[n]=='\t')) n++; //Left trim
		
		char name[256];
		char value[256];
		
		int i=0;
		int o=0;

		//Extract name
		while(str[n] && str[n]!='=' && i<255) name[i++]=str[n++];
		name[i]=0;

		if(str[n]=='='){
			n++;
			while(str[n] && (str[n]==' ' || str[n]=='\t')) n++; //Left trim
			
			if(str[n]=='"'){
				n++;
				while(str[n] && str[n]!='"' && o<255) value[o++]=str[n++];
				value[o]=0;
			}

			if(str[n]=='"') n++;
		}
		
		if(i>0 && o>0){
			xmlobject x;
			x.parent=xo;
			x.Set(name, value);
			x.isclosed=true;
			xo->AddObject(&x);	
		}
	}

	//tag[pos]=0;
	tag=tag.substr(0, pos);
	return true;
}
/*
bool xmlparser::ParseTag(xmlobject *xo, string &tag)
{
	//xo->AddObject(
	int pos=tag.find_first_of(' ');

	string newtag=tag.substr(0, pos);
	string prop;

	//Extract variables
	if(pos!=string::npos){
		prop=tag.substr(pos, string::npos);

		int eqpos=prop.find_first_of('=');
		
		while(eqpos!=string::npos){
			
			int trimpos=prop.find_first_not_of(" \t");
			if(trimpos!=string::npos && trimpos<eqpos){
				string name=prop.substr(trimpos, eqpos-1); 
				prop=prop.substr(eqpos+1);
				const char *v=ltrim(prop.c_str());					
				string value;
				if(prop[0]=='"'){
					
					int k=1;
					while(v[k] && v[k]!='"')
						value+=v[k++];

					//xmlobject *x=new xmlobject;
					xmlobject x;
					x.parent=xo;
					x.Set(name, value);
					x.isclosed=true;
					xo->AddObject(&x);	

					prop=prop.substr(k);
				}
			}
			else prop[eqpos]='_'; //Some syntax error...

			eqpos=prop.find_first_of('=');
		}

	}
	tag=newtag;
	return true;
}
*/

bool xmlparser::ParseXMLData(char *inbf, xmlobject *xo)
{
	//if(!xo) return;

	char *s=ltrim(inbf);
	char str[256];

	if(!s[0]) 
		return true;

	string tag;
	string value;

	if(*s!='<')
		return true;

	s++;

	if(*s=='!'){
		while(*s && *s!='>'){
			doctype+=*s;
			s++;
		}					
	}
	else if(*s!='/')//StartTag
	{
		while(*s && *s!='>' && !(s[0]=='/' && s[1]=='>') ){
			tag+=*s;
			s++;
		}
		bool onetag=false;

		if(*s=='/'){
			s++;
			onetag=true;
		}
		
		if(*s!='>'){
			sprintf_s(str, "XML Syntax Error: Line %d.", LineNumber);
			ErrorMessage=str;
			return false;
		}
		xmlobject x_new;
		ParseTag(&x_new, tag);

		s++;
		s=ltrim(s);

		if(onetag){
			if(x_new.objects.empty())
				x_new.Set(tag, "");
			else
				x_new.Set(tag, x_new.objects.front().value);

			x_new.parent=xo;
			x_new.isclosed=true;
			xo->AddObject(&x_new);
			return ParseXMLData(s, xo);
		}
		else if(*s && *s!='<'){
			char *data=s;
			while(*s && *s!='<') s++;
			*s=0;
			value=data;
			
			s=ltrim(s+1);

			if(*s) s++;

			char *endtag=s;
			while(*s && *s!='>') s++;
			*s=0;
			
			if(strcmp(endtag, tag.c_str())!=0){
				sprintf_s(str, "XML Error. Tag mismatch: Line %d.", LineNumber);
				ErrorMessage=str;
				return false;
			}

			//Create XML-object and insert.
			//xmlobject *x=new xmlobject;
			x_new.parent=xo;
			x_new.Set(tag, value);
			x_new.isclosed=true;
			xo->AddObject(&x_new);	

			return ParseXMLData(s+1, xo);
		}
		else
		{
			if(xo->name==""){
				xo->name=tag;
				return ParseXMLData(s, xo);
			}
			else
			{
				//xmlobject *x=new xmlobject;
				x_new.parent=xo;
				x_new.Set(tag, "");
				
				if(OnlyData){
					xo->AddObject(&x_new);

					KillProcess=true;
					return true;
				}
				else{
					if(!ParseXMLData(s, &x_new))
						return false;

					if(x_new.isclosed){
						xo->AddObject(&x_new);
						return true;
					}
					else if(!ReadXMLData(&x_new))
						return false;	

					xo->AddObject(&x_new);	

					return true;
				}
			}
		}
	}
	else //End tag
	{		
		s++;
		while(*s && *s!='>'){
			tag+=*s;
			s++;
		}

		if(*s!='>'){
			sprintf_s(str, "XML Syntax Error: Line %d.", LineNumber);
			ErrorMessage=str;
			return false;
		}

		if(xo->name!=tag){
			sprintf_s(str, "XML Error. Tag mismatch: Line %d.", LineNumber);
			ErrorMessage=str;
			return false;
		}
		xo->isclosed=true;

		return ParseXMLData(s+1, xo->parent);
	}
	return true;
}



bool xmlparser::ReadXMLData(xmlobject *xo)
{
	char bf[1024];
	KillProcess=false;
	while(fin.good() && !KillProcess){
		fin.getline(bf, 1024);
		LineNumber++;
		if(!ParseXMLData(bf, xo))
			return false;

		if(xo->isclosed) 
			return true;
	}

	return true;
}


bool xmlparser::Read(int MaxObj)
{
	if(!fin.good()) 
		return false;

	if(!xmlobj) 
		return false;

	char bf[1024];
	KillProcess=false;
	//int rcount=0;

	while(fin.good() && !KillProcess){
		fin.getline(bf, 1024);
		LineNumber++;

		if(!ParseXMLData(bf, xmlobj))
			return false;


		if(MaxObj && xmlobj->objects.size()>unsigned(MaxObj))
			return true;

		if(xmlobj->isclosed) 
			return true;
	}

	return true;
}


xmlobject *xmlparser::CloneTakeBase()
{
	if(!xmlobj) return 0;

	xmlobject *xo=new xmlobject;

	//Clone object
	xo->name=xmlobj->name;
	xo->value=xmlobj->value;
	xo->parent=xmlobj->parent;

	//OK!
	xmlobject *xtmp=xmlobj;	
	xmlobj=xo;

	return xtmp;
}

char * xmlparser::ltrim(char *s)
{
	while(*s && isspace(BYTE(*s)))
		s++;

	return s;
}

const char * xmlparser::ltrim(const char *s)
{
	while(*s && isspace(BYTE(*s)))
		s++;

	return s;
}

const char * xmlparser::GetError()
{
	return ErrorMessage.c_str();

}

const xmlobject * xmlparser::GetObject(const char *pname) const
{
	if(xmlobj){
		if(xmlobj->Is(pname)) return xmlobj;
		else return xmlobj->GetObject(pname);
	}
	else return 0;	
}

void xmlparser::GetOnlyData()
{
	OnlyData=true;
}
